#region Using Directives
#region System
using System;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Data;
using System.Text;
using System.Reflection;
using System.Diagnostics;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
#endregion
#region Assetlink
using Assetlink;
#endregion
#endregion

//--------------------------------------------------------------------------------------------------- //
// File Name: QueryProcessor.cs                                                                       //   
// Author   : Miss. Afsar Khan                                                                        // 
// Class    : QueryProcessor                                                                          //
// Purpose  : Core class Data Access Layer. Base class for all database queryprocessor, contains      //
//             implementation for basic DML and DDL database functions.                               //
//--------------------------------------------------------------------------------------------------- //

namespace Assetlink.DataAccessLayer
{
    public partial class QueryProcessor
    {
        /// <summary>
        /// This function is used to prepare the Update Query based on the InputXML.
        /// The Insert query is prepared according to different datatypes used. 
        /// For fields having DateTime as datatype the date is converted to make it 
        /// compatible with SQL and Oracle by calling the ConvertDate function.
        /// </summary>
        /// <param name="TableName">This function takes TableName as parameter.This 
        /// specifies which table the record has to be inserted</param>
        /// <param name="InputXML">This function takes InputXML as parameter.
        /// The structure of the XML will be like this
        /// <Plan>
        ///		<PlanName>Plan F'or Tes'ting</PlanName>
        ///		<Description>This is for Testing</Description>
        ///		<Age>25</Age>
        ///		<DOB>
        ///		<Day>3</Day>
        ///		<Month>10</Month>
        ///		<Year>1979</Year>
        ///		</DOB>
        /// </Plan>
        /// </param>
        /// <returns>
        /// This function returns OMError object if there is any error occured.
        /// </returns>
        public virtual OMError PrepareUpdateQuery(string TableName, string InputXML, ref string UpdateQuery)
        {
            StringBuilder       Query = null;	//This Query prepared is stored in this variable
            XmlNode             DayNode;						//This contains the Day node which specifies the day in the date in InputXML
            XmlNode             MonthNode;						//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode             YearNode;						//This contains the Year node which specifies the Year in the date in InputXML
            StringBuilder       DateString = null;		//The complete date including Day/Month/Year is saved in this variable
            string              DataType = "";					//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            string              Seperator = "";					//This contains the seperator value like "," which is used for query formation
            string              FieldSize = "";					//This has the field size of the individual field.This is used for truncation of data
            XmlDocument         InputXMLDocument;				//This Contains the Main Document node of the InputXML
            Hashtable           FieldNameValueCollection;		//HashTable variable contains the FieldName,FieldType pair retrieved form the ALOM_Schema
            XmlElement          RootNode;						//Contains the Root node in the InputXML
            OMError             ErrorObj = null;				//In case there is an error in the function this object will be populated
            FieldInformation    FieldDataObj = new FieldInformation();

            //For XML data type support. Code added on date 8th March 2006 by Kiran.
            String              PrimaryKeyFieldName         = null;
            String              ObjectTypeName              = null;
            String              ObjectType                  = null;
            ulong               PrimaryKeyValue             = 0;
            ulong               ObjectTypeID                = 0;
            IXMLRepository      ObjXMLManager               = null;

            //Variables to support Currency As Data Type 
            XmlNode TypeNode;						//This contains the Type node which specifies the Currency Type in the InputXML
            XmlNode ValueNode;						//This contains the Value node which specifies the Value in the InputXML
            StringBuilder CurrencyString;           //This contains the Type and Value of Currency in string format
            try
            {
                ErrorObj = GetObjectTypeSchemaDetails(TableName, out PrimaryKeyFieldName, out ObjectTypeID,
                            out ObjectType, out ObjectTypeName, out FieldNameValueCollection);
                if (ErrorObj != null)
                    return ErrorObj;

                Query           = new StringBuilder();
                ObjXMLManager   = OMDataManager.XMLRepository;
                FieldDataObj    = new FieldInformation();
                CurrencyString  = new StringBuilder();

                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                RootNode = InputXMLDocument.DocumentElement;

                //Get the primary key value. This is for XML data type.
                if ((PrimaryKeyFieldName != null) && (PrimaryKeyFieldName.Length > 0))
                {
                    if (RootNode.SelectSingleNode(PrimaryKeyFieldName) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName).InnerText);
                    else if (RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()).InnerText);
                }

                Query.AppendFormat("UPDATE {0} SET ", TableName.ToUpper());
                foreach (XmlElement ChildNode in RootNode.ChildNodes)
                {
                    if ((FieldNameValueCollection.ContainsKey((ChildNode.Name).Trim()))
                            && (PrimaryKeyFieldName.ToUpper() != ChildNode.Name.Trim().ToUpper()))  
                    {
                        FieldDataObj = (FieldInformation)FieldNameValueCollection[(ChildNode.Name).Trim()];
                        DataType = FieldDataObj.FieldType;

                        if ((DataType.ToUpper() != "XML")
                                && (DataType.ToUpper() != "BINDER")
                                && (DataType.ToUpper() != "FILE"))
                        {
                            FieldSize = FieldDataObj.FieldSize;

                            switch (DataType.ToUpper())
                            {
                                case "STRING":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        Query.AppendFormat("{0}{1} = N'", Seperator, ChildNode.Name);
                                        Query.AppendFormat("{0}' ", TruncateData(ReplaceSingleQuote(ChildNode.InnerText), Convert.ToInt32(FieldSize)));
                                    }
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    if (!String.IsNullOrEmpty(ChildNode.InnerText))
                                    {
                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", Convert.ToString(ChildNode.InnerText));
                                    }
                                    else
                                    {
                                        //Bug Fix : 4435. If value for field is not provided, set it to NULL. 
                                        Query.AppendFormat("{0}{1} = NULL ", Seperator, ChildNode.Name);
                                    }
                                    break;

                                case "BOOLEAN":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        Query.AppendFormat("{0}{1} = N'", Seperator, ChildNode.Name);
                                        Query.AppendFormat("{0}' ", ChildNode.InnerText);
                                    }
                                    break;

                                case "FLOAT":
                                case "DECIMAL":
                                    if (!String.IsNullOrEmpty(ChildNode.InnerText))  
                                    {
                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", Convert.ToString(ChildNode.InnerText));
                                    }
                                    else
                                    {
                                        //Bug Fix : 4435. If value for field is not provided, set it to NULL. 
                                        Query.AppendFormat("{0}{1} = NULL ", Seperator, ChildNode.Name);
                                    }
                                    break;

                                case "DATETIME":
                                    //Bug Fix: 4702 - reinitialize DateString
                                    DateString = new StringBuilder();

                                    DayNode = ChildNode.SelectSingleNode("DAY");
                                    MonthNode = ChildNode.SelectSingleNode("MONTH");
                                    YearNode = ChildNode.SelectSingleNode("YEAR");
                                    DateString.AppendFormat("{0}/{1}/{2}", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                                    Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                    Query.AppendFormat("{0} ", ConvertDate(DateString.ToString()));
                                    break;

                                case "CURRENCY":
                                    TypeNode = ChildNode.SelectSingleNode("CURRENCY");
                                    ValueNode = ChildNode.SelectSingleNode("AMOUNT");
                                    if ((TypeNode != null && ValueNode == null) || (TypeNode == null && ValueNode != null))
                                    {                                        
                                        ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                                                                            "Both fields of currency type data not present.", OMErrorObjectType.TABLE,
                                                                             null, OMErrorType.ERROR, null, null);
                                    }
                                    if (TypeNode != null && ValueNode != null)
                                    {
                                        CurrencyString.AppendFormat("{0}_C =N'{1}',{2}_A={3}", ChildNode.Name, TypeNode.InnerText == "" ? "NULL" : TypeNode.InnerText, ChildNode.Name, ValueNode.InnerText == "" ? "NULL" : ValueNode.InnerText);
                                        Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }                                    
                                    else
                                    {
                                        CurrencyString.AppendFormat("{0}_C =N'{1}',{2}_A={3}", ChildNode.Name, "NULL", ChildNode.Name, "NULL");
                                        Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }

                                    CurrencyString.Remove(0, CurrencyString.Length);
                                    break;
                            }
                            Seperator = ",";
                        }
                        //If column data type is XML then create an XML file and add XML data into the
                        //newly created file by using XML manager. This is for XML data type.
                        else if ((DataType.ToUpper() == "XML") && (ObjectTypeName != null) && (PrimaryKeyValue != 0))
                        {
                            ErrorObj = ObjXMLManager.Save(ObjectTypeName, PrimaryKeyValue, ChildNode.Name.Trim().ToUpper(), ChildNode.InnerXml);
                            if (ErrorObj != null)
                                return ErrorObj;
                        }
                    }
                }

                if ((m_Clause != "") && (m_Clause != null))
                    Query.AppendFormat(" WHERE {0}", m_Clause);

                //Query.AppendFormat(") ");
                if ((DataReaderObj != null) && (DataReaderObj.IsClosed == false))
                    DataReaderObj.Close();
                m_Clause = "";
                m_ClauseForDataTables = "";
                UpdateQuery = Query.ToString();

                //Update The Cache
                RemoveFromCache(TableName);
            }
            catch (Exception ExceptionObj)
            {
                DataReaderObj.Close();
                m_Clause = "";
                m_ClauseForDataTables = "";
                ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            return (ErrorObj);
        }

        /// <summary>
        /// This function is used to prepare the Update Query based on the InputXML.
        /// The query is prepared according to different datatypes used. 
        /// For fields having DateTime as datatype the date is converted to make it 
        /// compatible with SQL and Oracle by calling the ConvertDate function.
        /// </summary>
        /// <param name="TableName">This function takes TableName as parameter.This 
        /// indicates the table whose record is to be updated</param>
        /// <param name="InputXML">This function takes InputXML as parameter.
        /// The structure of the XML will be like this
        /// <Plan>
        ///		<PlanName>Plan F'or Tes'ting</PlanName>
        ///		<Description>This is for Testing</Description>
        ///		<Age>25</Age>
        ///		<DOB>
        ///		<Day>3</Day>
        ///		<Month>10</Month>
        ///		<Year>1979</Year>
        ///		</DOB>
        /// </Plan>
        /// </param>
        /// <param name="ObjectTypeDetailsXML">XML containing Type and Field information.
        /// The structure of the XML will be like this
        /// <TYPE>
	    /// <TYPEID></TYPEID>
	    /// <TYPENAME></TYPENAME>
	    /// <TYPEDESCRIPTION />
	    /// <OBJECTTYPE></OBJECTTYPE>
	    /// <PRIMARYKEY></PRIMARYKEY>
	    /// <FIELDCOLLECTION>
	    ///	<FIELD>1..*
	    ///		<SCHEMAID></SCHEMAID>
	    ///		<OBJECTTYPEID></OBJECTTYPEID>
	    ///		<OBJECTTYPE></OBJECTTYPE>
	    ///		<FIELDNAME></FIELDNAME>
	    ///			<DISPLAYNAME></DISPLAYNAME>
	    ///			<FIELDTYPE></FIELDTYPE>
	    ///			<FIELDSIZE></FIELDSIZE>
	    ///			<FIELDUITYPE></FIELDUITYPE>
	    ///			<ISPRIMARYKEY></ISPRIMARYKEY>
	    ///			<ISMANDATORY></ISMANDATORY>
	    ///			<ISSEARCHFIELD></ISSEARCHFIELD>
	    ///			<ISNAMEDHIERARCHY></ISNAMEDHIERARCHY>
	    ///			<DEFAULTVALUE />
	    ///			<DISPLAYORDER></DISPLAYORDER>
	    ///		</FIELD>
		///	</FIELDCOLLECTION>
        ///	</TYPE>
        /// </param>
        /// <param name="UpdateQueryToBeUsedInDynamicSQL">Boolean value containing True|False. It specifies whether the output
        /// update query is to be used within and stored procedure or not.
        /// </param>
        /// <param name="XMLFieldValueCollection">Collection containing Fields of type XML and their corresponding values. 
        /// It is the responsibility of the caller to update the XML fields</param>
        /// <returns>
        /// This function returns OMError object if there is any error occured.
        /// </returns>
        public virtual OMError PrepareUpdateQuery(String TableName, String InputXML, String ObjectTypeDetailsXML,Boolean UpdateQueryToBeUsedInDynamicSQL, ref NameValueCollection XMLFieldValueCollection, ref string UpdateQuery)
        {
            StringBuilder       Query = null;	                //This Query prepared is stored in this variable
            XmlNode             DayNode;						//This contains the Day node which specifies the day in the date in InputXML
            XmlNode             MonthNode;						//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode             YearNode;						//This contains the Year node which specifies the Year in the date in InputXML
            XmlNode             HourNode                    = null;
            XmlNode             MinuteNode                  = null;
            XmlNode             SecondNode                  = null;
            StringBuilder       DateString = null;		        //The complete date including Day/Month/Year is saved in this variable
            string              DataType = "";					//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            string              Seperator = "";					//This contains the seperator value like "," which is used for query formation
            string              FieldSize = "";					//This has the field size of the individual field.This is used for truncation of data
            XmlDocument         InputXMLDocument;				//This Contains the Main Document node of the InputXML
            Hashtable           FieldNameValueCollection;		//HashTable variable contains the FieldName,FieldType pair retrieved form the ALOM_Schema
            XmlElement          RootNode;						//Contains the Root node in the InputXML
            OMError             ErrorObj = null;				//In case there is an error in the function this object will be populated
            FieldInformation    FieldDataObj = new FieldInformation();

            //For XML data type support. Code added on date 8th March 2006 by Kiran.
            String              PrimaryKeyFieldName         = null;
            ulong               PrimaryKeyValue             = 0;
            IXMLRepository      ObjXMLManager               = null;
            XPathDocument       ObjectTypeDetailsXMLDoc     = null;
            String              SingleQuote                 = null;
            Boolean             NonXMLTypeFieldPresent      = false;

            XmlNode TypeNode;						//This contains the Type node which specifies the Currency Type in the InputXML
            XmlNode ValueNode;						//This contains the Value node which specifies the Value in the InputXML
            StringBuilder CurrencyString;           //This contains the Type and Value of Currency in string format

            try
            {
                ObjectTypeDetailsXMLDoc = new XPathDocument(new StringReader(ObjectTypeDetailsXML));
                ErrorObj                = GetObjectTypeSchemaDetails(ObjectTypeDetailsXMLDoc, out PrimaryKeyFieldName,out FieldNameValueCollection);
                if (ErrorObj != null)
                    return ErrorObj;

                Query           = new StringBuilder();
                ObjXMLManager   = OMDataManager.XMLRepository;
                FieldDataObj    = new FieldInformation();
                CurrencyString = new StringBuilder();
                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                RootNode = InputXMLDocument.DocumentElement;

                //Anand : Code added because in case of DynamicSQL we need extra quote character.
                if (UpdateQueryToBeUsedInDynamicSQL)
                    SingleQuote = "'";

                //Get the primary key value. This is for XML data type.
                if ((PrimaryKeyFieldName != null) && (PrimaryKeyFieldName.Length > 0))
                {
                    if (RootNode.SelectSingleNode(PrimaryKeyFieldName) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName).InnerText);
                    else if (RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()).InnerText);
                }

                Query.AppendFormat("UPDATE {0} SET ", TableName.ToUpper());
                foreach (XmlElement ChildNode in RootNode.ChildNodes)
                {
                    if ((FieldNameValueCollection.ContainsKey((ChildNode.Name).Trim()))
                            && (PrimaryKeyFieldName.ToUpper() != ChildNode.Name.Trim().ToUpper()))  
                    {
                        FieldDataObj = (FieldInformation)FieldNameValueCollection[(ChildNode.Name).Trim()];
                        DataType = FieldDataObj.FieldType;

                        if ((DataType.ToUpper() != "XML")
                                && (DataType.ToUpper() != "BINDER")
                                && (DataType.ToUpper() != "FILE"))
                        {
                            NonXMLTypeFieldPresent  = true;
                            FieldSize               = FieldDataObj.FieldSize;

                            switch (DataType.ToUpper())
                            {
                                case "STRING":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        Query.AppendFormat("{0}{1} = N'{2}", Seperator, ChildNode.Name, SingleQuote);
                                        if (ChildNode.InnerText.Contains("'"))
                                            Query.AppendFormat("{0}{1}' ", TruncateData(ReplaceSingleQuote(ChildNode.InnerText, UpdateQueryToBeUsedInDynamicSQL), Convert.ToInt32(FieldSize)), SingleQuote);
                                        else
                                            Query.AppendFormat("{0}{1}' ", TruncateData(ChildNode.InnerText, Convert.ToInt32(FieldSize)), SingleQuote);
                                    }
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    if (!String.IsNullOrEmpty(ChildNode.InnerText))
                                    {
                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", Convert.ToString(ChildNode.InnerText));
                                    }
                                    else
                                    {
                                        //Bug Fix : 4435. If value for field is not provided, set it to NULL. 
                                        Query.AppendFormat("{0}{1} = NULL ", Seperator, ChildNode.Name);
                                    }
                                    break;

                                case "BOOLEAN":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        Query.AppendFormat("{0}{1} = N'{2}", Seperator, ChildNode.Name, SingleQuote);
                                        Query.AppendFormat("{0}{1}' ", ChildNode.InnerText, SingleQuote);
                                    }
                                    break;

                                case "FLOAT":
                                case "DECIMAL":
                                    if (!String.IsNullOrEmpty(ChildNode.InnerText))  
                                    {
                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", Convert.ToString(ChildNode.InnerText));
                                    }
                                    else
                                    {
                                        //Bug Fix : 4435. If value for field is not provided, set it to NULL. 
                                        Query.AppendFormat("{0}{1} = NULL ", Seperator, ChildNode.Name);
                                    }
                                    break;

                                case "DATETIME":
                                    //Bug Fix: 4702 - reinitialize DateString
                                    DateString = new StringBuilder();
                                    
                                    DayNode     = ChildNode.SelectSingleNode("DAY");
                                    MonthNode   = ChildNode.SelectSingleNode("MONTH");
                                    YearNode    = ChildNode.SelectSingleNode("YEAR");
                                    HourNode    = ChildNode.SelectSingleNode("HOUR");
                                    MinuteNode  = ChildNode.SelectSingleNode("MINUTE");
                                    SecondNode  = ChildNode.SelectSingleNode("SECOND");

                                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                                        DateString.AppendFormat("{0}{1}/{2}/{3} {4}:{5}:{6}{0}",SingleQuote, MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                                    else if ((MonthNode != null) && (DayNode != null) && (YearNode != null))
                                        DateString.AppendFormat("{0}{1}/{2}/{3}{0}",SingleQuote, MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                                    
                                    Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                    
                                    //Development bug - fixed by Roopa Mukim.
                                    //This code is added because if we pass empty node (e.g. <FieldName /> ) for a date field, then query was getting 
                                    //prepared as <FieldName> = CONVERT(DATETIME,'',101) but SQL parser parses '' (double sing;e quote) as ' (single single quote)
                                    //and thus exception was thrown. To avoid this exception we set value of this field as NULL. So for empty date field
                                    //query will be prepared as <FieldName> = NULL
                                    if(String.IsNullOrEmpty(DateString.ToString()))
                                        Query.AppendFormat("{0} ", "null");
                                    else
                                        Query.AppendFormat("{0} ", ConvertDate(DateString.ToString()));
                                    break;
                                //Bug Fix:4473- Currency support is handled.
                                case "CURRENCY":
                                    TypeNode = ChildNode.SelectSingleNode("CURRENCY");
                                    ValueNode = ChildNode.SelectSingleNode("AMOUNT");
                                    if ((TypeNode != null && ValueNode == null) || (TypeNode == null && ValueNode != null))
                                    {
                                        ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                                                                            "Both fields of currency type data not present.", OMErrorObjectType.TABLE,
                                                                             null, OMErrorType.ERROR, null, null);
                                    }
                                    if (TypeNode != null && ValueNode != null)
                                    {
                                        // Two Sigle Quoates are needed for Dynamic SQL. 
                                        CurrencyString.AppendFormat("{0}_C =N''{1}'',{2}_A={3}", ChildNode.Name, TypeNode.InnerText == "" ? "NULL" : TypeNode.InnerText, ChildNode.Name, ValueNode.InnerText == "" ? "NULL" : ValueNode.InnerText);
                                        Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }                                   
                                    else
                                    {
                                        CurrencyString.AppendFormat("{0}_C =N''{1}'',{2}_A={3}", ChildNode.Name, "NULL", ChildNode.Name, "NULL");
                                        Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }

                                    CurrencyString.Remove(0, CurrencyString.Length);
                                    break;
                            }
                            Seperator = ",";
                        }
                        //If column data type is XML then create an XML file and add XML data into the
                        //newly created file by using XML manager. This is for XML data type.
                        else if ((DataType.ToUpper() == "XML"))
                        {
                            XMLFieldValueCollection.Add(ChildNode.Name, ChildNode.InnerXml);
                        }
                    }
                }

                if ((m_Clause != "") && (m_Clause != null))
                    Query.AppendFormat(" WHERE {0}", m_Clause);

                //Query.AppendFormat(") ");
                if ((DataReaderObj != null) && (DataReaderObj.IsClosed == false))
                    DataReaderObj.Close();
                m_Clause = "";
                m_ClauseForDataTables = "";

                if(NonXMLTypeFieldPresent)
                    UpdateQuery = Query.ToString();

                //Update The Cache
                RemoveFromCache(TableName);
            }
            catch (Exception ExceptionObj)
            {
                DataReaderObj.Close();
                m_Clause = "";
                m_ClauseForDataTables = "";
                ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            return (ErrorObj);
        }

        /// <summary>
        /// This function is used to prepare the Insert Query based on the InputXML.
        /// The Insert query is prepared according to different datatypes used. 
        /// For fields having DateTime as datatype the date is converted to make it 
        /// compatible with SQL and Oracle by calling the ConvertDate function.
        /// </summary>
        /// <param name="TableName">This function takes TableName as parameter.This 
        /// specifies which table the record has to be inserted</param>
        /// <param name="InputXML">This function takes InputXML as parameter.
        /// The structure of the XML will be like this
        /// <Plan>
        ///		<PlanName>Plan F'or Tes'ting</PlanName>
        ///		<Description>This is for Testing</Description>
        ///		<Age>25</Age>
        ///		<DOB>
        ///		<Day>3</Day>
        ///		<Month>10</Month>
        ///		<Year>1979</Year>
        ///		</DOB>
        /// </Plan>
        /// </param>
        /// <returns>
        /// This function returns OMError object if there is any error occured.
        /// </returns>
        public virtual OMError PrepareInsertQuery(String TableName, String InputXML, ref String InsertQuery)
        {
            XmlNode             DayNode                     = null;					//This contains the Day node which specifies the day in the date in InputXML
            XmlNode             MonthNode                   = null;					//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode             YearNode                    = null;					//This contains the Year node which specifies the Year in the date in InputXML
            string              DataType                    = null;				    //The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            string              Seperator                   = "";				    //This contains the seperator value like "," which is used for query formation
            string              FieldSize                   = null;				    //This has the field size of the individual field.This is used for truncation of data
            XmlDocument         InputXMLDocument            = null;			        //This Contains the Main Document node of the InputXML
            Hashtable           FieldNameValueCollection    = null;	                //HashTable variable contains the FieldName,FieldType pair retrieved form the ALOM_Schema
            XmlElement          RootNode                    = null;					//Contains the Root node in the InputXML
            StringBuilder       FieldList                   = null;					//The FieldList represents the list of fields.This is used to append in the Insert Query 
            StringBuilder       FieldValues                 = null;				    //The FieldValues represents the list of field vlaues.This is used to append in the Insert Query 
            StringBuilder       DateString                  = null;					//The Value of the DateString is saved in this variable
            OMError             ErrorObj                    = null;			        //In case there is an error in the function this object will be populated
            XmlNode             HourNode                    = null;					//This contains the Hour node which specifies the Hour in the date in InputXML
            XmlNode             MinuteNode                  = null;					//This contains the Minute node which specifies the Minute in the date in InputXML
            XmlNode             SecondNode                  = null;					//This contains the Second node which specifies the Second in the date in InputXML
            StringBuilder       QuerySB                     = null;
            FieldInformation    FieldDataObj;                                       //This is the Field Object stored in the struct, It contains the value of the FieldName and FieldType

            //For XML data type support.  Code added on date 7th March 2006 by Kiran.
            String              PrimaryKeyFieldName         = null;
            String              ObjectTypeName              = null;
            String              ObjectType                  = null;
            ulong               PrimaryKeyValue             = 0;
            ulong               ObjectTypeID                = 0;
            IXMLRepository      ObjXMLManager               = null;

            XmlNode TypeNode;						//This contains the Type node which specifies the Currency Type in the InputXML
            XmlNode ValueNode;						//This contains the Value node which specifies the Value in the InputXML
            StringBuilder CurrencyString;           //This contains the Type and Value of Currency in string format
            DynamicSQLParameter m_parameter_1, m_parameter_2;

            try
            {
                //StringDictionary FieldNameValueCollection = new StringDictionary(); 
                // Needs to change CaseInsensitiveHashCodeProvider to IEqualityComparer
                //In this code section the records from the ALOM_Schema is retrieved based on the TableName and kept in the HashTable. 
                //Later to determine the DataType of the Field this Hashtable is used.

                ErrorObj = GetObjectTypeSchemaDetails(TableName, out PrimaryKeyFieldName, out ObjectTypeID,
                                            out ObjectType, out ObjectTypeName, out FieldNameValueCollection);
                if (ErrorObj != null)
                    return ErrorObj;

                QuerySB         = new StringBuilder();
                FieldList       = new StringBuilder();
                FieldValues     = new StringBuilder();
                DateString      = new StringBuilder();
                CurrencyString  = new StringBuilder();
                ObjXMLManager   = OMDataManager.XMLRepository;
                FieldDataObj    = new FieldInformation();

                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                RootNode = InputXMLDocument.DocumentElement;

                //Get the primary key value. This is for XML data type.
                if ((PrimaryKeyFieldName != null) && (PrimaryKeyFieldName.Length > 0))
                {
                    if (RootNode.SelectSingleNode(PrimaryKeyFieldName) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName).InnerText);
                    else if (RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()).InnerText);
                }

                QuerySB.AppendFormat("INSERT INTO {0} (", TableName.ToUpper());
                foreach (XmlElement ChildNode in RootNode.ChildNodes)
                {
                    if (FieldNameValueCollection.ContainsKey((ChildNode.Name).Trim()))
                    {
                        FieldDataObj = (FieldInformation)FieldNameValueCollection[(ChildNode.Name).Trim()];
                        DataType = FieldDataObj.FieldType;
                        
                        if ((DataType.ToUpper() != "XML") &&
                            (DataType.ToUpper() != "BINDER") &&
                            (DataType.ToUpper() != "FILE"))
                        {
                            FieldList.AppendFormat("{0} {1}", Seperator, (ChildNode.Name).ToUpper());
                            FieldSize = FieldDataObj.FieldSize;

                            switch (DataType.ToUpper())
                            {
                                case "STRING":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter_1 = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TruncateData(ReplaceSingleQuote(ChildNode.InnerText), Convert.ToInt32(FieldSize)) + "'" , "NVARCHAR(MAX)");
                                        if (IsQueryExecutionDeferred)
                                            DynamicParameters_Defer.Add(m_parameter_1);
                                        else
                                            DynamicParameters.Add(m_parameter_1);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter_1.m_ParameterName);
                                    }
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter_1 = new DynamicSQLParameter(GetNextParameterizedVariable(), Convert.ToString(ChildNode.InnerText), "BIGINT");
                                        if (IsQueryExecutionDeferred)
                                            DynamicParameters_Defer.Add(m_parameter_1);
                                        else
                                            DynamicParameters.Add(m_parameter_1);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter_1.m_ParameterName);
                                    }
                                    break;

                                case "BOOLEAN":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter_1 = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + ChildNode.InnerText + "'", "BIT");
                                        if (IsQueryExecutionDeferred)
                                            DynamicParameters_Defer.Add(m_parameter_1);
                                        else
                                            DynamicParameters.Add(m_parameter_1);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter_1.m_ParameterName);
                                    }
                                    break;
                                
                                case "FLOAT":
                                case "DECIMAL":
                                    m_parameter_1 = new DynamicSQLParameter();
                                    m_parameter_1.m_ParameterName = GetNextParameterizedVariable();
                                    m_parameter_1.m_ParameterDataType = "REAL";
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter_1.m_ParameterValue = Convert.ToString(ChildNode.InnerText);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter_1.m_ParameterName);
                                    }
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                    {
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter_1.m_ParameterName);
                                        m_parameter_1.m_ParameterValue = "0.0";

                                    }
                                    break;

                                case "DATETIME":
                                    DayNode     = ChildNode.SelectSingleNode("DAY");
                                    MonthNode   = ChildNode.SelectSingleNode("MONTH");
                                    YearNode    = ChildNode.SelectSingleNode("YEAR");
                                    HourNode    = ChildNode.SelectSingleNode("HOUR");
                                    MinuteNode  = ChildNode.SelectSingleNode("MINUTE");
                                    SecondNode  = ChildNode.SelectSingleNode("SECOND");
                                    
                                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                                        DateString.AppendFormat("{0}/{1}/{2} {3}:{4}:{5}", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                                    else if ((MonthNode != null) && (DayNode != null) && (YearNode != null) )
                                        DateString.AppendFormat("{0}/{1}/{2}", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);

                                    if ((MonthNode != null) && (DayNode != null) && (YearNode != null))
                                    {
                                        if ((MonthNode.InnerText != null && MonthNode.InnerText != "") && (DayNode.InnerText != null && DayNode.InnerText != "") && (YearNode.InnerText != null && YearNode.InnerText != ""))
                                        {
                                            m_parameter_1 = new DynamicSQLParameter();
                                            m_parameter_1.m_ParameterName = GetNextParameterizedVariable();
                                            m_parameter_1.m_ParameterDataType = "DATETIME";
                                            FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter_1.m_ParameterName);
                                            m_parameter_1.m_ParameterValue = ConvertDate(DateString.ToString());
                                            if (IsQueryExecutionDeferred)
                                                DynamicParameters_Defer.Add(m_parameter_1);
                                            else
                                                DynamicParameters.Add(m_parameter_1);
                                        }
                                        else
                                            FieldValues.AppendFormat("{0}{1} ", Seperator, "null");    
                                    }
                                    else
                                        FieldValues.AppendFormat("{0}{1}", Seperator,"null");
                                    break;

                                case "CURRENCY":

                                    TypeNode = ChildNode.SelectSingleNode("CURRENCY");
                                    ValueNode = ChildNode.SelectSingleNode("AMOUNT");
                                    if ((TypeNode != null && ValueNode == null) || (TypeNode == null && ValueNode != null))
                                    {
                                        ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                                                                            "Both fields of currency type data not present.", OMErrorObjectType.TABLE,
                                                                             null, OMErrorType.ERROR, null, null);
                                    }
                                    if (TypeNode != null && ValueNode != null)
                                    {
                                        if (TypeNode.InnerText == "")
                                        {
                                            CurrencyString.AppendFormat("{0}_C =N'{1}',", ChildNode.Name, "NULL");
                                        }
                                        else
                                        {
                                            m_parameter_1 = new DynamicSQLParameter();
                                            m_parameter_1.m_ParameterName = GetNextParameterizedVariable();
                                            m_parameter_1.m_ParameterDataType = "NVARCHAR(25)";
                                            m_parameter_1.m_ParameterValue = "N'" + TypeNode.InnerText + "'";
                                            CurrencyString.AppendFormat("{0}_C = {1} ,", ChildNode.Name, m_parameter_1.m_ParameterName);
                                        }

                                        if (ValueNode.InnerText == "")
                                        {
                                            CurrencyString.AppendFormat("{0}_A={1}", ChildNode.Name, "NULL");
                                        }
                                        else
                                        {
                                            m_parameter_2 = new DynamicSQLParameter();
                                            m_parameter_2.m_ParameterName = GetNextParameterizedVariable();
                                            m_parameter_2.m_ParameterDataType = "NVARCHAR(4)";
                                            m_parameter_2.m_ParameterValue = ValueNode.InnerText;
                                            CurrencyString.AppendFormat("{0}_A= {1} ", ChildNode.Name, m_parameter_2.m_ParameterName);
                                        }
                                        //CurrencyString.AppendFormat("{0}_C =N'{1}',{2}_A={3}", ChildNode.Name, TypeNode.InnerText == "" ? "NULL" : TypeNode.InnerText, ChildNode.Name, ValueNode.InnerText == "" ? "NULL" : ValueNode.InnerText);
                                        FieldValues.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }                                    
                                    else
                                    {
                                        CurrencyString.AppendFormat("{0}_C =N'{1}',{2}_A={3}", ChildNode.Name, "NULL", ChildNode.Name, "NULL");
                                        FieldValues.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }

                                    CurrencyString.Remove(0, CurrencyString.Length);
                                    break;

                            }
                            Seperator = ",";
                        }
                        //If column data type is XML then create an XML file and add XML data into the
                        //newly created file by using XML manager. This is for XML data type.
                        else if ((DataType.ToUpper() == "XML") && (ObjectTypeName != null) && (PrimaryKeyValue != 0))
                            ErrorObj = ObjXMLManager.Save(ObjectTypeName, PrimaryKeyValue, ChildNode.Name.Trim().ToUpper(), ChildNode.InnerXml);
                    }
                }

                QuerySB.AppendFormat("{0}) VALUES ({1})", FieldList, FieldValues);
                if ((DataReaderObj != null) && (DataReaderObj.IsClosed == false))
                    DataReaderObj.Close();
                DataReaderObj = null;
                InsertQuery = QuerySB.ToString();

                RemoveFromCache(TableName);
            }
            catch (Exception ExceptionObj)
            {
                DataReaderObj.Close();
                ErrorObj = new OMError(OMErrorCodes.CREATE_RECORD_FAILED,
                                            ExceptionObj.Message, OMErrorObjectType.TABLE,
                                            null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }

            return (ErrorObj);
        }
    }
}