﻿#region Using Directives
#region System
using System;
using System.IO;
using System.Xml;
using System.Xml.XPath; 
using System.Data;
using System.Text;
using System.Reflection;
using System.Diagnostics;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
#endregion
#region Assetlink
using Assetlink;
#endregion
#endregion

using System.Data.DataSetExtensions;


//--------------------------------------------------------------------------------------------------- //
// File Name: QueryProcessor.cs                                                                       //   
// Author   : Miss. Afsar Khan                                                                        // 
// Class    : QueryProcessor                                                                          //
// Purpose  : Core class Data Access Layer. Base class for all database queryprocessor, contains      //
//             implementation for basic DML and DDL database functions.                               //
//--------------------------------------------------------------------------------------------------- //


namespace Assetlink.DataAccessLayer
{

    public partial class DynamicSQLParameter
    {
        public string m_ParameterName;
        public string m_ParameterValue;
        public string m_ParameterDataType;

        public DynamicSQLParameter(String sParametename, String sParameterValue, String sParameterDataType)
        {
            m_ParameterName = sParametename;
            m_ParameterValue = sParameterValue;
            m_ParameterDataType = sParameterDataType;
        }

        public DynamicSQLParameter()
        {
        }
    }

	/// <summary>
	/// <CreatedDate> 18-04-2005</CreatedDate>
	/// <Author>Miss Afsar Khan</Author>
	/// <Purpose>This structure contains the FieldType and FieldSize.
	/// In the CreateRecord and other APIs we require to Datatype and size of each Field.
	/// This information is retrieved from the ALOM_Schema table and stored in the has table 
	/// instead of querying the table again and again for each field.
	/// The hash table stores an object. Hence this structure is used which is populated and 
	/// stored as a hash table entry.
	/// </Purpose>
	/// </summary>
	public struct FieldInformation
	{
		public string FieldType;
		public string FieldSize;
	}

	/// <summary>
	/// <CreatedDate> 18-04-2005</CreatedDate>
	/// <
	/// Author>Miss Afsar Khan</Author>
	/// <Purpose>This class provides all the basic functionality for Creating,updating and deleting tables.
	/// It also provides the basic functionality for Creating,Reading,Updating and Deleting the records from the specified table.
	/// The functionality provided will be common to all types of Databases. If the functionality needs to be overriden for a 
	/// specific Database that can also be done since all the functions are made virtual.
	/// </Purpose>
	/// </summary>
	public partial class QueryProcessor
	{
		#region Member Declaration

        public const string TAG_AL_FILESYSTEM = "AL$FILESYSTEM_AL$";
		public string           m_Clause        = null;
        public string           m_ClauseForCache = null;    //Uesd for Teradata cache  
        public string           m_ClauseForDataTables   = null; //This variable will hold non-parameterized where clause for cached tables
        public string           m_SortClause = null;
        public string           m_SortClauseForCache = null;    //Uesd for Teradata cache
        public IDbConnection	ConnectionObj   = null;	//Represents an open connection to a data source either SQL or Oracle depending on Database Type
		public IDbCommand		CommandObj      = null;	//Represents a Database statement that is executed while connected to a data source	
		public IDataReader		DataReaderObj   = null;	//Provides a mechanism to fetch result sets obtained by executing a command at a data source
		public IDbTransaction	TransactionObj  = null; //Represents a transaction to be performed at a data source
        public StringDictionary ParamDictionary = null; //Dictionary to store name-value pair for Stored procedure. 
		public String           TenantName      = null;
        protected  bool         m_DeferQueryExecution   = false;    //Specifies whether query execution is to be deferred
        //protected StringBuilder sb_DeferredQueries      = null;     //Maintains deferred queries
        public DeferredQueryData obj_DeferredQueryData  = null;     //Used to maintain records/data for deferred queries
        public String           FTSLogicalOperator      = null;
        private bool m_IsOperationDeleteInstance = false;   //Specifies whether operation is delete instance (entity/facet)
        protected List<String> sl_DeferredQueries = new List<string>();     //Maintains list of deferred queries
        /// <summary>
        /// Maintains Ids of the entity/facet instances which got deleted as disassociate or delete operation
        /// </summary>
        public Dictionary<ulong, String> dict_DeletedInstaceIds = null; 

        /// <summary>
        /// Specifies whether access control check is to be performed while updating/deleting entity instance. 
        /// This should be set to false only if the operation is CreateData or UpdateData.
        /// </summary>
        public Boolean       IsAccessControlCheckRequired = true;    
        private String       m_DatabaseType               = null;    //Specifies the database type for tenant.
        private DataManager  m_DataManager                = null;
        #endregion Member Declaration

        #region Properties
        /// <summary>
        /// Specifies whether paging is supported.
        /// </summary>
        public virtual bool IsPagingSupported
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Flag specifying query execution to be deferred
        /// </summary>
        public bool IsQueryExecutionDeferred
        {
            get
            {
                return m_DeferQueryExecution;
            }
            set
            {
                if (value)
                {
                    if (sl_DeferredQueries == null)
                        sl_DeferredQueries = new List<string>();

                    if (obj_DeferredQueryData == null)
                        obj_DeferredQueryData = new DeferredQueryData();
                }

                m_DeferQueryExecution = value;
            }
        }

        /// <summary>
        /// Specifies whether operation is delete entity/facet instance. Delete methods for entity and facet should set this to true.
        /// </summary>
        public bool IsOperationDeleteInstance
        {
            get { return m_IsOperationDeleteInstance; }
            set 
            { 
                m_IsOperationDeleteInstance = value;
                if (m_IsOperationDeleteInstance && dict_DeletedInstaceIds == null)
                    dict_DeletedInstaceIds = new Dictionary<ulong, string>();
            }
        }

        /// <summary>
        /// <Author>Hemant Kirad</Author>
        /// <CreatedDate>21 December 2010</CreatedDate>
        /// <Purpose>
        /// The value for this property will be set from the constructor of DataManager class.
        /// This will hold the DatabaseType for the tenant. Values for this could be: SQLSERVER or SQLEXPRESS or ORACLE or TERADATA.
        /// This was added because Full Text Search is not supported by SQLExpress and hence we need to know the database type. 
        /// Value of this property will be used in SQLQueryProcessor to decide whether to include the query for Search in files or not.
        /// </Purpose>
        /// </summary>
        public String DatabaseType
        {
            get { return m_DatabaseType; }
            set { m_DatabaseType = value; }
        }

        public DataManager OMDataManager
        {
            get { return m_DataManager; }
            set { m_DataManager = value; }
        }

        #endregion Properties
        
        public List<DynamicSQLParameter> DynamicParameters = new List<DynamicSQLParameter>();
        public List<DynamicSQLParameter> DynamicParameters_Defer = new List<DynamicSQLParameter>();
        Int64 int_ParameterizedVariableId = 0;

		#region Methods

        /// <summary>
        /// Executes deferred queries in case when query execution is deferred.
        /// </summary>
        /// <returns></returns>
        public virtual OMError ExecuteDeferredQueries()
        {
            OMError         obj_OMError             = null; //Used to return OM error
            OMError         obj_OMErrorForCleanUp   = null; //Used to return OM error related with cleanup operation
            List<DynamicSQLParameter> lDynamicSQLParameters = new List<DynamicSQLParameter>();
            int iVariableCounter = 0;
            String sVariableName = "@Var";
            String sNewBatch = string.Empty;

            if (m_DeferQueryExecution && sl_DeferredQueries != null)
            {
                //Set defer query execution flag to false
                m_DeferQueryExecution = false;

                //If deferred query is not formed, do not call execute method 
                if (sl_DeferredQueries.Count > 0)
                {
                    foreach (String strSQLStatment in sl_DeferredQueries)
                    {
                        iVariableCounter = 0;
                        StringBuilder strSQLBatch = new StringBuilder();
                        StringBuilder strParameterDeclaration = new StringBuilder();
                        StringBuilder strParameterValues = new StringBuilder();
                        StringBuilder strParameterList = new StringBuilder();

                        strSQLBatch.Append("DECLARE @SQLString nvarchar(MAX); DECLARE @ParmDefinition nvarchar(MAX);");
                        strSQLBatch.AppendFormat(System.Environment.NewLine);
                        sNewBatch = strSQLStatment;
                        foreach (DynamicSQLParameter oParameter in DynamicParameters_Defer)
                        {
                            if (strSQLStatment.Contains(" " + oParameter.m_ParameterName + " "))
                            {
                                sVariableName = "@Var";
                                sVariableName = sVariableName + iVariableCounter.ToString();
                                strSQLBatch.AppendFormat("DECLARE {0} {1};", sVariableName, oParameter.m_ParameterDataType);
                                strSQLBatch.AppendFormat(System.Environment.NewLine);

                                if (string.IsNullOrEmpty(strParameterDeclaration.ToString()))
                                {
                                    strParameterDeclaration.AppendFormat("{0} {1}", sVariableName, oParameter.m_ParameterDataType);
                                }
                                else
                                {
                                    strParameterDeclaration.AppendFormat(" ,{0} {1}", sVariableName, oParameter.m_ParameterDataType);
                                }

                                strParameterValues.AppendFormat("SET {0} = {1};", sVariableName, oParameter.m_ParameterValue);
                                strParameterValues.AppendFormat(System.Environment.NewLine);

                                if (string.IsNullOrEmpty(strParameterList.ToString()))
                                {
                                    strParameterList.AppendFormat("{0}", sVariableName);
                                }
                                else
                                {
                                    strParameterList.AppendFormat(",{0}", sVariableName);
                                }
                                lDynamicSQLParameters.Add(oParameter);
                                iVariableCounter++;
                                if (string.IsNullOrEmpty(sNewBatch))
                                    sNewBatch = strSQLStatment.Replace(" " + oParameter.m_ParameterName + " ", " " + sVariableName + " ");
                                else
                                    sNewBatch = sNewBatch.Replace(" " + oParameter.m_ParameterName + " ", " " + sVariableName + " ");
                            }
                        }

                        foreach (DynamicSQLParameter oParameter in lDynamicSQLParameters)
                        {
                            DynamicParameters_Defer.Remove(oParameter);
                        }
                        lDynamicSQLParameters.Clear();

                        strSQLBatch.AppendFormat("SET @SQLString = N'{0}';", sNewBatch.ToString().Replace("'", "''"));
                        strSQLBatch.AppendFormat(System.Environment.NewLine);
                        strSQLBatch.AppendFormat("SET @ParmDefinition = N'");
                        strSQLBatch.AppendFormat(strParameterDeclaration.ToString());
                        strSQLBatch.AppendFormat("'");
                        strSQLBatch.AppendFormat(System.Environment.NewLine);

                        strSQLBatch.Append(strParameterValues);

                        if (strParameterList.Length > 0)
                            strSQLBatch.AppendFormat("EXECUTE sp_executesql @SQLString, @ParmDefinition, {0}", strParameterList);
                        else
                            strSQLBatch.AppendFormat("EXECUTE sp_executesql @SQLString, @ParmDefinition");

                        //Execute deferred queries
                        obj_OMError = ExecuteEx(strSQLBatch.ToString());
                    }
                    //sb_DeferredQueries.Remove(0, sb_DeferredQueries.Length);
                    sl_DeferredQueries.Clear();
                }
                else
                {
                    //Prepare warning, since deferred query is not formed
                    obj_OMError = new OMError(OMErrorCodes.DQ_NOQUERYTOEXECUTE, "No query to execute for deferred query execution.",
                                            OMErrorObjectType.DEFERREDQUERY, "DQ_DEFERREDQUERY", OMErrorType.WARNING, null, null);
                }

                //Perform cleanup
                obj_OMErrorForCleanUp = obj_DeferredQueryData.PerformCleanup();

                /* If query execution is successful, then set obj_OMError to obj_OMErrorForCleanUp to return error occured in cleanup operation
                 * otherwise the error occured while executing operation will be returned.*/
                if (obj_OMError == null)
                    obj_OMError = obj_OMErrorForCleanUp;
            }
            else
            {
                obj_OMError = new OMError(OMErrorCodes.DQ_QUERYEXECUTION_FAILED, "ExecuteDeferredQueries: Deferred query execution failed",
                                            OMErrorObjectType.DEFERREDQUERY, "DQ_DEFERREDQUERY", OMErrorType.ERROR, null, null);
            }

            return obj_OMError;
        }

        public String DisposeDeferredQueryDataTablesIfAny()
        {
            String              str_ErrorXml                = null;
            OMError             obj_OMError                 = null;

            //In case of deferred query execution, dispose data tables created by DeferredQueryData class. 
            if (obj_DeferredQueryData != null)
            {
                obj_OMError = obj_DeferredQueryData.DisposeDataTables();

                if (obj_OMError != null)
                    str_ErrorXml = obj_OMError.Serialize();
            }

            return str_ErrorXml;
        }

        /// <summary>
        /// Adds instance id (entity/facet id) to dictionary (of deleted ids).
        /// </summary>
        /// <param name="ul_InstanceId">Instance id to be added to dictionary.</param>
        /// <param name="str_Type">Type i.e. ENTITY/FACET.</param>
        /// <returns>true if the instance gets added to dictionary successfully, otherwise false</returns>
        public bool AddInstanceIdToDeletedInstanceDictionary(ulong ul_InstanceId, String str_Type)
        {
            //Check whether the operation is instance delete
            if (m_IsOperationDeleteInstance)
            {
                if (!String.IsNullOrEmpty(str_Type))
                {
                    if (dict_DeletedInstaceIds != null)
                    {
                        if (dict_DeletedInstaceIds.ContainsKey(ul_InstanceId))
                            return false;

                        //Add instance id to dictionary if it is not present
                        dict_DeletedInstaceIds.Add(ul_InstanceId, str_Type.ToUpper());
                        return true;
                    }
                    return false;
                }
            }
            return false;
        }

        /// <summary>
        /// Checks whether the instance is already deleted by looking up dictionary (of instance id's which got deleted).
        /// </summary>
        /// <param name="ul_InstanceId">Instance id to be identify whether instance is deleted.</param>
        /// <param name="str_Type">Type i.e. ENTITY/FACET</param>
        /// <returns>true if the given instance id is present in dictionary object otherwise false.</returns>
        public bool IsInstanceAlreadyDeleted(ulong ul_InstanceId, String str_Type)
        {
            String      str_TypeForKey      = null;

            //Check whether operation is instance delete
            if (m_IsOperationDeleteInstance)
            {
                if (dict_DeletedInstaceIds != null)
                {
                    //Check whether instance id is present in dictionary and return true/false accordingly
                    if (dict_DeletedInstaceIds.TryGetValue(ul_InstanceId, out str_TypeForKey))
                    {
                        if (String.IsNullOrEmpty(str_TypeForKey) || String.IsNullOrEmpty(str_Type))
                            return false;

                        if (str_TypeForKey.ToUpper().CompareTo(str_Type.ToUpper()) == 0)
                            return true;
                    }
                }
            }
            return false;
        }


        /// <summary>
		/// This function is used for creation of Table. The syntax of the Create statement
		/// differs according to the database used.Hence this function is made virtual. The actual 
		/// implementation is done in the appropriate classes i.e SQLQueryProcessor, 
		/// OralceQueryProcessor etc.
		/// </summary>
		/// <param name="InputXML">The input XML contains the Details of the Table to be created
		/// </param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		virtual public OMError CreateTable(string InputXML) 
		{
			return null;
		}

        virtual public OMError PrepareCreateTableQuery(String InputXML, ref String CreateTableQuery)
        {
            return null;
        }

        public virtual OMError ExecuteDynamicSQLForScalar(String DynamicSQL, String ConnectionString, ref object ScalarResult)
        {
            return null;
        }

        public virtual string GetUnicodeCharacterSpecifier()
        {
            return null;
        }

		/// <summary>
		/// This function is used for updating the table structure. The syntax of the Alter statement
		/// differs according to the database used.Hence this function is made virtual. The actual 
		/// implementation is done in the appropriate classes i.e SQLQueryProcessor, 
		/// OralceQueryProcessor etc.
		/// </summary>
		/// <param name="InputXML">The input XML contains the Details of the Table to be updated
		/// </param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		virtual public OMError UpdateTable(string InputXML)
		{
			return null;
		}
        /// <summary>
        /// Initialaizes the Cache
        /// </summary>
        public virtual void InitializeCache(String ConnectionString,String[] TableList)
        {
            String query = null;
            String cacheItemName = null;
            DataTable cacheTable = null;

            try
            {
                if (TableList == null)
                    return;

                if (ConnectionObj.State != ConnectionState.Open)
                {
                    InitializeDatabaseSetup(ConnectionString);
                }
                foreach (string tableName in TableList)
                {
                    query = "SELECT TOP 1 * FROM " + tableName;
                    CommandObj.CommandText = query;
                    try
                    {
                        DataReaderObj = CommandObj.ExecuteReader();
                        
                        //Bug fix 5451 : If table does not contain rows, do not add it to cache. Cache should always contain records.
                        if (DataReaderObj.Read())
                        {
                            if (!DataReaderObj.IsClosed)
                                DataReaderObj.Close();
                            
                            query = "SELECT * FROM " + tableName;
                            CommandObj.CommandText = query;
                            try
                            {
                                DataReaderObj = CommandObj.ExecuteReader();

                                cacheTable = new DataTable();
                                cacheTable.Load(DataReaderObj);

                                cacheItemName = TenantName.ToUpper() + "_" + tableName.ToUpper();
                                CacheManager.AddCacheItem(cacheItemName.ToUpper(), cacheTable);
                            }
                            catch
                            {
                                //Ignore any errors that happen while loading a table into the cache
                            }
                            finally
                            {
                                if (DataReaderObj != null && !DataReaderObj.IsClosed)
                                    DataReaderObj.Close();
                            }
                        }
                    }
                    catch
                    {
                        //Ignore any errors that happen while loading a table into the cache
                    }
                    finally
                    {
                        if (DataReaderObj != null && !DataReaderObj.IsClosed)
                            DataReaderObj.Close();
                    }
                }
            }
            catch
            {
                
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ConnectionString"></param>
        /// <param name="TransactionFlag"></param>
        public virtual void InitializeDatabaseSetup(string ConnectionString, bool TransactionFlag)
        {
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="ConnectionString"></param>
        public virtual void InitializeDatabaseSetup(string ConnectionString)
        {

        }
         /// <summary>
        /// This function will replace 1st occurance of a string by another string.
        /// </summary>
        /// <param name="OldValue">String to be replaced</param>
        /// <param name="NewValue">New String valueto be used for replacing the old value</param>
        /// <param name="TableNameFieldNameCollectionXML">Contains a list of tables and its fields that should be updated</param>
        public virtual OMError Replace(String OldValue, String NewValue, String TableNameFieldNameCollectionXML)
        {
            return null;
        }

        /// <summary>
        /// This function will return an FieldName ensuring that it will not be an reserved name.
        /// </summary>
        /// <param name="FieldName">Name of the Entity/Facet field.</param>
        public virtual String ChangeFieldNameIncaseItIsReserved(String FieldName)
        {
            return null;
        }

		/// <summary>
		/// <purpose>This function is used to drop a table</purpose>
		/// </summary>
		/// <param name="TableName">This function takes TableName as parameter,
		/// this specifies which table has to be dropped.</param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		virtual public OMError DropTable(string TableName)
		{
			OMError         ErrorObj    = null;
            StringBuilder   Query       = null;
			
            try
			{
                TableName = TableName.ToUpper();
                //Delete all XSD (files) and XML files for an object type.
                //Method added on date 9th March 2006 by Kiran Abhang.
                ErrorObj  = DeleteAllXSDsAndXMLsForObjectType(TableName);
                if (ErrorObj != null)
                    return ErrorObj;


				Query = new StringBuilder();
				Query.AppendFormat("DROP Table {0}", TableName);

				ErrorObj = Execute(Query.ToString());

                //UpdateThe Cache
                if (ErrorObj == null)
                {
                    RemoveFromCache(TableName.ToUpper());
                }
				return (ErrorObj);
			}
			catch (Exception ExceptionObj)
			{
                throw ExceptionObj;
			}
		}

		/// <summary>
		/// This function is used to prepare the Insert Query based on the InputXML.
		/// The Insert query is prepared according to different datatypes used. 
		/// For fields having DateTime as datatype the date is converted to make it 
		/// compatible with SQL and Oracle by calling the ConvertDate function.
		/// </summary>
		/// <param name="TableName">This function takes TableName as parameter.This 
		/// specifies which table the record has to be inserted</param>
		/// <param name="InputXML">This function takes InputXML as parameter.
		/// The structure of the XML will be like this
		/// <Plan>
		///		<PlanName>Plan F'or Tes'ting</PlanName>
		///		<Description>This is for Testing</Description>
		///		<Age>25</Age>
		///		<DOB>
		///		<Day>3</Day>
		///		<Month>10</Month>
		///		<Year>1979</Year>
		///		</DOB>
		/// </Plan>
		/// </param>
		/// <returns>
		/// This function returns OMError object if there is any error occured.
		/// </returns>
		virtual public OMError CreateRecord(string TableName, string InputXML)
		{
            XmlNode             DayNode;					//This contains the Day node which specifies the day in the date in InputXML
            XmlNode             MonthNode;					//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode             YearNode;					//This contains the Year node which specifies the Year in the date in InputXML
            string              DataType = "";				//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            string              Seperator = "";				//This contains the seperator value like "," which is used for query formation
            string              FieldSize = "";				//This has the field size of the individual field.This is used for truncation of data
            XmlDocument         InputXMLDocument;			//This Contains the Main Document node of the InputXML
            Hashtable           FieldNameValueCollection;	//HashTable variable contains the FieldName,FieldType pair retrieved form the ALOM_Schema
            XmlElement          RootNode;					//Contains the Root node in the InputXML
            StringBuilder       FieldList;					//The FieldList represents the list of fields.This is used to append in the Insert Query 
            StringBuilder       FieldValues;				//The FieldValues represents the list of field vlaues.This is used to append in the Insert Query 
            StringBuilder       DateString;						//The Value of the Date is saved in this variable
            OMError             ErrorObj = null;			//In case there is an error in the function this object will be populated
            XmlNode             HourNode;					//This contains the Hour node which specifies the Hour in the date in InputXML
            XmlNode             MinuteNode;					//This contains the Minute node which specifies the Minute in the date in InputXML
            XmlNode             SecondNode;					//This contains the Second node which specifies the Second in the date in InputXML
            StringBuilder       QuerySB = null;
            FieldInformation    FieldDataObj = new FieldInformation(); //This is the Field Object stored in the struct, It contains the value of the FieldName and FieldType

            //For XML data type support. Code added on date 7th March 2006 by Kiran.
            String              PrimaryKeyFieldName         = null;
            String              ObjectTypeName              = null;
            String              ObjectType                  = null;
            ulong               PrimaryKeyValue             = 0;
            ulong               ObjectTypeID                = 0;
            IXMLRepository      ObjXMLManager               = null;
            DynamicSQLParameter m_parameter = null;


            //Variables to support Currency As Data Type
            XmlNode TypeNode;						//This contains the Type node which specifies the Currency Type in the InputXML
            XmlNode ValueNode;						//This contains the Value node which specifies the Value in the InputXML

            try
            {
                //StringDictionary FieldNameValueCollection = new StringDictionary(); 
                // Needs to change CaseInsensitiveHashCodeProvider to IEqualityComparer
                //In this code section the records from the ALOM_Schema is retrieved based on the TableName and kept in the HashTable. 
                //Later to determine the DataType of the Field this Hashtable is used.

                ErrorObj = GetObjectTypeSchemaDetails(TableName, out PrimaryKeyFieldName, out ObjectTypeID,
                            out ObjectType, out ObjectTypeName, out FieldNameValueCollection);

                if (ErrorObj != null)
                    return ErrorObj;

                QuerySB = new StringBuilder();
                FieldList = new StringBuilder();
                FieldValues = new StringBuilder();
                DateString = new StringBuilder();
                ObjXMLManager = OMDataManager.XMLRepository;
                FieldDataObj = new FieldInformation();

                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                RootNode = InputXMLDocument.DocumentElement;

                //Get the primary key value. This is for XML data type.
                if ((PrimaryKeyFieldName != null) && (PrimaryKeyFieldName.Length > 0))
                {
                    if (RootNode.SelectSingleNode(PrimaryKeyFieldName) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName).InnerText);
                    else if (RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()).InnerText);
                }

                QuerySB.AppendFormat("INSERT INTO {0} (", TableName.ToUpper());
                foreach (XmlElement ChildNode in RootNode.ChildNodes)
                {
                    if (FieldNameValueCollection.ContainsKey((ChildNode.Name).Trim()))
                    {
                        FieldDataObj = (FieldInformation)FieldNameValueCollection[(ChildNode.Name).Trim()];
                        DataType = FieldDataObj.FieldType;

                        if ((DataType.ToUpper() != "XML") &&
                            (DataType.ToUpper() != "BINDER") &&
                            (DataType.ToUpper() != "FILE"))
                        {
                            if (DataType.ToUpper() != "CURRENCY")
                                FieldList.AppendFormat("{0} {1}", Seperator, (ChildNode.Name).ToUpper());
                            FieldSize = FieldDataObj.FieldSize;

                            switch (DataType.ToUpper())
                            {
                                case "STRING":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + ReplaceSingleQuote(TruncateData(ChildNode.InnerText, Convert.ToInt32(FieldSize))) + "'", "NVARCHAR(" + FieldSize + ")");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter.m_ParameterName);
                                    }
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}'' ", Seperator);
                                    break;

                                case "XMLTEXT":
                                    if (!(ChildNode.InnerXml == null) && !(ChildNode.InnerXml == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TruncateData(ReplaceSingleQuote(ChildNode.InnerXml), Convert.ToInt32(FieldSize)) + "'", "NVARCHAR(" + FieldSize + ")"); 
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter.m_ParameterName);
                                    }
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), Convert.ToString(ChildNode.InnerText), "BIGINT");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter.m_ParameterName);
                                    }
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}NULL ", Seperator);
                                    break;

                                case "BOOLEAN":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + ChildNode.InnerText + "'", "NVARCHAR(" + FieldSize + ")");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter.m_ParameterName);
                                    }
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}NULL ", Seperator);
                                    break;

                                case "FLOAT":
                                case "DECIMAL":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), Convert.ToString(ChildNode.InnerText), "REAL");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter.m_ParameterName);
                                    }
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}NULL ", Seperator);
                                    break;

                                case "DATETIME":
                                    DayNode = ChildNode.SelectSingleNode("DAY");
                                    MonthNode = ChildNode.SelectSingleNode("MONTH");
                                    YearNode = ChildNode.SelectSingleNode("YEAR");
                                    HourNode = ChildNode.SelectSingleNode("HOUR");
                                    MinuteNode = ChildNode.SelectSingleNode("MINUTE");
                                    SecondNode = ChildNode.SelectSingleNode("SECOND");

                                    if (MonthNode != null && DayNode != null && YearNode != null)
                                    {
                                        if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                                            DateString.AppendFormat("{0}/{1}/{2} {3}:{4}:{5}",
                                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText,
                                                        HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                                        else
                                            DateString.AppendFormat("{0}/{1}/{2}",
                                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);

                                        if ((MonthNode.InnerText != null && MonthNode.InnerText != "") && (DayNode.InnerText != null && DayNode.InnerText != "") && (YearNode.InnerText != null && YearNode.InnerText != ""))
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), ConvertDate(DateString.ToString()), "DATETIME");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);
                                            FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter.m_ParameterName);
                                        }
                                        else
                                            FieldValues.AppendFormat("{0}{1} ", Seperator, "null");
                                        DateString.Remove(0, DateString.Length);
                                    }
                                    else
                                        FieldValues.AppendFormat("{0}{1}", Seperator, "null");

                                    break;

                                //For CURRENCY data type support. Code added on date 14th June 2006 by Pallavi
                                case "CURRENCY":
                                    TypeNode = ChildNode.SelectSingleNode("CURRENCY");
                                    ValueNode = ChildNode.SelectSingleNode("AMOUNT");
                                    FieldList.AppendFormat("{0} {1}_C , {2}_A", Seperator, (ChildNode.Name).ToUpper(), (ChildNode.Name).ToUpper());

                                    if (TypeNode != null && ValueNode != null)
                                    {
                                        if (TypeNode.InnerText == "")
                                            FieldValues.AppendFormat("{0}N'{1}',", Seperator, "NULL");
                                        else
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TypeNode.InnerText + "'", "NVARCHAR(25)");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);
                                            FieldValues.AppendFormat("{0} {1} ,", Seperator, m_parameter.m_ParameterName);
                                        }
                                        if (ValueNode.InnerText == "")
                                        {
                                            FieldValues.AppendFormat("{0}", Seperator, "NULL");
                                        }
                                        else
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), ValueNode.InnerText, "MONEY");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);

                                            FieldValues.AppendFormat(" {0} ", m_parameter.m_ParameterName);
                                        }
                                    }
                                    else if (TypeNode != null && ValueNode == null)
                                    {
                                        if (TypeNode.InnerText == "")
                                            FieldValues.AppendFormat("{0}N'{1}',{2}", Seperator, "NULL", "NULL");
                                        else
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TypeNode.InnerText + "'", "NVARCHAR(25)");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);

                                            FieldValues.AppendFormat("{0} {1} ,{2}", Seperator, m_parameter.m_ParameterName, "NULL");
                                        }
                                    }
                                    else
                                    {
                                        FieldValues.AppendFormat("{0}N'{1}',{2}", Seperator, "NULL", "NULL");
                                    }
                                    //CurrencyString.AppendFormat("{0}_T =N'{1}'{2}{3}_V={4}", ChildNode.Name, TypeNode.InnerText, Seperator, ChildNode.Name, ValueNode.InnerText);
                                    //FieldList.AppendFormat("{0} {1}_C , {2}_A", Seperator, (ChildNode.Name).ToUpper(),  (ChildNode.Name).ToUpper());
                                    //FieldValues.AppendFormat("{0}N'{1}',{2}", Seperator, TypeNode.InnerText, ValueNode.InnerText);
                                    //Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    //CurrencyString.Remove(0, CurrencyString.Length);
                                    break;
                                case "TEXT":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + ReplaceSingleQuote(ChildNode.InnerText) + "'", "NVARCHAR(MAX)");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);
                                        FieldValues.AppendFormat("{0} {1} ", Seperator, m_parameter.m_ParameterName);
                                    }
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}'' ", Seperator);
                                    break;

                            }
                            Seperator = ",";
                        }
                        //If column data type is XML then create an XML file and add XML data into the
                        //newly created file by using XML manager. This is for XML data type.
                        else if ((DataType.ToUpper() == "XML") && (ObjectTypeName != null) && (PrimaryKeyValue != 0))
                            ErrorObj = ObjXMLManager.Save(ObjectTypeName, PrimaryKeyValue, ChildNode.Name.Trim().ToUpper(), ChildNode.InnerXml);
                    }
                }

                QuerySB.AppendFormat("{0}) VALUES ({1})", FieldList, FieldValues);

                //Store deferred queries if query execution is to be deferred
                if (m_DeferQueryExecution)
                {
                    sl_DeferredQueries.Add(QuerySB.ToString());
                }
                else
                {
                    //The Execute method takes the Query as parameter and the query is executed.
                    ErrorObj = Execute(QuerySB.ToString());
                }

                //UpdateThe Cache
                if (ErrorObj == null)
                {
                    RemoveFromCache(TableName.ToUpper());
                }
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.CREATE_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            finally
            {
                //This should not be here. Any error should be caught and thrown at the right place. Before the error is thrown,
                //the possibly open data reader should be closed
                if (DataReaderObj != null && !DataReaderObj.IsClosed)
                    DataReaderObj.Close();
            }

            return (ErrorObj);
		}
        
        /// <summary>
        /// This function is an extension of the CreateRecord method. 
        /// This does a insert based on XML.
        /// </summary>
        /// <param name="EntityTypeName">Name of the Type i.e. EntityType or FacetType</param>
        /// <param name="TableName">This function takes TableName as parameter.This 
        /// specifies which table the record has to be inserted</param>
        /// <param name="InputXML">This function takes InputXML as parameter.
        /// The structure of the XML will be like this
        /// <Plan>
        ///		<PlanName>Plan F'or Tes'ting</PlanName>
        ///		<Description>This is for Testing</Description>
        ///		<Age>25</Age>
        ///		<DOB>
        ///		<Day>3</Day>
        ///		<Month>10</Month>
        ///		<Year>1979</Year>
        ///		</DOB>
        /// </Plan>
        /// </param>
        /// <returns>
        /// This function returns OMError object if there is any error occured.
        /// </returns>
        virtual public OMError CreateRecord(String TypeName,String TableName, String InputXML)
        {
            return null;
        }
        
		/// <summary>
		/// This function is used to select records based on the criteria.
		/// This function takes TypeName as parameter for e.g. Plan,Item etc.
		/// This function also takes FieldListXML as parameter which will indicate the 
		/// Fields to retrieve in the select statement.If the parameter is blank then all 
		/// the fields will be fetched.
		/// This function also takes the OutputXML as parameter. If the number of records 
		/// is greater than 1 then the FieldCollection is returned else only detail of one 
		/// record is given.
		/// </summary>
		/// <param name="TableName">This function takes TableName as parameter.This 
		/// specifies from which table to fetch the records.</param>
		/// <param name="FieldListXML">This function takes FieldListXML as parameter.
		/// For the structure of the XML refer to the FieldCollection.XSD
		/// </param>
		/// <returns>
		/// This function returns OMError object if there is any error occured.
		/// </returns>
		virtual public OMError ReadRecord(string TypeName, string TableName, string FieldListXML, ref string OutputXML)
		{
			string			FieldName="";				//This has the field name of the individual field.This field name is used for formation of OutputXML to return the data back
			string			FieldValue="";				//This has the field Value of the individual field.This field value is used for formation of OutputXML to return the data back
			string			Seperator= "";				//This contains the seperator value like "," which is used for query formation
			string			SortType="";				//This decides whether the data will be fetched in ascending or descending manner
			XmlNodeList		FieldNodes=null;			//This hold the list of nodes in the FieldCollection node in the Input XML
			XmlNode			FieldCollectionNode = null;	//This has the FieldCollectionNode which holds all the information about the fields
			XmlNode			FieldNameNode = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
			XmlNode			ObjectNode = null;			//This is the Type name node
			XmlNode			SortTypeNode = null;		//The records are selected based on the sortType (Ascending/Descending) specified in the InputXML.This contains the SortType node in the InputXML
			XmlNodeList		SortOrderNodes = null;		//The records are selected based on the sortOrder specified in the InputXML.This contains the SortOrder node in the InputXML
			XmlElement		OutputFieldNode = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
			int				index=0;					//If the String is found in another string then Index will be 0 else it will be -1
			StringBuilder	Query = null;				//The select query formed is saved in this variable
			StringBuilder	FieldList = null;			//The FieldList represents the list of fields.This is used to append in the Insert Query 
			XmlDocument		InputXMLDocument = null;	//This Contains the Main Document node of the InputXML
			OMError			ErrorObj = null;			//In case there is an error in the function this object will be populated
			StringBuilder	SortClause = new StringBuilder();
			XmlNode			HourNode;					//This contains the Hour node which specifies the Hour in the date
			XmlNode			MinuteNode;					//This contains the Minute node which specifies the Minute in the date 
			XmlNode			SecondNode;					//This contains the Second node which specifies the Second in the date 
			XmlNode			DayNode;					//This contains the Day node which specifies the day in the date 
			XmlNode			MonthNode;					//This contains the Month node which specifies the Month in the date 
			XmlNode			YearNode;					//This contains the Year node which specifies the Year in the date 
			DateTime		DateValue;
			string			FieldType = "";				//This has the field Data Type of the individual field.This field type is used for formation of OutputXML to return the data back
            bool            CacheEnabled =  false;
            String          cachedItemName  = null;
			try
            {
                #region Read From Cache
                CacheEnabled    = true;//Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
                cachedItemName  = TenantName.ToUpper() + "_" + TableName.ToUpper();
               
                if (CacheEnabled == true &&
                    CacheManager.IsCached(cachedItemName) == true)
                {
                    ErrorObj = ReadFromCache(false,cachedItemName, TypeName, FieldListXML,ref  OutputXML);
                    if (ErrorObj == null)
                        return null;

                }
                #endregion Read From Cache

                /*If the FieldListXML is blank then select all the fields of the given TypeName 
				else select the fields specified in the InputXML*/
				Query = new StringBuilder();
				if ((FieldListXML == "") || (FieldListXML==null))
				{
					Query.AppendFormat("SELECT TOP 1 * FROM {0}", TableName.ToUpper());
				}
				else
				{
					FieldList           = new StringBuilder();
					InputXMLDocument    = new XmlDocument();
					InputXMLDocument.LoadXml(FieldListXML);
					Query.AppendFormat("SELECT TOP 1 ");
					
					FieldCollectionNode = InputXMLDocument.SelectSingleNode("FIELDCOLLECTION");
					FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
					Seperator = "";
					foreach (XmlElement FieldNode in FieldNodes)
					{
						FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                        FieldList.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
                        Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
						Seperator = ",";
					}

					/* While selecting the records the consumer can demand for all the fields or selected fields. 
					The mandatory fields also needs to be returned along with the Fields asked for, otherwise the Solution 
					layer will have problem.Hence the mandatory fields are selected from the ALOM_Schema table*/

					StringBuilder QuerySB = new StringBuilder();
					QuerySB.AppendFormat("Select FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME ='{0}' AND Upper(ISMANDATORY) = 'TRUE'", TableName);

                    try
                    {
                        ErrorObj = RetrieveResult(QuerySB.ToString());
                        if (ErrorObj != null)
                        {
                            m_Clause = "";
                            m_SortClause = "";
                            m_ClauseForCache = "";
                            m_SortClauseForCache = "";
                            m_ClauseForDataTables = "";
                            return ErrorObj;
                        }
                        while (DataReaderObj.Read())
                        {
                            FieldName = DataReaderObj.GetValue(0).ToString();
                            index = FieldList.ToString().IndexOf(FieldName.ToUpper());
                            // Determine if a valid index is returned. 
                            if (index == -1)
                            {
                                Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()));
                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                    finally
                    {
                        if (DataReaderObj != null && !DataReaderObj.IsClosed)
                            DataReaderObj.Close();
                    }

					/* The consumer might specify the Sort order and SortType while selecting the records
					this part of the code prepares the order by Clause which will be used in the select Query*/
					
					SortOrderNodes = FieldCollectionNode.SelectNodes("FIELD/SORTORDER");
					Seperator = "";
					for (int NodeCount = 0; NodeCount  < SortOrderNodes.Count; NodeCount ++)
					{
						FieldNodes = FieldCollectionNode.SelectNodes("FIELD[SORTORDER='" + NodeCount + "']");
						if (FieldNodes.Count > 0)
						{
							FieldNameNode = FieldNodes[0].SelectSingleNode("FIELDNAME");
							SortTypeNode  = FieldNodes[0].SelectSingleNode("SORTTYPE");
							if ((SortTypeNode.InnerXml).ToUpper() == "ASCENDING")
							{
								SortType = "ASC";
							}
							else
							{
								SortType = "DESC";
							}
                            SortClause.AppendFormat("{0} {1} {2}", Seperator, ChangeFieldNameIncaseItIsReserved(FieldNameNode.InnerXml.ToUpper()), SortType);
							Seperator = ",";
						}
					}
					Query.AppendFormat("FROM {0}", TableName.ToUpper());
				}
				if ((m_Clause != "") && (m_Clause != null))
				{
					Query.AppendFormat(" WHERE {0}", m_Clause);
				}
				if ((SortClause.ToString() != "") && (SortClause.ToString() != null))
				{
					Query.AppendFormat(" ORDER BY {0}", SortClause.ToString());
				}

                /* Based on the Query the Records are fetched. If the number of records is greater than one 
				 * the collection of type nodes is formed and returned else only a sicgle Type node is returned*/
                try
                {
                    ErrorObj = RetrieveResult(Query.ToString());
                    if (ErrorObj != null)
                    {
                        m_Clause = "";
                        m_SortClause = "";
                        m_ClauseForCache = "";
                        m_SortClauseForCache = "";
                        m_ClauseForDataTables = "";
                        return ErrorObj;
                    }


                    int RowCount = 0;
                    XmlDocument OutputXMLDocument = new XmlDocument();
                    ObjectNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName.ToUpper(), "");
                    OutputXMLDocument.AppendChild(ObjectNode);
                    while (DataReaderObj.Read())
                    {
                        for (int FieldCount = 0; FieldCount < DataReaderObj.FieldCount; FieldCount++)
                        {
                            FieldType = DataReaderObj.GetFieldType(FieldCount).ToString();
                            FieldName = DataReaderObj.GetName(FieldCount);
                            FieldValue = DataReaderObj.GetValue(FieldCount).ToString();

                            OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                            OutputFieldNode.InnerText = "";

                            if ((FieldValue != null) && (FieldValue != ""))
                            {
                                switch (FieldType)
                                {
                                    case "System.DateTime":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        DayNode = OutputXMLDocument.CreateElement("DAY");
                                        DateValue = Convert.ToDateTime(FieldValue);
                                        DayNode.InnerText = DateValue.Day.ToString();
                                        OutputFieldNode.AppendChild(DayNode);
                                        MonthNode = OutputXMLDocument.CreateElement("MONTH");
                                        MonthNode.InnerText = DateValue.Month.ToString();
                                        OutputFieldNode.AppendChild(MonthNode);
                                        YearNode = OutputXMLDocument.CreateElement("YEAR");
                                        YearNode.InnerText = DateValue.Year.ToString();
                                        OutputFieldNode.AppendChild(YearNode);
                                        if (DateValue.Hour != 0)
                                        {
                                            HourNode = OutputXMLDocument.CreateElement("HOUR");
                                            HourNode.InnerText = DateValue.Hour.ToString();
                                            OutputFieldNode.AppendChild(HourNode);
                                            MinuteNode = OutputXMLDocument.CreateElement("MINUTE");
                                            MinuteNode.InnerText = DateValue.Minute.ToString();
                                            OutputFieldNode.AppendChild(MinuteNode);
                                            SecondNode = OutputXMLDocument.CreateElement("SECOND");
                                            SecondNode.InnerText = DateValue.Second.ToString();
                                            OutputFieldNode.AppendChild(SecondNode);
                                        }
                                        break;
                                    default:
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerText = FieldValue;
                                        break;
                                }
                            }
                            ObjectNode.AppendChild(OutputFieldNode);
                        }
                        RowCount = RowCount + 1;
                    }

                    if (RowCount == 0)
                        OutputXML = null;
                    else
                        OutputXML = ObjectNode.OuterXml;

                    m_Clause = "";
                    m_SortClause = "";
                    m_ClauseForCache = "";
                    m_SortClauseForCache = "";
                    m_ClauseForDataTables = "";
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
				return null;
			}
			catch (Exception ExceptionObj)
			{
                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";
				ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
						   ExceptionObj.Message, OMErrorObjectType.TABLE,
						   null, OMErrorType.EXCEPTION, ExceptionObj, null);
				return ErrorObj;
			}
		}

        //This method is mostly same as ReadRecord() method. 
        //The only difference is the field's data type checking in this (ReadRecordEx) method is done by using ALOM_SCHEMA table,
        //where as in ReadRecord() method the field's data type checkin is done from the existing table.
        //This changes are done for the new data type added in the system. This is XMLTEXT data type.
        virtual public OMError ReadRecordEx(string TypeName, string TableName, string FieldListXML, ref string OutputXML)
        {
            string FieldName = "";				//This has the field name of the individual field.This field name is used for formation of OutputXML to return the data back
            string FieldValue = "";				//This has the field Value of the individual field.This field value is used for formation of OutputXML to return the data back
            string Seperator = "";				//This contains the seperator value like "," which is used for query formation
            string SortType = "";				//This decides whether the data will be fetched in ascending or descending manner
            XmlNodeList FieldNodes = null;			//This hold the list of nodes in the FieldCollection node in the Input XML
            XmlNode FieldCollectionNode = null;	//This has the FieldCollectionNode which holds all the information about the fields
            XmlNode FieldNameNode = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            XmlNode ObjectNode = null;			//This is the Type name node
            XmlNode SortTypeNode = null;		//The records are selected based on the sortType (Ascending/Descending) specified in the InputXML.This contains the SortType node in the InputXML
            XmlNodeList SortOrderNodes = null;		//The records are selected based on the sortOrder specified in the InputXML.This contains the SortOrder node in the InputXML
            XmlElement OutputFieldNode = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            int index = 0;					//If the String is found in another string then Index will be 0 else it will be -1
            StringBuilder Query = null;				//The select query formed is saved in this variable
            StringBuilder FieldList = null;			//The FieldList represents the list of fields.This is used to append in the Insert Query 
            XmlDocument InputXMLDocument = null;	//This Contains the Main Document node of the InputXML
            OMError ErrorObj = null;			//In case there is an error in the function this object will be populated
            StringBuilder SortClause = new StringBuilder();
            XmlNode HourNode;					//This contains the Hour node which specifies the Hour in the date
            XmlNode MinuteNode;					//This contains the Minute node which specifies the Minute in the date 
            XmlNode SecondNode;					//This contains the Second node which specifies the Second in the date 
            XmlNode DayNode;					//This contains the Day node which specifies the day in the date 
            XmlNode MonthNode;					//This contains the Month node which specifies the Month in the date 
            XmlNode YearNode;					//This contains the Year node which specifies the Year in the date 
            DateTime DateValue;
            string FieldType = "";				//This has the field Data Type of the individual field.This field type is used for formation of OutputXML to return the data back

            //Following variable are used for the reading FieldNams and FieldType from ALOM_SCHEMA table.
            String[] ArrFieldName = null;
            String[] ArrFieldType = null;
            String FieldNameList = "";
            String FieldTypeList = "";
            StringBuilder QueryOnSchema = null;
            int FieldNameTypeCount = 0;

            try
            {
                /*If the FieldListXML is blank then select all the fields of the given TypeName 
                else select the fields specified in the InputXML*/
                Query = new StringBuilder();
                if ((FieldListXML == "") || (FieldListXML == null))
                {
                    Query.AppendFormat("SELECT TOP 1 * FROM {0}", TableName.ToUpper());
                }
                else
                {
                    FieldList = new StringBuilder();
                    InputXMLDocument = new XmlDocument();
                    InputXMLDocument.LoadXml(FieldListXML);
                    Query.AppendFormat("SELECT TOP 1 ");

                    FieldCollectionNode = InputXMLDocument.SelectSingleNode("FIELDCOLLECTION");
                    FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
                    Seperator = "";
                    foreach (XmlElement FieldNode in FieldNodes)
                    {
                        FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                        FieldList.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
                        Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
                        Seperator = ",";
                    }

                    /* While selecting the records the consumer can demand for all the fields or selected fields. 
                    The mandatory fields also needs to be returned along with the Fields asked for, otherwise the Solution 
                    layer will have problem.Hence the mandatory fields are selected from the ALOM_Schema table*/

                    StringBuilder QuerySB = new StringBuilder();
                    QuerySB.AppendFormat("Select FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME ='{0}' AND Upper(ISMANDATORY) = 'TRUE'", TableName);
                    try
                    {
                        ErrorObj = RetrieveResult(QuerySB.ToString());
                        if (ErrorObj != null)
                        {
                            m_Clause = "";
                            m_SortClause = "";
                            m_ClauseForCache = "";
                            m_SortClauseForCache = "";
                            m_ClauseForDataTables = "";
                            return ErrorObj;
                        }
                        while (DataReaderObj.Read())
                        {
                            FieldName = DataReaderObj.GetValue(0).ToString();
                            index = FieldList.ToString().IndexOf(FieldName.ToUpper());
                            // Determine if a valid index is returned. 
                            if (index == -1)
                            {
                                Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()));
                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                    finally
                    {
                        if (DataReaderObj != null && !DataReaderObj.IsClosed)
                            DataReaderObj.Close();
                    }

                    /* The consumer might specify the Sort order and SortType while selecting the records
                    this part of the code prepares the order by Clause which will be used in the select Query*/

                    SortOrderNodes = FieldCollectionNode.SelectNodes("FIELD/SORTORDER");
                    Seperator = "";
                    for (int NodeCount = 0; NodeCount < SortOrderNodes.Count; NodeCount++)
                    {
                        FieldNodes = FieldCollectionNode.SelectNodes("FIELD[SORTORDER='" + NodeCount + "']");
                        if (FieldNodes.Count > 0)
                        {
                            FieldNameNode = FieldNodes[0].SelectSingleNode("FIELDNAME");
                            SortTypeNode = FieldNodes[0].SelectSingleNode("SORTTYPE");
                            if ((SortTypeNode.InnerXml).ToUpper() == "ASCENDING")
                            {
                                SortType = "ASC";
                            }
                            else
                            {
                                SortType = "DESC";
                            }
                            SortClause.AppendFormat("{0} {1} {2}", Seperator, ChangeFieldNameIncaseItIsReserved(FieldNameNode.InnerXml.ToUpper()), SortType);
                            Seperator = ",";
                        }
                    }
                    Query.AppendFormat("FROM {0}", TableName.ToUpper());
                }
                if ((m_Clause != "") && (m_Clause != null))
                {
                    Query.AppendFormat(" WHERE {0}", m_Clause);
                }
                if ((SortClause.ToString() != "") && (SortClause.ToString() != null))
                {
                    Query.AppendFormat(" ORDER BY {0}", SortClause.ToString());
                }

                //Get the FieldName and FieldType of Table from ALOM_SCHEMA.
                //This is added on date 05th Jan 06 by Kiran Abhang.
                QueryOnSchema = new StringBuilder();
                QueryOnSchema.Append("SELECT FIELDNAME, FIELDTYPE FROM ALOM_SCHEMA");
                QueryOnSchema.AppendFormat(" WHERE TABLENAME = '{0}'", TableName);
                try
                {
                    ErrorObj = RetrieveResult(QueryOnSchema.ToString());
                    if (ErrorObj != null)
                    {
                        m_Clause = "";
                        m_SortClause = "";
                        m_ClauseForCache = "";
                        m_SortClauseForCache = "";
                        m_ClauseForDataTables = "";
                        return ErrorObj;
                    }

                    while (DataReaderObj.Read())
                    {
                        if (FieldNameList != "")
                            FieldNameList = FieldNameList + ",";
                        FieldNameList = FieldNameList + DataReaderObj.GetValue(0).ToString();

                        if (FieldTypeList != "")
                            FieldTypeList = FieldTypeList + ",";
                        FieldTypeList = FieldTypeList + DataReaderObj.GetValue(1).ToString();
                        FieldNameTypeCount++;
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }

                ArrFieldName = FieldNameList.Split(',');
                ArrFieldType = FieldTypeList.Split(',');
                //End of getting the FieldName and FieldType.

                /* Based on the Query the Records are fetched. If the number of records is greater than one 
				 * the collection of type nodes is formed and returned else only a sicgle Type node is returned*/
                try
                {
                    ErrorObj = RetrieveResult(Query.ToString());
                    if (ErrorObj != null)
                    {
                        m_Clause = "";
                        m_SortClause = "";
                        m_ClauseForCache = "";
                        m_SortClauseForCache = "";
                        m_ClauseForDataTables = "";
                        return ErrorObj;
                    }

                    int RowCount = 0;
                    XmlDocument OutputXMLDocument = new XmlDocument();
                    ObjectNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName.ToUpper(), "");
                    OutputXMLDocument.AppendChild(ObjectNode);
                    while (DataReaderObj.Read())
                    {
                        for (int FieldCount = 0; FieldCount < DataReaderObj.FieldCount; FieldCount++)
                        {
                            //FieldType = DataReaderObj.GetFieldType(FieldCount).ToString();
                            FieldType = "";
                            FieldName = DataReaderObj.GetName(FieldCount);
                            FieldValue = DataReaderObj.GetValue(FieldCount).ToString();

                            OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                            OutputFieldNode.InnerText = "";

                            if ((FieldValue != null) && (FieldValue != ""))
                            {
                                //Compare with FieldName from Schema table and from actual table.
                                //Added on date 05th Jan 06 by Kiran Abhang.
                                for (FieldNameTypeCount = 0; FieldNameTypeCount < ArrFieldName.Length; FieldNameTypeCount++)
                                {
                                    if (FieldName.ToUpper() == ArrFieldName[FieldNameTypeCount].ToUpper())
                                    {
                                        FieldType = ArrFieldType[FieldNameTypeCount];
                                        break;
                                    }
                                }

                                switch (FieldType)
                                {
                                    case "DATETIME":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        DayNode = OutputXMLDocument.CreateElement("DAY");
                                        DateValue = Convert.ToDateTime(FieldValue);
                                        DayNode.InnerText = DateValue.Day.ToString();
                                        OutputFieldNode.AppendChild(DayNode);
                                        MonthNode = OutputXMLDocument.CreateElement("MONTH");
                                        MonthNode.InnerText = DateValue.Month.ToString();
                                        OutputFieldNode.AppendChild(MonthNode);
                                        YearNode = OutputXMLDocument.CreateElement("YEAR");
                                        YearNode.InnerText = DateValue.Year.ToString();
                                        OutputFieldNode.AppendChild(YearNode);
                                        if (DateValue.Hour != 0)
                                        {
                                            HourNode = OutputXMLDocument.CreateElement("HOUR");
                                            HourNode.InnerText = DateValue.Hour.ToString();
                                            OutputFieldNode.AppendChild(HourNode);
                                            MinuteNode = OutputXMLDocument.CreateElement("MINUTE");
                                            MinuteNode.InnerText = DateValue.Minute.ToString();
                                            OutputFieldNode.AppendChild(MinuteNode);
                                            SecondNode = OutputXMLDocument.CreateElement("SECOND");
                                            SecondNode.InnerText = DateValue.Second.ToString();
                                            OutputFieldNode.AppendChild(SecondNode);
                                        }
                                        break;
                                    case "XMLTEXT":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerXml = FieldValue;
                                        break;
                                    default:
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerText = FieldValue;
                                        break;
                                }
                            }
                            ObjectNode.AppendChild(OutputFieldNode);
                        }
                        RowCount = RowCount + 1;
                    }

                    if (RowCount == 0)
                        OutputXML = null;
                    else
                        OutputXML = ObjectNode.OuterXml;

                    m_Clause = "";
                    m_SortClause = "";
                    m_ClauseForCache = "";
                    m_SortClauseForCache = "";
                    m_ClauseForDataTables = "";
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";
                ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
                return ErrorObj;
            }
        }

		virtual public OMError ReadAllRecord(string TypeName, string TableName, string FieldListXML, ref string OutputXML)
		{
			string          FieldName               = null;	//This has the field name of the individual field.This field name is used for formation of OutputXML to return the data back
			string          FieldValue              = null;	//This has the field Value of the individual field.This field value is used for formation of OutputXML to return the data back
			string          FieldType               = null;	//This has the field Data Type of the individual field.This field type is used for formation of OutputXML to return the data back
			string          Seperator               = null;	//This contains the seperator value like "," which is used for query formation
			string          SortType                = null;	//This decides whether the data will be fetched in ascending or descending manner
			XmlNodeList     FieldNodes              = null;	//This hold the list of nodes in the FieldCollection node in the Input XML
			XmlNode         FieldCollectionNode     = null;	//This has the FieldCollectionNode which holds all the information about the fields
			XmlNode         FieldNameNode           = null;	//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
			XmlNode         ObjectNode              = null;	//This is the Type name node
			XmlNode         SortTypeNode            = null;	//The records are selected based on the sortType (Ascending/Descending) specified in the InputXML.This contains the SortType node in the InputXML
			XmlNodeList     SortOrderNodes          = null;	//The records are selected based on the sortOrder specified in the InputXML.This contains the SortOrder node in the InputXML
			XmlNode         ObjectCollectionNode    = null; //This contains the FieldCollection node that is passed back.
			XmlElement      OutputFieldNode         = null;	//This variable holds the FieldName Node of the individual field node specified in the FieldCollection			
			StringBuilder   Query                   = null;	//The select query formed is saved in this variable
			StringBuilder   FieldList               = null;	//The FieldList represents the list of fields.This is used to append in the Insert Query 
			XmlDocument     InputXMLDocument        = null; //This Contains the Main Document node of the InputXML
			OMError         ErrorObj                = null;	//In case there is an error in the function this object will be populated
			StringBuilder   SortClause              = null;
			XmlNode         HourNode                = null;	//This contains the Hour node which specifies the Hour in the date
			XmlNode         MinuteNode              = null;	//This contains the Minute node which specifies the Minute in the date 
			XmlNode         SecondNode              = null;	//This contains the Second node which specifies the Second in the date 
			XmlNode         DayNode                 = null;	//This contains the Day node which specifies the day in the date 
			XmlNode         MonthNode               = null;	//This contains the Month node which specifies the Month in the date 
			XmlNode         YearNode                = null;	//This contains the Year node which specifies the Year in the date 
			DateTime        DateValue;
            bool            CacheEnabled            = false; //To check for OM Caching
            String          cachedItemName          = null;
			try
            {

                #region Read From Cache
          
                CacheEnabled    = true;//Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
                cachedItemName  = TenantName.ToUpper() + "_" + TableName.ToUpper();

                if (CacheEnabled == true &&
                    CacheManager.IsCached(cachedItemName) == true)
                {
                    ErrorObj = ReadFromCache(true,cachedItemName, TypeName, FieldListXML, ref  OutputXML);
                    
                    if (ErrorObj == null)
                        return null;

                }
                #endregion

                /*If the FieldListXML is blank then select all the fields of the given TypeName 
				else select the fields specified in the InputXML*/

                


                SortClause  = new StringBuilder();
				Query       = new StringBuilder();
				if ((FieldListXML == "") || (FieldListXML == null))
				{
					Query.AppendFormat("SELECT * FROM {0}", TableName.ToUpper());
				}
				else
				{
					FieldList           = new StringBuilder();
					InputXMLDocument    = new XmlDocument();
					InputXMLDocument.LoadXml(FieldListXML);
					
                    Query.AppendFormat("SELECT ");

					FieldCollectionNode = InputXMLDocument.SelectSingleNode("FIELDCOLLECTION");
					FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
					
                    Seperator = "";
					foreach (XmlElement FieldNode in FieldNodes)
					{
						FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                        FieldList.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
						Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
						Seperator = ",";
					}

					/* While selecting the records the consumer can demand for all the fields or selected fields. 
					The mandatory fields also needs to be returned along with the Fields asked for, otherwise the Solution 
					layer will have problem.Hence the mandatory fields are selected from the ALOM_Schema table*/

                    if ((ErrorObj = retrieveMandatoryFields(TableName, Seperator, Query, FieldList)) != null)
                        return ErrorObj;

					/* The consumer might specify the Sort order and SortType while selecting the records
					this part of the code prepares the order by Clause which will be used in the select Query*/

					SortOrderNodes = FieldCollectionNode.SelectNodes("FIELD/SORTORDER");
					Seperator = "";
					for (int NodeCount = 0; NodeCount < SortOrderNodes.Count; NodeCount++)
					{
						FieldNodes = FieldCollectionNode.SelectNodes("FIELD[SORTORDER='" + NodeCount + "']");
						if (FieldNodes.Count > 0)
						{
							FieldNameNode = FieldNodes[0].SelectSingleNode("FIELDNAME");
							SortTypeNode = FieldNodes[0].SelectSingleNode("SORTTYPE");
							if ((SortTypeNode.InnerXml).ToUpper() == "ASCENDING")
							{
								SortType = "ASC";
							}
							else
							{
								SortType = "DESC";
							}
							SortClause.AppendFormat("{0} {1} {2}", Seperator, ChangeFieldNameIncaseItIsReserved(FieldNameNode.InnerXml.ToUpper()), SortType);
							Seperator = ",";
						}
					}
					Query.AppendFormat("FROM {0}", TableName.ToUpper());
				}
				if ((m_Clause != "") && (m_Clause != null))
				{
					Query.AppendFormat(" WHERE {0}", m_Clause);
				}
				if ((SortClause.ToString() != "") && (SortClause.ToString() != null))
				{
					Query.AppendFormat(" ORDER BY {0}", SortClause.ToString());
				}
                if ((SortClause.ToString() == "") && m_SortClause != null && m_SortClause != "")
                {
                    Query.AppendFormat(" ORDER BY {0}", m_SortClause);
                }
                /* Based on the Query the Records are fetched. If the number of records is greater than one 
				 * the collection of type nodes is formed and returned else only a sicgle Type node is returned*/
                try
                {
                    ErrorObj = RetrieveResult(Query.ToString());
                    if (ErrorObj != null)
                    {
                        m_Clause = "";
                        m_SortClause = "";
                        m_ClauseForCache = "";
                        m_SortClauseForCache = "";
                        m_ClauseForDataTables = "";
                        return ErrorObj;
                    }
                    int RowCount = 0;
                    XmlDocument OutputXMLDocument = new XmlDocument();
                    ObjectCollectionNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName.ToUpper() + "COLLECTION", "");
                    OutputXMLDocument.AppendChild(ObjectCollectionNode);
                    while (DataReaderObj.Read())
                    {
                        ObjectNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName.ToUpper(), "");
                        for (int FieldCount = 0; FieldCount < DataReaderObj.FieldCount; FieldCount++)
                        {
                            FieldType = DataReaderObj.GetFieldType(FieldCount).ToString();
                            FieldName = DataReaderObj.GetName(FieldCount);
                            FieldValue = DataReaderObj.GetValue(FieldCount).ToString();

                            OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                            OutputFieldNode.InnerText = "";

                            if ((FieldValue != null) && (FieldValue != ""))
                            {
                                switch (FieldType)
                                {
                                    case "System.DateTime":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        DayNode = OutputXMLDocument.CreateElement("DAY");
                                        DateValue = Convert.ToDateTime(FieldValue);
                                        DayNode.InnerText = DateValue.Day.ToString();
                                        OutputFieldNode.AppendChild(DayNode);
                                        MonthNode = OutputXMLDocument.CreateElement("MONTH");
                                        MonthNode.InnerText = DateValue.Month.ToString();
                                        OutputFieldNode.AppendChild(MonthNode);
                                        YearNode = OutputXMLDocument.CreateElement("YEAR");
                                        YearNode.InnerText = DateValue.Year.ToString();
                                        OutputFieldNode.AppendChild(YearNode);
                                        if (DateValue.Hour != 0)
                                        {
                                            HourNode = OutputXMLDocument.CreateElement("HOUR");
                                            HourNode.InnerText = DateValue.Hour.ToString();
                                            OutputFieldNode.AppendChild(HourNode);
                                            MinuteNode = OutputXMLDocument.CreateElement("MINUTE");
                                            MinuteNode.InnerText = DateValue.Minute.ToString();
                                            OutputFieldNode.AppendChild(MinuteNode);
                                            SecondNode = OutputXMLDocument.CreateElement("SECOND");
                                            SecondNode.InnerText = DateValue.Second.ToString();
                                            OutputFieldNode.AppendChild(SecondNode);
                                        }
                                        break;
                                    default:
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerText = FieldValue;
                                        break;
                                }
                            }

                            ObjectNode.AppendChild(OutputFieldNode);
                        }
                        ObjectCollectionNode.AppendChild(ObjectNode);
                        RowCount = RowCount + 1;
                    }

                    if (RowCount == 0)
                        OutputXML = null;
                    else
                        OutputXML = ObjectCollectionNode.OuterXml;

                    m_Clause = "";
                    m_SortClause = "";
                    m_ClauseForCache = "";
                    m_SortClauseForCache = "";
                    m_ClauseForDataTables = "";
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
				return null;
			}
			catch (Exception ExceptionObj)
			{
                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";
				ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
						   ExceptionObj.Message, OMErrorObjectType.TABLE,
						   null, OMErrorType.EXCEPTION, ExceptionObj, null);
				return ErrorObj;
			}
		}

        /// <summary>
        /// retrive the list of Mandatory Fields from Cache if not then from ALOM_SCHEMA
        /// </summary>
        /// <param name="TableName">Table Name for which schema needs to be retrieved</param>
        /// <param name="Seperator"></param>
        /// <param name="Query">The list of fields to be appended</param>
        /// <param name="FieldList">The existsing list of fields</param>
        /// <returns>OMError Object</returns>
        public OMError retrieveMandatoryFields(string TableName,  string Seperator,  StringBuilder Query, StringBuilder FieldList)
        {
            OMError     ErrorObj        = null;
            string      FieldName       = null;
            int         index           = 0;
            DataTable   systemTable     = null;
            string      cacheItemName   = null;
            bool        CacheEnabled    = false;


            CacheEnabled    = true;//Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
            cacheItemName   = TenantName.ToUpper() + "_ALOM_SCHEMA";
            if (CacheEnabled == false || CacheManager.IsCached(cacheItemName) == false)    //If ALOM_SCHEMA is not in cache or Cache is not enabled
            {
                StringBuilder QuerySB = new StringBuilder();
                QuerySB.AppendFormat("Select FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME ='{0}' AND Upper(ISMANDATORY) = 'TRUE'", TableName);
                try
                {
                    ErrorObj = RetrieveResult(QuerySB.ToString());
                    if (ErrorObj != null)
                    {
                        m_Clause = "";
                        m_SortClause = "";
                        m_ClauseForDataTables = "";
                        return ErrorObj;
                    }
                    while (DataReaderObj.Read())
                    {
                        FieldName = DataReaderObj.GetValue(0).ToString();
                        index = FieldList.ToString().IndexOf(FieldName.ToUpper());
                        // Determine if a valid index is returned. 
                        if (index == -1)
                        {
                            Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()));
                        }
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
            }
            else
            {
                systemTable     = CacheManager.GetCacheItem(cacheItemName) as DataTable;

                string query    = "TABLENAME ='" + TableName.ToUpper() + "' AND ISMANDATORY = 'TRUE'";
                DataRow[] rows  = systemTable.Select(query);
                foreach (DataRow  row in rows)
	            {
                    FieldName   = row["FIELDNAME"] as String;
                    index       = FieldList.ToString().IndexOf(FieldName.ToUpper());
                    // Determine if a valid index is returned. 
                    if (index == -1)
                    {
                        Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()));
                    }
	            }
            }
            return null;
        }

        //This method is mostly same as ReadAllRecord() method. 
        //The only difference is the field's data type checking in this (ReadAllRecordEx) method is done by using ALOM_SCHEMA table,
        //where as in ReadAllRecord() method the field's data type checkin is done from the existing table.
        //This changes are done for the new data type added in the system. This is XMLTEXT data type.
        virtual public OMError ReadAllRecordEx(string TypeName, string TableName, string FieldListXML, ref string OutputXML)
        {
            string FieldName = "";				//This has the field name of the individual field.This field name is used for formation of OutputXML to return the data back
            string FieldValue = "";				//This has the field Value of the individual field.This field value is used for formation of OutputXML to return the data back
            string FieldType = "";				//This has the field Data Type of the individual field.This field type is used for formation of OutputXML to return the data back
            string Seperator = "";				//This contains the seperator value like "," which is used for query formation
            string SortType = "";				//This decides whether the data will be fetched in ascending or descending manner
            XmlNodeList FieldNodes = null;		//This hold the list of nodes in the FieldCollection node in the Input XML
            XmlNode FieldCollectionNode = null;	//This has the FieldCollectionNode which holds all the information about the fields
            XmlNode FieldNameNode = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            XmlNode ObjectNode = null;			//This is the Type name node
            XmlNode SortTypeNode = null;		//The records are selected based on the sortType (Ascending/Descending) specified in the InputXML.This contains the SortType node in the InputXML
            XmlNodeList SortOrderNodes = null;	//The records are selected based on the sortOrder specified in the InputXML.This contains the SortOrder node in the InputXML
            XmlNode ObjectCollectionNode = null;//This contains the FieldCollection node that is passed back.
            XmlElement OutputFieldNode = null;	//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            int index = 0;					//If the String is found in another string then Index will be 0 else it will be -1
            StringBuilder Query = null;			//The select query formed is saved in this variable
            StringBuilder FieldList = null;		//The FieldList represents the list of fields.This is used to append in the Insert Query 
            XmlDocument InputXMLDocument = null;//This Contains the Main Document node of the InputXML
            OMError ErrorObj = null;			//In case there is an error in the function this object will be populated
            StringBuilder SortClause = new StringBuilder();
            XmlNode HourNode;					//This contains the Hour node which specifies the Hour in the date
            XmlNode MinuteNode;					//This contains the Minute node which specifies the Minute in the date 
            XmlNode SecondNode;					//This contains the Second node which specifies the Second in the date 
            XmlNode DayNode;					//This contains the Day node which specifies the day in the date 
            XmlNode MonthNode;					//This contains the Month node which specifies the Month in the date 
            XmlNode YearNode;					//This contains the Year node which specifies the Year in the date 
            DateTime DateValue;

            //Following variable are used for the reading FieldNams and FieldType from ALOM_SCHEMA table.
            String[] ArrFieldName = null;
            String[] ArrFieldType = null;
            String FieldNameList = "";
            String FieldTypeList = "";
            StringBuilder QueryOnSchema = null;
            int FieldNameTypeCount = 0;

            DynamicSQLParameter m_parameter;

            try
            {
                /*If the FieldListXML is blank then select all the fields of the given TypeName 
                else select the fields specified in the InputXML*/

                Query = new StringBuilder();
                if ((FieldListXML == "") || (FieldListXML == null))
                {
                    Query.AppendFormat("SELECT * FROM {0}", TableName.ToUpper());
                }
                else
                {
                    FieldList = new StringBuilder();
                    InputXMLDocument = new XmlDocument();
                    InputXMLDocument.LoadXml(FieldListXML);
                    Query.AppendFormat("SELECT ");

                    FieldCollectionNode = InputXMLDocument.SelectSingleNode("FIELDCOLLECTION");
                    FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
                    Seperator = "";
                    foreach (XmlElement FieldNode in FieldNodes)
                    {
                        FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                        FieldList.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
                        Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
                        Seperator = ",";
                    }

                    /* While selecting the records the consumer can demand for all the fields or selected fields. 
                    The mandatory fields also needs to be returned along with the Fields asked for, otherwise the Solution 
                    layer will have problem.Hence the mandatory fields are selected from the ALOM_Schema table*/

                    //m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TableName + "'", "NVARCHAR(255)");
                    //if (m_DeferQueryExecution)
                    //    DynamicParameters_Defer.Add(m_parameter);
                    //else
                    //    DynamicParameters.Add(m_parameter);

                    StringBuilder QuerySB = new StringBuilder();
                    //QuerySB.AppendFormat("Select FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME = {0} AND Upper(ISMANDATORY) = 'TRUE'", m_parameter.m_ParameterName);
                    QuerySB.AppendFormat("Select FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME = {0} AND Upper(ISMANDATORY) = 'TRUE'", "'" + TableName + "'");

                    try
                    {
                        ErrorObj = RetrieveResult(QuerySB.ToString());
                        if (ErrorObj != null)
                        {
                            m_Clause = "";
                            m_SortClause = "";
                            m_ClauseForCache = "";
                            m_SortClauseForCache = "";
                            m_ClauseForDataTables = "";
                            return ErrorObj;
                        }
                        while (DataReaderObj.Read())
                        {
                            FieldName = DataReaderObj.GetValue(0).ToString();
                            index = FieldList.ToString().IndexOf(FieldName.ToUpper());
                            // Determine if a valid index is returned. 
                            if (index == -1)
                            {
                                Query.AppendFormat("{0}{1} ", Seperator, ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()));
                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                    finally
                    {
                        if (DataReaderObj != null && !DataReaderObj.IsClosed)
                            DataReaderObj.Close();
                    }

                    /* The consumer might specify the Sort order and SortType while selecting the records
                    this part of the code prepares the order by Clause which will be used in the select Query*/

                    SortOrderNodes = FieldCollectionNode.SelectNodes("FIELD/SORTORDER");
                    Seperator = "";
                    for (int NodeCount = 0; NodeCount < SortOrderNodes.Count; NodeCount++)
                    {
                        FieldNodes = FieldCollectionNode.SelectNodes("FIELD[SORTORDER='" + NodeCount + "']");
                        if (FieldNodes.Count > 0)
                        {
                            FieldNameNode = FieldNodes[0].SelectSingleNode("FIELDNAME");
                            SortTypeNode = FieldNodes[0].SelectSingleNode("SORTTYPE");
                            if ((SortTypeNode.InnerXml).ToUpper() == "ASCENDING")
                            {
                                SortType = "ASC";
                            }
                            else
                            {
                                SortType = "DESC";
                            }
                            SortClause.AppendFormat("{0} {1} {2}", Seperator, ChangeFieldNameIncaseItIsReserved(FieldNameNode.InnerXml.ToUpper()), SortType);
                            Seperator = ",";
                        }
                    }
                    Query.AppendFormat("FROM {0}", TableName.ToUpper());
                }
                if ((m_Clause != "") && (m_Clause != null))
                {
                    Query.AppendFormat(" WHERE {0}", m_Clause);
                }
                if ((SortClause.ToString() != "") && (SortClause.ToString() != null))
                {
                    Query.AppendFormat(" ORDER BY {0}", SortClause.ToString());
                }

                //Get the FieldName and FieldType of Table from ALOM_SCHEMA.
                //This is added on date 05th Jan 06 by Kiran Abhang.

                //m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TableName + "'", "NVARCHAR(255)");
                //if (m_DeferQueryExecution)
                //    DynamicParameters_Defer.Add(m_parameter);
                //else
                //    DynamicParameters.Add(m_parameter);

                QueryOnSchema = new StringBuilder();
                QueryOnSchema.Append("SELECT FIELDNAME, FIELDTYPE FROM ALOM_SCHEMA");
                //QueryOnSchema.AppendFormat(" WHERE TABLENAME = {0} ", m_parameter.m_ParameterName);
                QueryOnSchema.AppendFormat(" WHERE TABLENAME = {0} ", "'" + TableName + "'");

                try
                {
                    ErrorObj = RetrieveResult(QueryOnSchema.ToString());
                    if (ErrorObj != null)
                    {
                        m_Clause = "";
                        m_SortClause = "";
                        m_ClauseForCache = "";
                        m_SortClauseForCache = "";
                        m_ClauseForDataTables = "";
                        return ErrorObj;
                    }

                    while (DataReaderObj.Read())
                    {
                        if (FieldNameList != "")
                            FieldNameList = FieldNameList + ",";
                        FieldNameList = FieldNameList + DataReaderObj.GetValue(0).ToString();

                        if (FieldTypeList != "")
                            FieldTypeList = FieldTypeList + ",";
                        FieldTypeList = FieldTypeList + DataReaderObj.GetValue(1).ToString();
                        FieldNameTypeCount++;
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }

                ArrFieldName = FieldNameList.Split(',');
                ArrFieldType = FieldTypeList.Split(',');
                //End of getting the FieldName and FieldType.

                /* Based on the Query the Records are fetched. If the number of records is greater than one 
				 * the collection of type nodes is formed and returned else only a sicgle Type node is returned*/
                try
                {
                    ErrorObj = RetrieveResult(Query.ToString());
                    if (ErrorObj != null)
                    {
                        m_Clause = "";
                        m_SortClause = "";
                        m_ClauseForCache = "";
                        m_SortClauseForCache = "";
                        m_ClauseForDataTables = "";
                        return ErrorObj;
                    }
                    int RowCount = 0;
                    XmlDocument OutputXMLDocument = new XmlDocument();
                    ObjectCollectionNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName.ToUpper() + "COLLECTION", "");
                    OutputXMLDocument.AppendChild(ObjectCollectionNode);
                    while (DataReaderObj.Read())
                    {
                        ObjectNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName.ToUpper(), "");
                        for (int FieldCount = 0; FieldCount < DataReaderObj.FieldCount; FieldCount++)
                        {
                            //FieldType = DataReaderObj.GetFieldType(FieldCount).ToString();
                            FieldType = "";
                            FieldName = DataReaderObj.GetName(FieldCount);
                            FieldValue = DataReaderObj.GetValue(FieldCount).ToString();

                            OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                            OutputFieldNode.InnerText = "";

                            if ((FieldValue != null) && (FieldValue != ""))
                            {
                                //Compare with FieldName from Schema table and from actual table.
                                //Added on date 05th Jan 06 by Kiran Abhang.
                                for (FieldNameTypeCount = 0; FieldNameTypeCount < ArrFieldName.Length; FieldNameTypeCount++)
                                {
                                    if (FieldName.ToUpper() == ArrFieldName[FieldNameTypeCount].ToUpper())
                                    {
                                        FieldType = ArrFieldType[FieldNameTypeCount];
                                        break;
                                    }
                                }

                                switch (FieldType)
                                {
                                    case "DATETIME":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        DayNode = OutputXMLDocument.CreateElement("DAY");
                                        DateValue = Convert.ToDateTime(FieldValue);
                                        DayNode.InnerText = DateValue.Day.ToString();
                                        OutputFieldNode.AppendChild(DayNode);
                                        MonthNode = OutputXMLDocument.CreateElement("MONTH");
                                        MonthNode.InnerText = DateValue.Month.ToString();
                                        OutputFieldNode.AppendChild(MonthNode);
                                        YearNode = OutputXMLDocument.CreateElement("YEAR");
                                        YearNode.InnerText = DateValue.Year.ToString();
                                        OutputFieldNode.AppendChild(YearNode);
                                        if (DateValue.Hour != 0)
                                        {
                                            HourNode = OutputXMLDocument.CreateElement("HOUR");
                                            HourNode.InnerText = DateValue.Hour.ToString();
                                            OutputFieldNode.AppendChild(HourNode);
                                            MinuteNode = OutputXMLDocument.CreateElement("MINUTE");
                                            MinuteNode.InnerText = DateValue.Minute.ToString();
                                            OutputFieldNode.AppendChild(MinuteNode);
                                            SecondNode = OutputXMLDocument.CreateElement("SECOND");
                                            SecondNode.InnerText = DateValue.Second.ToString();
                                            OutputFieldNode.AppendChild(SecondNode);
                                        }
                                        break;
                                    case "XMLTEXT":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerXml = FieldValue;
                                        break;
                                    default:
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerXml = FieldValue;
                                        break;
                                }
                            }
                            ObjectNode.AppendChild(OutputFieldNode);
                        }
                        ObjectCollectionNode.AppendChild(ObjectNode);
                        RowCount = RowCount + 1;
                    }

                    if (RowCount == 0)
                        OutputXML = null;
                    else
                        OutputXML = ObjectCollectionNode.OuterXml;

                    m_Clause = "";
                    m_SortClause = "";
                    m_ClauseForCache = "";
                    m_SortClauseForCache = "";
                    m_ClauseForDataTables = "";
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";
                ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
                return ErrorObj;
            }
        }

        /// <summary>
        /// This function is used to prepare the Update Query based on the InputXML.
        /// The Insert query is prepared according to different datatypes used. 
        /// For fields having DateTime as datatype the date is converted to make it 
        /// compatible with SQL and Oracle by calling the ConvertDate function.
        /// </summary>
        /// <param name="TableName">This function takes TableName as parameter.This 
        /// specifies which table the record has to be inserted</param>
        /// <param name="InputXML">This function takes InputXML as parameter.
        /// The structure of the XML will be like this
        /// <Plan>
        ///		<PlanName>Plan F'or Tes'ting</PlanName>
        ///		<Description>This is for Testing</Description>
        ///		<Age>25</Age>
        ///		<DOB>
        ///		<Day>3</Day>
        ///		<Month>10</Month>
        ///		<Year>1979</Year>
        ///		</DOB>
        /// </Plan>
        /// </param>
        /// <returns>
        /// This function returns OMError object if there is any error occured.
        /// </returns>
        virtual public OMError UpdateRecord(string TableName, string InputXML)
        {
            StringBuilder       Query = null;	                //This Query prepared is stored in this variable
            XmlNode             DayNode;						//This contains the Day node which specifies the day in the date in InputXML
            XmlNode             MonthNode;						//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode             YearNode;						//This contains the Year node which specifies the Year in the date in InputXML
            XmlNode             HourNode;					    //This contains the Hour node which specifies the Hour in the date in InputXML
            XmlNode             MinuteNode;					    //This contains the Minute node which specifies the Minute in the date in InputXML
            XmlNode             SecondNode;					    //This contains the Second node which specifies the Second in the date in InputXML
            StringBuilder       DateString = null;		        //The complete date including Day/Month/Year is saved in this variable
            string              DataType = "";					//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            string              Seperator = "";					//This contains the seperator value like "," which is used for query formation
            string              FieldSize = "";					//This has the field size of the individual field.This is used for truncation of data

            XmlDocument         InputXMLDocument;				//This Contains the Main Document node of the InputXML
            Hashtable           FieldNameValueCollection;		//HashTable variable contains the FieldName,FieldType pair retrieved form the ALOM_Schema
            XmlElement          RootNode;						//Contains the Root node in the InputXML
            OMError             ErrorObj = null;				//In case there is an error in the function this object will be populated
            FieldInformation    FieldDataObj = new FieldInformation();

            //For XML data type support. Code added on date 7th March 2006 by Kiran.
            String              PrimaryKeyFieldName         = null;
            String              ObjectTypeName              = null;
            String              ObjectType                  = null;
            ulong               PrimaryKeyValue             = 0;
            ulong               ObjectTypeID                = 0;
            IXMLRepository      ObjXMLManager               = null;
            long                RowsAffected                = 0;
            Boolean             FlagIsFieldOtherThanXML     = false;

            //Variables to support Currency As Data Type
            XmlNode TypeNode;						//This contains the Type node which specifies the Currency Type in the InputXML
            XmlNode ValueNode;						//This contains the Value node which specifies the Value in the InputXML
            StringBuilder CurrencyString;
            DynamicSQLParameter m_parameter;

            try
            {
                ErrorObj = GetObjectTypeSchemaDetails(TableName, out PrimaryKeyFieldName, out ObjectTypeID,
                                            out ObjectType, out ObjectTypeName, out FieldNameValueCollection);
                if (ErrorObj != null)
                    return (ErrorObj);

                Query           = new StringBuilder();
                DateString      = new StringBuilder();
                ObjXMLManager   = OMDataManager.XMLRepository;
                FieldDataObj    = new FieldInformation();
                CurrencyString = new StringBuilder();

                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                RootNode = InputXMLDocument.DocumentElement;

                //Get the primary key value. This is for XML data type.
                if ((PrimaryKeyFieldName != null) && (PrimaryKeyFieldName.Length > 0))
                {
                    if (RootNode.SelectSingleNode(PrimaryKeyFieldName) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName).InnerText);
                    else if (RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()).InnerText);
                }

                FlagIsFieldOtherThanXML = false;
                Query.AppendFormat("UPDATE {0} SET ", TableName.ToUpper());
                foreach (XmlElement ChildNode in RootNode.ChildNodes)
                {
                    //Primary Key value is not allow to change in the Update Query.
                    if ((FieldNameValueCollection.ContainsKey((ChildNode.Name).Trim()))
                            && (PrimaryKeyFieldName.ToUpper() != ChildNode.Name.Trim().ToUpper()))  
                    {
                        FieldDataObj = (FieldInformation)FieldNameValueCollection[(ChildNode.Name).Trim()];
                        DataType = FieldDataObj.FieldType;
                        
                        if ((DataType.ToUpper() != "XML")
                                && (DataType.ToUpper() != "BINDER")
                                && (DataType.ToUpper() != "FILE"))
                        {
                            FieldSize = FieldDataObj.FieldSize;
                            switch (DataType.ToUpper())
                            {
                                case "STRING":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + ReplaceSingleQuote(TruncateData(ChildNode.InnerText, Convert.ToInt32(FieldSize))) + "'", "NVARCHAR(MAX)");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);

                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    break;

                                case "XMLTEXT":
                                    if (!(ChildNode.InnerXml == null) || !(ChildNode.InnerXml == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), Convert.ToString(ChildNode.InnerXml), "NVARCHAR(MAX)");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);

                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    if (!String.IsNullOrEmpty(ChildNode.InnerText)) 
                                    {
                                        Int64 iResult = 0;
                                        if (Int64.TryParse(ChildNode.InnerText.Trim(), out iResult))
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), Convert.ToString(ChildNode.InnerText), "BIGINT");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);

                                            Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                            Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);
                                        }
                                        else
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + "Table Name:" + TableName + ", Column Name:" + ChildNode.Name + ", Expression:" + ChildNode.InnerText + @"""");
                                            Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                            Query.AppendFormat(" {0} ", Convert.ToString(ChildNode.InnerText));
                                        }
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    else
                                    {
                                        //Bug Fix : 4435. If value for field is not provided, set it to NULL. 
                                        Query.AppendFormat("{0}{1} = NULL ", Seperator, ChildNode.Name);
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    break;

                                case "BOOLEAN":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(),"N'" + ChildNode.InnerText + "'",  "BIT");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);

                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    break;

                                case "FLOAT":
                                case "DECIMAL":
                                    if (!String.IsNullOrEmpty(ChildNode.InnerText)) 
                                    {
                                        decimal iResult = 0;
                                        if (decimal.TryParse(ChildNode.InnerText.Trim(), out iResult))
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), Convert.ToString(ChildNode.InnerText), "REAL");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);

                                            Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                            Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);
                                        }
                                        else
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + "Table Name:" + TableName + ", Column Name:" + ChildNode.Name + ", Expression:" + ChildNode.InnerText + @"""");
                                            Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                            Query.AppendFormat(" {0} ", Convert.ToString(ChildNode.InnerText));
                                        }
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    else
                                    {
                                        //Bug Fix : 4435. If value for field is not provided, set it to NULL. 
                                        Query.AppendFormat("{0}{1} = NULL ", Seperator, ChildNode.Name);
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    break;

                                case "DATETIME":
                                    DayNode     = ChildNode.SelectSingleNode("DAY");
                                    MonthNode   = ChildNode.SelectSingleNode("MONTH");
                                    YearNode    = ChildNode.SelectSingleNode("YEAR");
                                    HourNode    = ChildNode.SelectSingleNode("HOUR");
                                    MinuteNode  = ChildNode.SelectSingleNode("MINUTE");
                                    SecondNode  = ChildNode.SelectSingleNode("SECOND");

                                    if (MonthNode != null && DayNode != null && YearNode != null)
                                    {
                                        m_parameter = new DynamicSQLParameter();
                                        m_parameter.m_ParameterName = GetNextParameterizedVariable();
                                        m_parameter.m_ParameterDataType = "DATETIME";

                                        if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                                        {
                                            DateString.AppendFormat("{0}/{1}/{2} {3}:{4}:{5}",
                                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText,
                                                        HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);

                                            Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                            Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);

                                            m_parameter.m_ParameterValue = ConvertDate(DateString.ToString());
                                        }
                                        else
                                        {
                                            DateString.AppendFormat("{0}/{1}/{2}",
                                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                                            Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                            Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);

                                            m_parameter.m_ParameterValue = ConvertDate(DateString.ToString());
                                        }
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);
                                        DateString.Remove(0, DateString.Length);
                                    }
                                    else
                                    {
                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat("{0} ", "null");
                                    }
                                    /*Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                    Query.AppendFormat("{0} ", ConvertDate(DateString.ToString()));
                                    DateString.Remove(0, DateString.Length);*/
                                    FlagIsFieldOtherThanXML = true;
                                    break;
                                //For CURRENCY data type support. Code added on date 14th June 2006 by Pallavi
                                case "CURRENCY":
                                    TypeNode = ChildNode.SelectSingleNode("CURRENCY");
                                    ValueNode = ChildNode.SelectSingleNode("AMOUNT");
                                    if (TypeNode != null && ValueNode != null)
                                    {
                                        if (TypeNode.InnerText == "")
                                        {
                                            CurrencyString.AppendFormat("{0}_C =N'{1}',", ChildNode.Name, "NULL");
                                        }
                                        else
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TypeNode.InnerText + "'", "NVARCHAR(25)");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);

                                            CurrencyString.AppendFormat("{0}_C = {1} ,", ChildNode.Name, m_parameter.m_ParameterName);
                                        }

                                        if (ValueNode.InnerText == "")
                                        {
                                            CurrencyString.AppendFormat("{0}_A={1}", ChildNode.Name, "NULL");
                                        }
                                        else
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), ValueNode.InnerText, "MONEY");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);

                                            CurrencyString.AppendFormat("{0}_A= {1} ", ChildNode.Name, m_parameter.m_ParameterName);
                                        }
                                        Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }
                                    else if (TypeNode != null & ValueNode == null)
                                    {
                                        if (TypeNode.InnerText == "")
                                        {
                                            CurrencyString.AppendFormat("{0}_C =N'{1}',{2}_A={3}", ChildNode.Name, "NULL", ChildNode.Name, "NULL");
                                        }
                                        else
                                        {
                                            m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + TypeNode.InnerText + "'", "NVARCHAR(25)");
                                            if (m_DeferQueryExecution)
                                                DynamicParameters_Defer.Add(m_parameter);
                                            else
                                                DynamicParameters.Add(m_parameter);

                                            CurrencyString.AppendFormat("{0}_C ={1},{2}_A={3}", ChildNode.Name, TypeNode.InnerText, ChildNode.Name, "NULL");
                                        }
                                        Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }
                                    else
                                    {
                                        CurrencyString.AppendFormat("{0}_C =N'{1}',{2}_A={3}", ChildNode.Name, "NULL", ChildNode.Name, "NULL");
                                        Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    }

                                    CurrencyString.Remove(0, CurrencyString.Length);
                                    FlagIsFieldOtherThanXML = true;
                                    break;

                                case "TEXT":
                                    if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
                                    {
                                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "N'" + ReplaceSingleQuote(ChildNode.InnerText) + "'", "NVARCHAR(MAX)");
                                        if (m_DeferQueryExecution)
                                            DynamicParameters_Defer.Add(m_parameter);
                                        else
                                            DynamicParameters.Add(m_parameter);

                                        Query.AppendFormat("{0}{1} = ", Seperator, ChildNode.Name);
                                        Query.AppendFormat(" {0} ", m_parameter.m_ParameterName);
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    else
                                    {
                                        Query.AppendFormat("{0}{1} = N''", Seperator, ChildNode.Name);
                                        FlagIsFieldOtherThanXML = true;
                                    }
                                    break;
                            }
                            Seperator = ",";
                        }
                        //If column data type is XML then create an XML file and add XML data into the
                        //newly created file by using XML manager. This is for XML data type.
                        else if ((DataType.ToUpper() == "XML") && (ObjectTypeName != null) && (PrimaryKeyValue != 0))
                            ErrorObj = ObjXMLManager.Save(ObjectTypeName, PrimaryKeyValue, ChildNode.Name.Trim().ToUpper(), ChildNode.InnerXml);
                    }
                }

                if (FlagIsFieldOtherThanXML == false)
                {
                    //No reader is used here
                    if (DataReaderObj != null)
                        DataReaderObj.Close();
                    m_Clause = "";
                    m_SortClause = "";
                    m_ClauseForCache = "";
                    m_SortClauseForCache = "";
                    m_ClauseForDataTables = "";
                    return (null);
                }

                if ((m_Clause != "") && (m_Clause != null))
                    Query.AppendFormat(" WHERE {0}", m_Clause);
                
                //Store deferred queries if query execution is to be deferred
                if (m_DeferQueryExecution)
                {
                    sl_DeferredQueries.Add(Query.ToString());
                }
                else
                {
                    ErrorObj = Execute(Query.ToString(), ref RowsAffected);
                    if (RowsAffected == 0)
                    {
                        ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                               "No records found for updation", OMErrorObjectType.TABLE,
                               null, OMErrorType.ERROR, null, null);
                    }
                }

                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";

                //No reader used over here
                if (DataReaderObj != null)                
                    DataReaderObj.Close();
              
                //UpdateThe Cache
                if (ErrorObj == null)
                {
                    RemoveFromCache(TableName.ToUpper());
                }
            }
            catch (Exception ExceptionObj)
            {
                //No reader used over here
                if (DataReaderObj != null)
                    DataReaderObj.Close();
                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";
                ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            return (ErrorObj);
        }

        virtual public OMError UpdateRecords(BatchUpdateInputParam BatchUpdateInputParamObj, out BatchUpdateOutputParam BatchUpdateOutputParamObj)
        {
            BatchUpdateOutputParamObj = null;
            return null;
        }

		/// <summary>
		/// This function is used to delete the records from a given table.
		/// </summary>
		/// <param name="TableName">This specifies the table name from which 
		/// the records have to be deleted
		/// </param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		virtual public OMError DeleteRecord(string TableName)
		{
			OMError         ErrorObj        = null;
            StringBuilder   Query           = null;
            long            RowsAffected    = 0;

			try
			{
                TableName = TableName.ToUpper();

                //Delete the XML data (files) for XML fields for each object.
                //Method added on date 9th March 2006 by Kiran Abhang.
                ErrorObj = DeleteAllXMLFilesForObject(TableName);
                if (ErrorObj != null)
                    return ErrorObj;

                //Delete the records from table.
                Query = new StringBuilder();
				Query.AppendFormat("DELETE FROM {0}", TableName);
				if ((m_Clause != "") && (m_Clause != null))
					Query.AppendFormat(" WHERE {0}", m_Clause);
				
                //Store deferred queries if query execution is to be deferred
                if (m_DeferQueryExecution)
                {
                    sl_DeferredQueries.Add(Query.ToString());
                }
                else
                {
                    ErrorObj = Execute(Query.ToString(), ref RowsAffected);
                    if (ErrorObj != null)
                    {
                        ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                               "Error while deletion of records", OMErrorObjectType.TABLE,
                               null, OMErrorType.ERROR, null, null);
                    }
                }
				m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";

                //UpdateThe Cache
                if (ErrorObj == null)
                {
                    RemoveFromCache(TableName.ToUpper());
                }
			}
			catch (Exception ExceptionObj)
			{
				m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";
				ErrorObj = new OMError(OMErrorCodes.DELETE_RECORD_FAILED,
						   ExceptionObj.Message, OMErrorObjectType.TABLE,
						   null, OMErrorType.EXCEPTION, ExceptionObj, null); 
			}
            return (ErrorObj);
		}

        /// <summary>
        /// This function is used to delete the records from a given table.
        /// </summary>
        /// <param name="TableName">This specifies the table name from which 
        /// the records have to be deleted
        /// </param>
        /// <param name="DeleteQuery">This specifies the batch of delete queries which will
        /// be executed on the database.
        /// </param>
        /// <returns>
        /// This function returns OMError object if there is any error occured.
        /// </returns>
        virtual public OMError PrepareDeleteQuery(string TableName, ref string DeleteQuery)
        {
            OMError         ErrorObj    = null;
            StringBuilder   Query       = null;

            try
            {
                TableName = TableName.ToUpper();

                //Delete the XML data (files) for XML fields for each object.
                //Method added on date 9th March 2006 by Kiran Abhang.
                ErrorObj = DeleteAllXMLFilesForObject(TableName);
                if (ErrorObj != null)
                    return ErrorObj;

                Query = new StringBuilder();
                Query.AppendFormat("DELETE FROM {0}", TableName);
                if ((m_Clause != "") && (m_Clause != null))
                    Query.AppendFormat(" WHERE {0}", m_Clause);
                
                DeleteQuery = Query.ToString();
                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";

                //UpdateThe Cache
                RemoveFromCache(TableName);
            }
            catch (Exception ExceptionObj)
            {
                m_Clause = "";
                m_SortClause = "";
                m_ClauseForCache = "";
                m_SortClauseForCache = "";
                m_ClauseForDataTables = "";
                ErrorObj = new OMError(OMErrorCodes.DELETE_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            return (ErrorObj);
        }

		/// <summary>
		/// This function is used to add the where clause part in the Read,Update,Delete query.
		/// e.g. If you want to prepare a select query of this type
		/// "Select * from Plan where PlanName='Plan%' and Description='XYZ'"
		/// The Where clause will pe prepared using the Add Clause method.
		/// The series of calls will be as follows.
		/// AddClause("PlanName", OMFieldDataType.STRING, OMOperator.LIKE, "Plan%", OMConjunction.AND);
		/// AddClause("Description", OMFieldDataType.STRING, OMOperator.EQUALTO, "XYZ", OMConjunction.NOTHING);
		/// This will populate the m_Clause with value "PlanName='Plan%' and Description='XYZ'"
		/// </summary>
		/// <param name="FieldName">The FieldName on which the clause is to be made</param>
		/// <param name="FieldDataType">The datatype of the Field</param>
		/// <param name="Operator">This is an enumarator.The operators that can be used is "=,<=,>=, LIKE etc."</param>
		/// <param name="FieldValue">The value of the Field</param>
		/// <param name="Conjunction">Whether you want "And","OR" or "NOTHING" with the next clause.Depending on the
		/// conjunction value the next clause will be "AND'ed","OR'ed". The "NOTHING" will be used for the last AddClause. 
		/// </param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		virtual public OMError AddClause(string FieldName, OMFieldDataType FieldDataType, OMOperator Operator, string FieldValue, OMConjunction Conjunction)
		{
            DynamicSQLParameter m_parameter;
			string ConjunctionValue = GetConjunctionValue(Conjunction);
			string OperatorValue = GetOperatorValue(Operator);
            if (m_ClauseForCache != null)
                if (!(m_ClauseForCache.EndsWith("AND ") || m_ClauseForCache.EndsWith("OR ")))
                {
                    m_ClauseForCache = "";
                    m_SortClauseForCache = "";
                }

			switch (FieldDataType)
			{
				case OMFieldDataType.STRING:
				case OMFieldDataType.BOOLEAN:
                    if (Operator == OMOperator.IN)
                    {
                        //Anand: Here it is the caller's responsibility to invoke ReplaceSingleQuote() for values if needed. 
                        m_Clause = m_Clause + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue + " " + FieldValue + " " + ConjunctionValue + " ";
                        //Satish: Here we are passing FieldName Without double quotes to Cache 
                        m_ClauseForCache = m_ClauseForCache + FieldName.ToUpper() + " " + OperatorValue + " " + FieldValue + " " + ConjunctionValue + " ";
                        m_ClauseForDataTables = m_ClauseForDataTables + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue + " " + FieldValue + " " + ConjunctionValue + " ";
                    }
                    else
                    {
                        m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), GetUnicodeCharacterSpecifier() + "'" + ReplaceSingleQuote(FieldValue) + "' ", "NVARCHAR(MAX)");
                        if (m_DeferQueryExecution)
                            DynamicParameters_Defer.Add(m_parameter);
                        else
                            DynamicParameters.Add(m_parameter);

                        m_Clause = m_Clause + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue +
                                    " " + m_parameter.m_ParameterName + " " + ConjunctionValue + " ";

                        m_ClauseForCache = m_ClauseForCache + FieldName.ToUpper() + " " + OperatorValue +
                                    "  '" +
                                    ReplaceSingleQuote(FieldValue) + "' " + ConjunctionValue + " ";

                        m_ClauseForDataTables = m_ClauseForDataTables + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue +
                                    " " + GetUnicodeCharacterSpecifier() + "'" + ReplaceSingleQuote(FieldValue) + "' " +
                                    ConjunctionValue + " ";
                    }
					break;
				case OMFieldDataType.DATETIME:
                    m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), ConvertDate(FieldValue), "DATETIME");
                    if (m_DeferQueryExecution)
                        DynamicParameters_Defer.Add(m_parameter);
                    else
                        DynamicParameters.Add(m_parameter);

                    m_Clause = m_Clause + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue + " " + m_parameter.m_ParameterName + "  " + ConjunctionValue + " ";
                    m_ClauseForCache = m_ClauseForCache + FieldName.ToUpper() + " " + OperatorValue + " " + ConvertDate(FieldValue) + "  " + ConjunctionValue + " ";
                    m_ClauseForDataTables = m_ClauseForDataTables + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue + " " + ConvertDate(FieldValue) + "  " + ConjunctionValue + " ";
                    break;
				case OMFieldDataType.INTEGER:
				case OMFieldDataType.DECIMAL:
                    m_parameter = new DynamicSQLParameter();
                    m_parameter.m_ParameterName = GetNextParameterizedVariable();
                    if (OperatorValue.ToUpper().Trim() == "IN" || OperatorValue.ToUpper().Trim() == "NOT IN")
                    {
                        if (FieldValue.Length > 2)
                        {
                            if (FieldValue[0] == '(')
                                FieldValue = FieldValue.Substring(1);
                            if (FieldValue[FieldValue.Length - 1] == ')')
                                FieldValue = FieldValue.Substring(0, FieldValue.Length - 1);
                        }
                        m_parameter.m_ParameterValue = "N'" + FieldValue + "'";
                        m_parameter.m_ParameterDataType = "NVARCHAR(MAX)";
                    }
                    else
                    {
                        m_parameter.m_ParameterValue = FieldValue;
                        m_parameter.m_ParameterDataType = "BIGINT";
                    }
                    if (m_DeferQueryExecution)
                        DynamicParameters_Defer.Add(m_parameter);
                    else
                        DynamicParameters.Add(m_parameter);

                    if (OperatorValue.ToUpper().Trim() == "IN" || OperatorValue.ToUpper().Trim() == "NOT IN")
                    {
                        m_Clause = m_Clause + "(" + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue + " (Select Data From dbo.Split( " + m_parameter.m_ParameterName + " ,',')))  " + ConjunctionValue + " ";
                    }
                    else
                    {
                        m_Clause = m_Clause + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue + " " + m_parameter.m_ParameterName + "  " + ConjunctionValue + " ";
                    }
                    m_ClauseForCache = m_ClauseForCache + FieldName.ToUpper() + " " + OperatorValue + FieldValue + "  " + ConjunctionValue + " ";
                    m_ClauseForDataTables = m_ClauseForDataTables + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + OperatorValue + FieldValue + "  " + ConjunctionValue + " ";
                    break;
			}
			return null;
		}

        /// <summary>
        /// This function is used to Add a bracket i.e '(' or ')' in a Where Clause.
        /// Care has to be taken from OM to check the number of brackets match and in proper order.
        /// </summary>
        /// <param name="BracketString">Opening or closing Bracket i.e '(' or ')'</param>
        /// <returns></returns>
        virtual public OMError AddBracket(String BracketString)
        {
            m_Clause = m_Clause + " " + BracketString.Trim() + " ";
            m_ClauseForDataTables = m_ClauseForDataTables + " " + BracketString.Trim() + " "; 
            return null;
        }

        /// <summary>
        /// <CreatedDate>11 May 09</CreatedDate>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// <purpose>Adds specified conjunction in clause (including clause for cache).</purpose>
        /// </summary>
        /// <param name="Conjunction">Conjunction to be added to clause.</param>
        /// <returns>OMError if any, otherwise returns null.</returns>
        virtual public OMError AddConjuction(OMConjunction Conjunction)
        {
            switch (Conjunction)
            {
                case OMConjunction.AND:
                    m_Clause += " AND ";
                    m_ClauseForCache += " AND ";
                    m_ClauseForDataTables += " AND ";
                    break;

                case OMConjunction.NOTHING:
                    m_Clause += " ";
                    m_ClauseForCache += " ";
                    m_ClauseForDataTables += " ";
                    break;

                case OMConjunction.OR:
                    m_Clause += " OR ";
                    m_ClauseForCache += " OR ";
                    m_ClauseForDataTables += " OR ";
                    break;
            }
            return null;
        }

        /// <summary>
        /// This function is used to add the Order by clause part in the Read.
        /// e.g. If you want to prepare a select query of this type
        /// "Select * from Plan where PlanName='Plan%' and Description='XYZ'"
        /// The sort clause will pe prepared using the AddSortClause method.
        /// The series of calls will be as follows.
        /// AddSortClause("DISPLAYORDER", "ALOM_SCHEMA", "ASC", "");
        /// 'DISPLAYORDER' is field name of table on which user wants to apply sort clause 
        /// 'ALOM_SCHEMA' is table name 
        /// 'ASC' sorted to be in ascending order or descending order.
        /// "" if user wants to order on more then one field then just specify "," for concatination.
        /// </summary>
        /// <param name="FieldName">The FieldName on which the sort clause is to be made</param>
        /// <param name="TableName">The Tablename of the Field</param>
        /// <param name="SortType">Whether ASC (ascending) or DESC(descinding)</param>
        /// <param name="Delimiter">if user wants to order on more then one field then just specify "," for concatination.</param>
        /// </param>
        /// <returns>This function returns OMError object if there is any error occured.</returns>
        virtual public OMError AddSortClause(string FieldName,string TableName ,string SortType,string Delimiter)
        {
            if (m_SortClauseForCache != null)
                if (!(m_SortClauseForCache.EndsWith(",")))
                {
                    //m_ClauseForCache = "";
                    m_SortClauseForCache = "";
                }
            m_SortClause = m_SortClause + TableName + "." + ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()) + " " + SortType.ToUpper() + Delimiter;
            //Satish: Here we are passing FieldName Without double quotes to Cache 
            m_SortClauseForCache = m_SortClauseForCache + TableName + "." + FieldName.ToUpper() + " " + SortType.ToUpper() + Delimiter;
            return null;
        }

        /// <summary>
        /// This function is used for replacing wildcard characters with proper escape characters. The syntax of the escape characters 
        /// differs according to the database used.Hence this function is made virtual. The actual 
        /// implementation is done in the appropriate classes i.e SQLQueryProcessor, 
        /// OralceQueryProcessor etc.
        /// </summary>
        /// <param name="FieldValue">Filter Value</param>
        /// <param name="SearchOperator">Search Operator like : EQUALTO, LIKE etc</param>
        /// <returns></returns>
        virtual public String ReplaceEscapeCharacters(String FieldValue, String SearchOperator)
        {
            return null;
        }

        /// <summary>
        /// This function is used for getting a String value based on Search Operator.
        /// String values may differ according to the database used.Hence this function is made virtual. The actual 
        /// implementation is done in the appropriate classes i.e SQLQueryProcessor, TeraDataQueryProcessor 
        /// OralceQueryProcessor etc.
        /// </summary>
        /// <param name="FieldValue">Filter Value</param>
        /// <param name="SearchOperator">Search Operator like : EQUALTO, LIKE etc</param>
        /// <returns></returns>
        virtual  public String GetStringValue(String FieldValue, String Operator)
        {
            return null;
        }
        
        /// <summary>
        /// Helper function which prepares the clause based on the IsCaseSensitive value
        /// </summary>
        /// <param name="FieldName">Name of the field</param>
        /// <param name="SearchOperator">Operator value</param>
        /// <param name="FieldValue">Value of the field</param>
        /// <param name="IsCaseSensitive">IsCaseSensitive</param>
        /// <returns>Case dependent Clause</returns>
        virtual public StringBuilder GetCaseDependentClause(string FieldName, string SearchOperator, string FieldValue, string IsCaseSensitive)
        {
            return null;
        }

        virtual public OMError GetFilterString(String TableAliasName, FilterCollection TempFilterObj, ref StringBuilder FilterClause, 
                                                ref XmlNode ObjectTypeNode, bool IsINToBeOptimized)
        {
            return null;
        }


        /// <summary>
        /// This function checks whether a record is present in the given table. Filterclause is appended in the select clause
        /// If there exists a record, this method returns True else False.
        /// </summary>
        /// <param name="TableName">Database table name in which record is to be checked</param>
        /// <returns></returns>
        virtual public OMError DoesRecordExist(string TableName, ref Boolean IsRecordPresent)
        {
            OMError ErrorObj = null;		//In case there is an error in the function this object will be populated
            String OutputXML = null;
            try
            {
                if ((ErrorObj = ReadRecord(TableName, TableName, null, ref OutputXML)) != null)
                {
                    return ErrorObj;
                }
                else
                {
                    if (OutputXML != null)
                    {
                        m_Clause = null;
                        m_SortClause = null;
                        IsRecordPresent = true;
                        m_ClauseForDataTables = null;
                    }
                    else
                    {
                        m_Clause = null;
                        m_SortClause = null;
                        IsRecordPresent = false;
                        m_ClauseForDataTables = null;
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                m_Clause = null;
                m_SortClause = null;
                IsRecordPresent = false;
                m_ClauseForDataTables = null;
                ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
                return ErrorObj;
            }
        }

        
		/// <summary>
		/// This function is used to replace the single quote with double single quote.
		/// This is required because if there is a single quote in the query the database 
		/// throws an error.
		/// </summary>
		/// <param name="FieldValue">This contains the string which contains the single 
		/// quote and needs to replace with double single quote</param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		public String ReplaceSingleQuote(string FieldValue)
		{
			FieldValue = FieldValue.Replace("'", "''");
			return FieldValue;
		}

        /// <summary>
        /// This function is used to replace the single quote with double single quote.
        /// This is a virtual function. The actual implementation is in the Appropriate 
		/// database class.
        /// </summary>
        /// <param name="FieldValue">This contains the string which contains the single 
        /// quote and needs to replace with double/Four single quotes</param>
        /// <returns>This function returns the modified string value.</returns>
        virtual public String ReplaceSingleQuote(string FieldValue, Boolean ToBeUsedWithinDynamicSQL)
		{
            return null;
		}

        /// <summary>
		/// The format of data is specific to the database used.
		/// This is a virtual function. The actual implementation is in the Appropriate 
		/// database class.
		/// </summary>
		/// <param name="Date">The date to be converted in the database specific format</param>
		/// <returns></returns>
		virtual public string ConvertDate(String Date)
		{
			return "";
		}
        
		/// <summary>
		/// The execute method takes the query as input. It sets the connection object and 
		/// Transaction object to the command object and executes the query on the appropriate table
		/// </summary>
		/// <param name="Query"></param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
        public OMError Execute(string Query,ref long RecordsAffected)
        {
            //StringBuilder strSQLBatch = new StringBuilder();
            try
            {
                //The Execute method takes the Query as parameter and the query is executed.
                String strSQLBatch = ParameterizeDynamicSQL(Query, m_DeferQueryExecution);
                CommandObj.Connection       = ConnectionObj;
                CommandObj.Transaction      = TransactionObj;
                CommandObj.CommandText      = strSQLBatch;
                CommandObj.CommandTimeout   = 180;
                RecordsAffected             = CommandObj.ExecuteNonQuery();
                return null;
            }
            catch (Exception ExceptionObj)
            {
                string ErrorXML = "MethodName: public OMError Execute(string Query)" + "InputXML:" + Query;
                OMError ErrorObj = new OMError(OMErrorCodes.GENERALDB_FAILURE,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, ErrorXML);
                //ExceptionTracing.LogError("Assetlink", ErrorObj);
                //if (DataManager.TraceEnabled == true)
                //    ExceptionTracing.TraceLog(ErrorXML, "Entity","Assetlink");
                return ErrorObj;
            }
        }

        /// <summary>
        /// The execute method takes the query as input. It sets the connection object and 
        /// Transaction object to the command object and executes the query on the appropriate table
        /// </summary>
        /// <param name="Query"></param>
        /// <returns>This function returns OMError object if there is any error occured.</returns>
        virtual public OMError Execute(string Query)
        {
            //StringBuilder strSQLBatch = new StringBuilder();
            try
            {
                //The Execute method takes the Query as parameter and the query is executed.
                String strSQLBatch = ParameterizeDynamicSQL(Query, false);
                CommandObj.Connection       = ConnectionObj;
                CommandObj.Transaction      = TransactionObj;
                CommandObj.CommandText      = strSQLBatch;
                CommandObj.CommandTimeout   = 180;
                int rowsAffected            = CommandObj.ExecuteNonQuery();
                m_ClauseForCache            = null;
                m_SortClauseForCache        = null;
                return null;
            }
            catch (Exception ExceptionObj)
            {
                string ErrorXML = "MethodName: public OMError Execute(string Query)" + "InputXML:" + Query;
                OMError ErrorObj = new OMError(OMErrorCodes.GENERALDB_FAILURE,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, ErrorXML);
                //ExceptionTracing.LogError("Assetlink", ErrorObj);
                //if (DataManager.TraceEnabled == true)
                //    ExceptionTracing.TraceLog(ErrorXML, "Entity","Assetlink");
                return ErrorObj;
            }
        }

        public OMError ExecuteEx(string Query)
        {
            try
            {
                CommandObj.Connection       = ConnectionObj;
                CommandObj.Transaction      = TransactionObj;
                CommandObj.CommandText      = Query;
                int rowsAffected            = CommandObj.ExecuteNonQuery();
                m_ClauseForCache            = null;
                m_SortClauseForCache = null;

                return null;
            }
            catch (Exception ExceptionObj)
            {
                string ErrorXML = "MethodName: public OMError ExecuteEx(string Query)" + "InputXML:" + Query;
                OMError ErrorObj = new OMError(OMErrorCodes.GENERALDB_FAILURE,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, ErrorXML);
                //ExceptionTracing.LogError("Assetlink", ErrorObj);
                //if (DataManager.TraceEnabled == true)
                //    ExceptionTracing.TraceLog(ErrorXML, "Entity","Assetlink");
                return ErrorObj;
            }
        }

		/// <summary>
		/// The RetrieveResult method takes the query as input. It sets the connection object and 
		/// Transaction object to the command object and executes the query on the appropriate table
		/// and returns the DataReader object, the DataReader Object contains the result returned.
		/// </summary>
		/// <param name="Query"></param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
        public OMError RetrieveResult(string Query)
        {
            //StringBuilder strSQLBatch = new StringBuilder();
            try
            {
                if (Query.ToUpper().StartsWith("SELECT"))
                {
                    DataReaderObj = ReadFromCache(Query);
                    if (DataReaderObj != null)
                    {
                        //Log("HIT:" + Query);
                        return null;
                    }

                }

                //The Execute method takes the Query as parameter and the query is executed.
                String strSQLBatch = ParameterizeDynamicSQL(Query, m_DeferQueryExecution);
                CommandObj.Connection       = ConnectionObj;
                CommandObj.Transaction      = TransactionObj;
                CommandObj.CommandText      = strSQLBatch;

                DataReaderObj = CommandObj.ExecuteReader();
                return null;
            }
            catch (Exception ExceptionObj)
            {
                OMError ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
                return ErrorObj;
            }
        }

        public String ParameterizeDynamicSQL(string Query, bool bIsExecutionDeferred)
        {
            StringBuilder strSQLBatch = new StringBuilder();
            try
            {
                //The Execute method takes the Query as parameter and the query is executed.

                StringBuilder strParameterDeclaration = new StringBuilder();
                StringBuilder strParameterValues = new StringBuilder();
                StringBuilder strParameterList = new StringBuilder();
                List<DynamicSQLParameter> lDynamicSQLParameters = new List<DynamicSQLParameter>();
                int iVariableCounter = 0;
                String sVariableName = "@Var";
                String sNewBatch = string.Empty;
                sNewBatch = Query;
                strSQLBatch.Append("DECLARE @SQLString nvarchar(MAX); DECLARE @ParmDefinition nvarchar(MAX);");
                strSQLBatch.AppendFormat(System.Environment.NewLine);

                if (bIsExecutionDeferred)
                {
                    foreach (DynamicSQLParameter oParameter in DynamicParameters_Defer)
                    {
                        if (Query.Contains(" " + oParameter.m_ParameterName + " "))
                        {
                            sVariableName = "@Var";
                            sVariableName = sVariableName + iVariableCounter.ToString();
                            lDynamicSQLParameters.Add(oParameter);
                            strSQLBatch.AppendFormat("DECLARE {0} {1};", sVariableName, oParameter.m_ParameterDataType);
                            strSQLBatch.AppendFormat(System.Environment.NewLine);

                            if (string.IsNullOrEmpty(strParameterDeclaration.ToString()))
                                strParameterDeclaration.AppendFormat("{0} {1}", sVariableName, oParameter.m_ParameterDataType);
                            else
                                strParameterDeclaration.AppendFormat(" ,{0} {1}", sVariableName, oParameter.m_ParameterDataType);

                            strParameterValues.AppendFormat("SET {0} = {1};", sVariableName, oParameter.m_ParameterValue);
                            strParameterValues.AppendFormat(System.Environment.NewLine);

                            if (string.IsNullOrEmpty(strParameterList.ToString()))
                            {
                                strParameterList.AppendFormat("{0}", sVariableName);
                            }
                            else
                            {
                                strParameterList.AppendFormat(",{0}", sVariableName);
                            }
                            iVariableCounter++;
                            if (string.IsNullOrEmpty(sNewBatch))
                                sNewBatch = Query.Replace(" " + oParameter.m_ParameterName + " ", " " + sVariableName + " ");
                            else
                                sNewBatch = sNewBatch.Replace(" " + oParameter.m_ParameterName + " ", " " + sVariableName + " ");
                        }
                    }

                    foreach (DynamicSQLParameter oParameter in lDynamicSQLParameters)
                    {
                        DynamicParameters_Defer.Remove(oParameter);
                    }
                    lDynamicSQLParameters.Clear();
                }
                else
                {
                    foreach (DynamicSQLParameter oParameter in DynamicParameters)
                    {
                        if (Query.Contains(" " + oParameter.m_ParameterName + " "))
                        {
                            sVariableName = "@Var";
                            sVariableName = sVariableName + iVariableCounter.ToString();
                            lDynamicSQLParameters.Add(oParameter);
                            strSQLBatch.AppendFormat("DECLARE {0} {1};", sVariableName, oParameter.m_ParameterDataType);
                            strSQLBatch.AppendFormat(System.Environment.NewLine);

                            if (string.IsNullOrEmpty(strParameterDeclaration.ToString()))
                                strParameterDeclaration.AppendFormat("{0} {1}", sVariableName, oParameter.m_ParameterDataType);
                            else
                                strParameterDeclaration.AppendFormat(" ,{0} {1}", sVariableName, oParameter.m_ParameterDataType);

                            strParameterValues.AppendFormat("SET {0} = {1};", sVariableName, oParameter.m_ParameterValue);
                            strParameterValues.AppendFormat(System.Environment.NewLine);

                            if (string.IsNullOrEmpty(strParameterList.ToString()))
                            {
                                strParameterList.AppendFormat("{0}", sVariableName);
                            }
                            else
                            {
                                strParameterList.AppendFormat(",{0}", sVariableName);
                            }
                            iVariableCounter++;
                            if (string.IsNullOrEmpty(sNewBatch))
                                sNewBatch = Query.Replace(" " + oParameter.m_ParameterName + " ", " " + sVariableName + " ");
                            else
                                sNewBatch = sNewBatch.Replace(" " + oParameter.m_ParameterName + " ", " " + sVariableName + " ");
                        }
                    }

                    foreach (DynamicSQLParameter oParameter in lDynamicSQLParameters)
                    {
                        DynamicParameters.Remove(oParameter);
                    }
                    lDynamicSQLParameters.Clear();
                }


                strSQLBatch.AppendFormat("SET @SQLString = N'{0}';", sNewBatch.Replace("'", "''"));
                strSQLBatch.AppendFormat(System.Environment.NewLine);
                if ((strParameterDeclaration != null) && (strParameterDeclaration.Length > 0))
                {
                    strSQLBatch.AppendFormat("SET @ParmDefinition = N'");
                    strSQLBatch.AppendFormat(strParameterDeclaration.ToString());
                    strSQLBatch.AppendFormat("'");
                }
                strSQLBatch.AppendFormat(System.Environment.NewLine);

                strSQLBatch.Append(strParameterValues);

                if (strParameterList.Length > 0)
                    strSQLBatch.AppendFormat("EXECUTE sp_executesql @SQLString, @ParmDefinition, {0}", strParameterList);
                else
                    strSQLBatch.AppendFormat("EXECUTE sp_executesql @SQLString");
            }
            catch (Exception ex)
            {
                strSQLBatch.Remove(0, strSQLBatch.Length);
            }
            return strSQLBatch.ToString();
        }
        
		string GetConjunctionValue(OMConjunction Conjunction)
		{
			string ConjunctionValue="";
			switch (Conjunction)
			{
				case OMConjunction.AND:
					ConjunctionValue = Conjunction.ToString();
					break;
				case OMConjunction.OR:
					ConjunctionValue = Conjunction.ToString();
					break;
				case OMConjunction.NOTHING:
					ConjunctionValue = "";
					break;
			}
			return ConjunctionValue;
		}
        
		string GetOperatorValue(OMOperator Operator)
		{
			string OperatorValue="";
			switch (Operator)
			{
				case OMOperator.LIKE:
					OperatorValue = " LIKE";
					break;
				case OMOperator.EQUALTO:
					OperatorValue = " =" ;
					break;
				case OMOperator.GREATERTHAN:
					OperatorValue = " >" ;
					break;
				case OMOperator.LESSTHAN:
					OperatorValue = " < " ;
					break;
				case OMOperator.GREATERTHANEQUALTO:
					OperatorValue = " >=" ;
					break;
				case OMOperator.LESSTHANEQUALTO:
					OperatorValue = " <=" ;
					break;
				case OMOperator.IN:
					OperatorValue = " IN" ;
					break;
				case OMOperator.BETWEEN:
					OperatorValue = " BETWEEN" ;
					break;
				case OMOperator.NOTEQUALTO:
					OperatorValue = " <>" ;
					break;
				case OMOperator.NOTIN:
					OperatorValue = " NOT IN" ;
					break;
				case OMOperator.NOTLIKE:
					OperatorValue = " NOT LIKE";
					break;

			}
			return OperatorValue;
		}
        
		/// <summary>
		/// This function is used to commit a transaction.
		/// </summary>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		public OMError Commit()
		{
			if (TransactionObj != null)
				TransactionObj.Commit();
			Close();
			return null;
		}
        
		/// <summary>
		/// This function is used to RollBack a transaction.
		/// </summary>
		/// 
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		public OMError RollBack()
		{
			try
			{
				if (TransactionObj != null)
					TransactionObj.Rollback();
				Close();
				return null;
			}
			catch (Exception ExceptionObj)
			{
                throw ExceptionObj;
			}
		}
				
		/// <summary>
		/// <Purpose>
		/// This function is used to execute a query.If some application wants to prepare complex 
		/// queries then the Application will create the query and pass the query to this function.
		/// Since the DAL will be accessible only to the OM hence the Application will pass the
		/// query to This function and the DAL will execute the query and return the results.
		/// /// </Purpose>
		/// </summary>
		/// <param name="Query">The query to be executed</param>
		/// <param name="ArrayList">This function returns the ArrayList which halds the IDs</param>
		/// <returns>This function returns the OMError object in case there is some error has occured</returns>
		public OMError ExecuteQuery(string Query, ref ArrayList ArrayList) 
		{ 
			try 
			{ 
				CommandObj.Connection  = ConnectionObj;
				CommandObj.Transaction = TransactionObj;
				CommandObj.CommandText = Query;

                try
                {
                    DataReaderObj = CommandObj.ExecuteReader();
                    while (DataReaderObj.Read())
                    {
                        for (int FieldCount = 0; FieldCount < DataReaderObj.FieldCount; FieldCount++)
                        {
                            ArrayList.Add(DataReaderObj.GetValue(FieldCount));
                        }
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
                m_ClauseForCache = null;
                m_SortClauseForCache = null;
				return null;
			}
			catch (Exception ExceptionObj) 
			{
				OMError ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
						   ExceptionObj.Message, OMErrorObjectType.TABLE,
						   null, OMErrorType.EXCEPTION, ExceptionObj, null);
				return ErrorObj;
			} 
		}

		/// <summary>
		/// <Purpose>
		/// This function is used to execute a query.If some application wants to prepare complex 
		/// queries then the Application will create the query and pass the query to this function.
		/// Since the DAL will be accessible only to the OM hence the Application will pass the
		/// query to This function and the DAL will execute the query and return the results.
		/// /// </Purpose>
		/// </summary>
		/// <param name="Query">The query to be executed</param>
		/// <param name="OutputXML">This function returns the Output XML which is the collection of objects</param>
		/// <returns>This function returns the OMError object in case there is some error has occured</returns>
		public OMError ExecuteQuery(string Query, string TypeName, ref string OutputXML)
		{
			XmlNode ObjectCollectionNode;	//This contains the FieldCollection node that is passed back.
			XmlElement OutputFieldNode;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
			XmlNode ObjectNode;				//This is the Type name node
			string FieldName;				//This has the field name of the individual field.This field name is used for formation of OutputXML to return the data back
			string FieldValue;				//This has the field Value of the individual field.This field value is used for formation of OutputXML to return the data back
			XmlNode HourNode;				//This contains the Hour node which specifies the Hour in the date
			XmlNode MinuteNode;				//This contains the Minute node which specifies the Minute in the date 
			XmlNode SecondNode;				//This contains the Second node which specifies the Second in the date 
			XmlNode DayNode;				//This contains the Day node which specifies the day in the date 
			XmlNode MonthNode;				//This contains the Month node which specifies the Month in the date 
			XmlNode YearNode;				//This contains the Year node which specifies the Year in the date 
			DateTime DateValue;
			string FieldType = "";			//This has the field Data Type of the individual field.This field type is used for formation of OutputXML to return the data back

			try
			{
				CommandObj.Connection = ConnectionObj;
				CommandObj.Transaction = TransactionObj;
				CommandObj.CommandText = Query;
                try
                {
                    DataReaderObj = ReadFromCache(Query);
                    if (DataReaderObj == null)
                        DataReaderObj = CommandObj.ExecuteReader();

                    XmlDocument OutputXMLDocument = new XmlDocument();
                    ObjectCollectionNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName + "COLLECTION", "");
                    OutputXMLDocument.AppendChild(ObjectCollectionNode);
                    while (DataReaderObj.Read())
                    {
                        ObjectNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName, "");
                        for (int FieldCount = 0; FieldCount < DataReaderObj.FieldCount; FieldCount++)
                        {
                            FieldName = DataReaderObj.GetName(FieldCount);
                            FieldValue = DataReaderObj.GetValue(FieldCount).ToString();
                            FieldType = DataReaderObj.GetFieldType(FieldCount).ToString();

                            OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                            OutputFieldNode.InnerText = "";

                            if ((FieldValue != null) && (FieldValue != ""))
                            {
                                switch (FieldType)
                                {
                                    case "System.DateTime":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        DayNode = OutputXMLDocument.CreateElement("DAY");
                                        DateValue = Convert.ToDateTime(FieldValue);
                                        DayNode.InnerText = DateValue.Day.ToString();
                                        OutputFieldNode.AppendChild(DayNode);
                                        MonthNode = OutputXMLDocument.CreateElement("MONTH");
                                        MonthNode.InnerText = DateValue.Month.ToString();
                                        OutputFieldNode.AppendChild(MonthNode);
                                        YearNode = OutputXMLDocument.CreateElement("YEAR");
                                        YearNode.InnerText = DateValue.Year.ToString();
                                        OutputFieldNode.AppendChild(YearNode);
                                        if (DateValue.Hour != 0)
                                        {
                                            HourNode = OutputXMLDocument.CreateElement("HOUR");
                                            HourNode.InnerText = DateValue.Hour.ToString();
                                            OutputFieldNode.AppendChild(HourNode);
                                            MinuteNode = OutputXMLDocument.CreateElement("MINUTE");
                                            MinuteNode.InnerText = DateValue.Minute.ToString();
                                            OutputFieldNode.AppendChild(MinuteNode);
                                            SecondNode = OutputXMLDocument.CreateElement("SECOND");
                                            SecondNode.InnerText = DateValue.Second.ToString();
                                            OutputFieldNode.AppendChild(SecondNode);
                                        }
                                        break;
                                    default:
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerText = FieldValue;
                                        break;
                                }
                            }
                            ObjectNode.AppendChild(OutputFieldNode);
                        }
                        ObjectCollectionNode.AppendChild(ObjectNode);
                    }
                    OutputXML = ObjectCollectionNode.OuterXml;
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }

                m_ClauseForCache = null;
                m_SortClauseForCache = null;

				return null;
			}
			catch (Exception ExceptionObj)
			{
				OMError ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
						   ExceptionObj.Message, OMErrorObjectType.TABLE,
						   null, OMErrorType.EXCEPTION, ExceptionObj, null);
				return ErrorObj;
			}
		}

        virtual public String GetDataTypesAndTheirSizes()
        {
            return null;
        }

        public void Close()
        {
            if (ConnectionObj.State == ConnectionState.Open)
            {
                CommandObj.Dispose();
                ConnectionObj.Close();
                ConnectionObj.Dispose();
                if(TransactionObj != null)
                    TransactionObj.Dispose();
            }
        }

		/// <summary>
		/// This function is used to truncate the data. If the database field size is 255 and the data passes 
		/// is more than the field size the remaining text will be truncated.
		/// </summary>
		/// <param name="FieldValue">The field value that is passed</param>
		/// <param name="Length">The database field size</param>
		/// <returns>This function returns the Truncated string</returns>
		public String TruncateData(string FieldValue, int Length)
		{
			if(FieldValue.Length > Length)
			{
				string TruncatedFieldValue = FieldValue.Substring(0, Length);
				return TruncatedFieldValue;
			}
			else
			{
				return FieldValue;
			}
		}

        public virtual OMError ExecuteStoredProcedureForRead(String ProcedureName)
        {
            return null;
        }

        public void AddParameter(String ParameterName, String ParameterValue)
        {
            if (ParamDictionary == null)
                ParamDictionary = new StringDictionary();

            ParameterName = "@" + ParameterName;
            ParamDictionary.Add(ParameterName, ParameterValue);
        }

        private OMError GetPrimaryKeyAndObjectType(String TableName, out String PrimaryKeyFieldName,
                                            out String ObjectType, out ulong ObjectTypeID)
        {
            OMError                 ErrorObj            = null;
            String                  FieldListXML        = null;
            String                  OutputXML           = null;
            XPathDocument           Xp_FieldXMLDocument = null;
            XPathNavigator          XpNav_FieldNav      = null;
            XPathNodeIterator       XpNodeItr           = null;
            String                  previousm_Clause    = null;
            String                  previousm_ClauseForDataTables = null;


            //Initialize the 'out' parameters.
            PrimaryKeyFieldName = null;
            ObjectType = null;
            ObjectTypeID = 0;

            try
            {
                FieldListXML = @" <FIELDCOLLECTION>
                                        <FIELD>
                                            <FIELDNAME>OBJECTTYPEID</FIELDNAME>
                                        </FIELD>
                                        <FIELD>
                                            <FIELDNAME>OBJECTTYPE</FIELDNAME>
                                        </FIELD>
                                        <FIELD>
                                            <FIELDNAME>FIELDNAME</FIELDNAME>
                                        </FIELD>
                                    </FIELDCOLLECTION>";

                // m_Clause possibly already has value set by caller.This value may not have been used yet
                // To avoid problems arising from this, the value is being preserved by storing it before calling AddClause
                // and restoring it after calling ReadRecord/ReadAllRecord.
                previousm_Clause = m_Clause;
                previousm_ClauseForDataTables = m_ClauseForDataTables;
                m_Clause = null;
                m_ClauseForDataTables = null;


                if ((ErrorObj = AddClause("TABLENAME", OMFieldDataType.STRING, OMOperator.EQUALTO, TableName, OMConjunction.AND)) == null)
                {
                    if ((ErrorObj = AddClause("ISPRIMARYKEY", OMFieldDataType.STRING, OMOperator.EQUALTO, "TRUE", OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = ReadRecord(TableName, "ALOM_SCHEMA", FieldListXML, ref OutputXML)) != null)
                        {
                            return ErrorObj;
                        }
                    }
                    else
                    {
                        return ErrorObj;
                    }
                }
                else
                {
                    return ErrorObj;
                }

                m_Clause = previousm_Clause; // restore value
                m_ClauseForDataTables = previousm_ClauseForDataTables;


                if (OutputXML != null)
                {
                    Xp_FieldXMLDocument = new XPathDocument(new StringReader(OutputXML));
                    XpNav_FieldNav = Xp_FieldXMLDocument.CreateNavigator();
                    XpNodeItr = XpNav_FieldNav.Select(TableName);

                    //Here retrieves values of the fiels from the output XML                    
                    while (XpNodeItr.MoveNext())
                    {
                        ObjectTypeID = Convert.ToUInt64(XpNodeItr.Current.SelectSingleNode("OBJECTTYPEID").Value);
                        ObjectType = XpNodeItr.Current.SelectSingleNode("OBJECTTYPE").Value;
                        PrimaryKeyFieldName = XpNodeItr.Current.SelectSingleNode("FIELDNAME").Value;

                    }

                }

            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ErrorObj);
        }

        protected OMError GetObjectTypeName(ulong ObjectTypeID, String ObjectType, out String ObjectTypeName)
        {
            OMError                 ErrorObj            = null;
            String                  TableName           = null;
            String                  FieldName           = null;
            String                  FilterField         = null;
            String                  FieldListXML        = null;
            String                  OutputXML           = null;
            String                  previousm_Clause    = null;
            String                  previousm_ClauseForDataTables = null;
            XPathDocument           Xp_FieldXMLDocument = null;
            XPathNavigator          XpNav_FieldNav      = null;


            //Initialize the 'out' parameter.
            ObjectTypeName = null;

            try
            {
                TableName = "ALOM_" + ObjectType + "TYPE";
                FieldName = ObjectType + "TYPENAME";
                FilterField = ObjectType + "TYPEID";
                OutputXML = null;

                if (ObjectType.Trim().ToUpper() == "ENTITY" || ObjectType.Trim().ToUpper() == "FACET")
                {

                    FieldListXML = @" <FIELDCOLLECTION>
                                        <FIELD>
                                            <FIELDNAME>" + ObjectType + @"TYPENAME</FIELDNAME>
                                        </FIELD>
                                    </FIELDCOLLECTION>";

                    // m_Clause possibly already has value set by caller from OM. This value may not have been used yet.
                    // To avoid problems arising from this, the value is being preserved by storing it before calling AddClause
                    // and restoring it after calling ReadRecord/ReadAllRecord.
                    previousm_Clause = m_Clause;
                    previousm_ClauseForDataTables = m_ClauseForDataTables;
                    m_Clause = null;
                    m_ClauseForDataTables = null;
                    if ((ErrorObj = AddClause(FilterField, OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectTypeID.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = ReadRecord(TableName, TableName, FieldListXML, ref OutputXML)) != null)
                        {
                            return ErrorObj;
                        }

                    }
                    else
                    {
                        return ErrorObj;
                    }

                    m_Clause = previousm_Clause;    // restore value of m_Clause
                    m_ClauseForDataTables = previousm_ClauseForDataTables;

                    if (OutputXML != null)
                    {
                        Xp_FieldXMLDocument = new XPathDocument(new StringReader(OutputXML));
                        XpNav_FieldNav = Xp_FieldXMLDocument.CreateNavigator();
                        ObjectTypeName = XpNav_FieldNav.SelectSingleNode(TableName + "/" + ObjectType + "TYPENAME").Value;
                    }
                }

            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ErrorObj);
        }

        ////////private void DeleteXMLFile(String TableName)
        ////////{
        ////////    OMError         ErrorObj            = null;
        ////////    XMLManager      ObjXMLManager       = null;
        ////////    String          ObjectType          = null;
        ////////    String          ObjectTypeName      = null;
        ////////    String          PrimaryKeyFieldName = null;
        ////////    String[]        ArrXMLFields        = null;
        ////////    String[]        ArrObjectIDs        = null;
        ////////    StringBuilder   XMLFieldList        = null;
        ////////    StringBuilder   ObjectIDsList       = null;
        ////////    ulong           ObjectTypeID        = 0;

        ////////    try
        ////////    {
        ////////        //Following code is added to delete XML files of XML data type fields.
        ////////        //Code is added by Kiran Abhang on date 09th March 2006.
        ////////        //Get the Primary Key and XML fields of given table from ALOM_SCHEMA table.
        ////////        ErrorObj = GetFieldsOfTypeXML(TableName, out XMLFieldList);
        ////////        if ((ErrorObj == null) && (XMLFieldList != null) && (XMLFieldList.Length > 0))
        ////////        {
        ////////            ArrXMLFields = XMLFieldList.ToString().Split(',');
        ////////            ErrorObj = GetPrimaryKeyAndObjectType(TableName, out PrimaryKeyFieldName,
        ////////                                        out ObjectType, out ObjectTypeID);
        ////////            if (ErrorObj == null)
        ////////            {
        ////////                ErrorObj = GetObjectTypeName(ObjectTypeID, ObjectType, out ObjectTypeName);
        ////////                if (ErrorObj == null)
        ////////                {
        ////////                    ErrorObj = GetObjectIDs(TableName, PrimaryKeyFieldName, out ObjectIDsList);
        ////////                    if ((ErrorObj == null) && (ObjectIDsList != null) && (ObjectIDsList.Length > 0))
        ////////                        ArrObjectIDs = (ObjectIDsList.ToString().Split(','));
        ////////                }
        ////////            }
        ////////        }

        ////////        if ((ArrXMLFields != null) && (ArrObjectIDs != null) 
        ////////                && (ArrXMLFields.Length > 0) && (ArrObjectIDs.Length > 0))
        ////////        {
        ////////            ObjXMLManager = new XMLManager(DataManager.m_NameOfTenant);
        ////////            for (int ArrCount = 0; ArrCount < ArrXMLFields.Length; ArrCount++)
        ////////            {
        ////////                ObjXMLManager.DeleteXML(ObjectTypeName, ArrObjectIDs, ArrXMLFields[ArrCount].ToString().Trim());
        ////////            }
        ////////        }
        ////////    }
        ////////    catch (Exception ExceptionObj)
        ////////    {
        ////////        throw (ExceptionObj);
        ////////    }
        ////////}

        //Code is added by Kiran Abhang on date 09th March 2006.
        private OMError DeleteAllXMLFilesForObject(String TableName)
        {
            OMError         ErrorObj            = null;
            IXMLRepository  ObjXMLManager       = null;
            String          ObjectType          = null;
            String          ObjectTypeName      = null;
            String          PrimaryKeyFieldName = null;
            String[]        ArrObjectIDs        = null;
            StringBuilder   ObjectIDsList       = null;
            StringBuilder   XMLFieldList        = null;
            ulong           ObjectTypeID        = 0;

            try
            {
                ErrorObj = GetFieldsOfTypeXML(TableName, out XMLFieldList);
                //If XML field data type column present for an object.
                if ((ErrorObj == null) && (XMLFieldList != null) && (XMLFieldList.Length > 0))
                {
                    ErrorObj = GetPrimaryKeyAndObjectType(TableName, out PrimaryKeyFieldName,
                                                out ObjectType, out ObjectTypeID);
                    if (ErrorObj == null)
                    {
                        ErrorObj = GetObjectTypeName(ObjectTypeID, ObjectType, out ObjectTypeName);
                        if (ErrorObj == null)
                        {
                            ErrorObj = GetObjectIDs(TableName, PrimaryKeyFieldName, out ObjectIDsList);
                            if ((ErrorObj == null) && (ObjectIDsList != null) && (ObjectIDsList.Length > 0))
                                ArrObjectIDs = (ObjectIDsList.ToString().Split(','));
                        }
                    }

                    if ((ArrObjectIDs != null) && (ArrObjectIDs.Length > 0))
                    {
                        ObjXMLManager = OMDataManager.XMLRepository;
                        for (int ArrCount = 0; ArrCount < ArrObjectIDs.Length; ArrCount++)
                        {
                            ErrorObj = ObjXMLManager.DeleteXML(ObjectTypeName, Convert.ToUInt64(ArrObjectIDs[ArrCount]));
                            if (ErrorObj != null)
                                break;
                        }
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        ////////private void DeleteXSDForXMLFields(String TableName)
        ////////{
        ////////    OMError         ErrorObj            = null;
        ////////    XMLManager      ObjXMLManager       = null;
        ////////    String          ObjectType          = null;
        ////////    String          ObjectTypeName      = null;
        ////////    String          PrimaryKeyFieldName = null;
        ////////    String[]        ArrXMLFields        = null;
        ////////    StringBuilder   XMLFieldList        = null;
        ////////    ulong           ObjectTypeID        = 0;

        ////////    try
        ////////    {
        ////////        //Following code is added to delete XSD files of XML data type fields.
        ////////        //Code is added by Kiran Abhang on date 09th March 2006.
        ////////        ErrorObj = GetFieldsOfTypeXML(TableName, out XMLFieldList);
        ////////        if ((ErrorObj == null) && (XMLFieldList != null) && (XMLFieldList.Length > 0))
        ////////        {
        ////////            ArrXMLFields = XMLFieldList.ToString().Split(',');
        ////////            ErrorObj = GetPrimaryKeyAndObjectType(TableName, out PrimaryKeyFieldName,
        ////////                                        out ObjectType, out ObjectTypeID);
        ////////            if (ErrorObj == null)
        ////////                ErrorObj = GetObjectTypeName(ObjectTypeID, ObjectType, out ObjectTypeName);
        ////////        }

        ////////        if ((ArrXMLFields != null) && (ArrXMLFields.Length > 0)
        ////////                    && (ObjectTypeName != null) && (ObjectTypeName.Length > 0))
        ////////        {
        ////////            ObjXMLManager = new XMLManager(DataManager.m_NameOfTenant);
        ////////            ObjXMLManager.DeleteXSD(ObjectTypeName, ArrXMLFields);
        ////////        }
        ////////    }
        ////////    catch (Exception ExceptionObj)
        ////////    {
        ////////        throw (ExceptionObj);
        ////////    }
        ////////}

        //Code is added by Kiran Abhang on date 09th March 2006.
        private OMError DeleteAllXSDsAndXMLsForObjectType(String TableName)
        {
            OMError         ErrorObj            = null;
            IXMLRepository  ObjXMLManager       = null;
            String          ObjectType          = null;
            String          ObjectTypeName      = null;
            String          PrimaryKeyFieldName = null;
            String[]        ArrXMLFields        = null;
            StringBuilder   XMLFieldList        = null;
            ulong           ObjectTypeID        = 0;

            try
            {
                ErrorObj = GetFieldsOfTypeXML(TableName, out XMLFieldList);
                //If XML field data type column present for an object type.
                if ((ErrorObj == null) && (XMLFieldList != null) && (XMLFieldList.Length > 0))
                {
                    ArrXMLFields = XMLFieldList.ToString().Split(',');
                    ErrorObj = GetPrimaryKeyAndObjectType(TableName, out PrimaryKeyFieldName,
                                                out ObjectType, out ObjectTypeID);
                    if (ErrorObj == null)
                    {
                        ErrorObj = GetObjectTypeName(ObjectTypeID, ObjectType, out ObjectTypeName);
                        if ((ErrorObj == null) && (ObjectTypeName != null) && (ObjectTypeName.Length > 0) && (ArrXMLFields.Length > 0))
                        {
                            ObjXMLManager = OMDataManager.XMLRepository;
                            //COMMENTED AS DISCUSSED AFTER CODE REVIEW
                            //ObjXMLManager.DeleteXSD(ObjectTypeName, ArrXMLFields);
                            ErrorObj = ObjXMLManager.DeleteXML(ObjectTypeName);
                        }
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        virtual public OMError GetObjectTypeSchemaDetails(String TableName, out String PrimaryKeyFieldName, out ulong ObjectTypeID,
                                            out String ObjectType, out String ObjectTypeName, out Hashtable FieldNameValueCollection)
        {

            OMError ErrorObj = null;
            String OutputXML = null;
            XPathDocument Xp_FieldXMLDocument = null;
            XPathNavigator XpNav_FieldNav = null;
            XPathNodeIterator XpNodeItr = null;
            FieldInformation FieldDataObj;
            String FieldName = null;
            String FieldType = null;
            String FieldSize = null;
            DynamicSQLParameter m_parameter = null;
            StringBuilder QuerySB = null;
            String previousm_Clause = null;
            String previousm_ClauseForDataTables = null;


            //Initialize 'out' paramter.
            PrimaryKeyFieldName = null;
            ObjectTypeID = 0;
            ObjectType = null;
            ObjectTypeName = null;
            FieldNameValueCollection = null;

            try
            {
                bool CacheEnabled = true; //Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
                if (CacheEnabled)
                {
                    #region Read From Cache
                    previousm_Clause = m_Clause;
                    previousm_ClauseForDataTables = m_ClauseForDataTables;
                    m_Clause = null;
                    m_ClauseForDataTables = null;

                    if ((ErrorObj = AddClause("TABLENAME", OMFieldDataType.STRING, OMOperator.EQUALTO, TableName, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = ReadAllRecord(TableName, "ALOM_SCHEMA", null, ref OutputXML)) != null)
                        {
                            return ErrorObj;
                        }
                        if (OutputXML != null)
                        {
                            FieldNameValueCollection = new Hashtable();
                            Xp_FieldXMLDocument = new XPathDocument(new StringReader(OutputXML));
                            XpNav_FieldNav = Xp_FieldXMLDocument.CreateNavigator();
                            XpNodeItr = XpNav_FieldNav.Select(TableName.ToUpper() + "COLLECTION/" + TableName.ToUpper());

                            //Here retrieves values of the fiels from the output XML                    
                            while (XpNodeItr.MoveNext())
                            {

                                FieldName = XpNodeItr.Current.SelectSingleNode("FIELDNAME").Value;
                                FieldType = XpNodeItr.Current.SelectSingleNode("FIELDTYPE").Value;
                                FieldSize = XpNodeItr.Current.SelectSingleNode("FIELDSIZE").Value;

                                FieldDataObj.FieldType = FieldType;
                                FieldDataObj.FieldSize = FieldSize;

                                FieldNameValueCollection.Add(FieldName.Trim(), FieldDataObj);

                                //Get the object type name, id and primary field.
                                if (XpNodeItr.Current.SelectSingleNode("ISPRIMARYKEY").Value.ToString().ToUpper() == "TRUE")
                                {
                                    PrimaryKeyFieldName = XpNodeItr.Current.SelectSingleNode("FIELDNAME").Value;
                                    ObjectTypeID = Convert.ToUInt64(XpNodeItr.Current.SelectSingleNode("OBJECTTYPEID").Value);
                                    ObjectType = XpNodeItr.Current.SelectSingleNode("OBJECTTYPE").Value;
                                }
                            }

                            //Get the name of an object type. This is for XML data type.
                            if ((ObjectType != null) && (ObjectType.Length > 0))
                            {
                                ErrorObj = GetObjectTypeName(ObjectTypeID, ObjectType, out ObjectTypeName);
                                if (ErrorObj != null)
                                    return (ErrorObj);
                            }
                        }
                    }
                    else
                    {
                        return ErrorObj;
                    }
                    m_Clause = previousm_Clause; // restore value
                    m_ClauseForDataTables = previousm_ClauseForDataTables;
                    #endregion Read From Cache
                }
                else
                {
                    #region Read from DB
                    //m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "'" + TableName.ToUpper() + "'", "NVARCHAR(50)");
                    //if (m_DeferQueryExecution)
                    //    DynamicParameters_Defer.Add(m_parameter);
                    //else
                    //    DynamicParameters.Add(m_parameter);

                    QuerySB = new StringBuilder();
                    QuerySB.Append("SELECT FIELDNAME, FIELDTYPE, FIELDSIZE, ISPRIMARYKEY, OBJECTTYPEID, OBJECTTYPE FROM ALOM_SCHEMA");
                    //QuerySB.AppendFormat(" WHERE TABLENAME = {0} ", m_parameter.m_ParameterName);
                    QuerySB.AppendFormat(" WHERE TABLENAME = {0} ", "'" + TableName.ToUpper() + "'");

                    try
                    {
                        ErrorObj = RetrieveResult(QuerySB.ToString());
                        if (ErrorObj != null)
                        {
                            m_Clause = "";
                            m_SortClause = "";
                            m_ClauseForCache = "";
                            m_SortClauseForCache = "";
                            m_ClauseForDataTables = "";
                            return (ErrorObj);
                        }

                        // Needs to change CaseInsensitiveHashCodeProvider to IEqualityComparer
                        FieldNameValueCollection = new Hashtable();//(new CaseInsensitiveHashCodeProvider(), new CaseInsensitiveComparer());
                        FieldDataObj = new FieldInformation();
                        while (DataReaderObj.Read())
                        {
                            FieldName = DataReaderObj.GetValue(0).ToString();
                            FieldType = DataReaderObj.GetValue(1).ToString();
                            FieldSize = DataReaderObj.GetValue(2).ToString();

                            FieldDataObj.FieldType = FieldType;
                            FieldDataObj.FieldSize = FieldSize;

                            FieldNameValueCollection.Add(FieldName.Trim(), FieldDataObj);

                            //Get the object type name, id and primary field.
                            if (DataReaderObj.GetValue(3).ToString().ToUpper() == "TRUE")
                            {
                                PrimaryKeyFieldName = DataReaderObj.GetValue(0).ToString().ToUpper();
                                ObjectTypeID = Convert.ToUInt64(DataReaderObj.GetValue(4).ToString());
                                ObjectType = DataReaderObj.GetValue(5).ToString().ToUpper();
                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                    finally
                    {
                        if (DataReaderObj != null && !DataReaderObj.IsClosed)
                            DataReaderObj.Close();
                    }

                    //Get the name of an object type. This is for XML data type.
                    if ((ObjectType != null) && (ObjectType.Length > 0))
                    {
                        ErrorObj = GetObjectTypeName(ObjectTypeID, ObjectType, out ObjectTypeName);
                        if (ErrorObj != null)
                            return (ErrorObj);
                    }
                    #endregion
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return ErrorObj;

        }

        protected OMError GetObjectTypeSchemaDetails(XPathDocument ObjectTypeDetailsXMLDoc, out String PrimaryKeyFieldName, out Hashtable FieldNameValueCollection)
        {
            FieldInformation    FieldDataObj;
            String              FieldName           = null;
            String              FieldType           = null;
            String              FieldSize           = null;
            XPathNavigator      ObjectTypeNode      = null;
            XPathNodeIterator   FieldNodes          = null;
          
            PrimaryKeyFieldName             = null;
            FieldNameValueCollection        = null;

            try
            {
                FieldNameValueCollection    = new Hashtable();
                FieldDataObj                = new FieldInformation();
                ObjectTypeNode              = ObjectTypeDetailsXMLDoc.CreateNavigator();
                PrimaryKeyFieldName         = ObjectTypeNode.SelectSingleNode("*/PRIMARYKEY").Value;

                FieldNodes                  = ObjectTypeNode.Select("*/FIELDCOLLECTION/FIELD");
                while (FieldNodes.MoveNext())
                {
                    FieldName = FieldNodes.Current.SelectSingleNode("FIELDNAME").Value;
                    FieldType = FieldNodes.Current.SelectSingleNode("FIELDTYPE").Value;
                    FieldSize = FieldNodes.Current.SelectSingleNode("FIELDSIZE").Value;

                    FieldDataObj.FieldType = FieldType;
                    FieldDataObj.FieldSize = FieldSize;

                    FieldNameValueCollection.Add(FieldName.Trim(), FieldDataObj);
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }


            return null;
        }

        private OMError GetFieldsOfTypeXML(String TableName, out StringBuilder XMLFieldList)
        {
            OMError ErrorObj = null;
            String Comma = "";
            StringBuilder Query = null;

            //Initialize 'out' paramter.
            XMLFieldList = null;

            try
            {
                //DynamicSQLParameter m_parameter = new DynamicSQLParameter(GetNextParameterizedVariable(), "'" + TableName + "'", "NVARCHAR(255)");
                Query = new StringBuilder();
                Query.Append("SELECT OBJECTTYPEID, OBJECTTYPE, FIELDNAME FROM ALOM_SCHEMA");
                Query.AppendFormat(" WHERE TABLENAME = {0} ", "'" + TableName + "'");
                Query.AppendFormat(" AND FIELDTYPE = 'XML'");
                /*
                if (m_DeferQueryExecution)
                {
                    DynamicParameters_Defer.Add(m_parameter);
                }
                {
                    DynamicParameters.Add(m_parameter);
                }
                 */
                try
                {
                    ErrorObj = RetrieveResult(Query.ToString());
                    if (ErrorObj == null)
                    {
                        XMLFieldList = new StringBuilder();
                        while (DataReaderObj.Read())
                        {
                            XMLFieldList.AppendFormat("{0}{1}", Comma, DataReaderObj.GetValue(2).ToString().ToUpper());
                            Comma = ",";
                        }

                    }
                }

                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ErrorObj);
        }

        private OMError GetObjectIDs(String TableName, String PrimaryKeyFieldName, out StringBuilder ObjectIDsList)
        {
            OMError         ErrorObj    = null;
            String          Comma       = "";
            StringBuilder   Query       = null;

            //Initialize 'out' paramter.
            ObjectIDsList = null;

            try
            {
                Query = new StringBuilder();
                Query.AppendFormat("SELECT {0} FROM {1} WHERE {2}", PrimaryKeyFieldName, TableName, m_Clause);

                try
                {
                    List<DynamicSQLParameter> DynamicParametersEx = new List<DynamicSQLParameter>();
                    List<DynamicSQLParameter> DynamicParameters_DeferEx = new List<DynamicSQLParameter>();
                    DynamicParametersEx = DynamicParameters.ToList();
                    DynamicParameters_DeferEx = DynamicParameters_Defer.ToList();

                    ErrorObj = RetrieveResult(Query.ToString());

                    DynamicParameters = null;
                    DynamicParameters_Defer = null;
                    DynamicParameters = DynamicParametersEx;
                    DynamicParameters_Defer = DynamicParameters_DeferEx;
                    if (ErrorObj == null)
                    {
                        ObjectIDsList = new StringBuilder();
                        while (DataReaderObj.Read())
                        {
                            ObjectIDsList.AppendFormat("{0}{1}", Comma, DataReaderObj.GetValue(0).ToString());
                            Comma = ",";
                        }
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ErrorObj);
        }
        
        /// <summary>This method is used to generate the stored procedures for data population of ODS entities/facets.
        /// This stored procedure resides into MOM database. The stored procedure collects the necessary instances from MOM database
        /// and will then populate into ODS database.
        /// This method also generates the stored procedure to manage the relations of populated objects. The second stored procedure resides
        /// into ODS database. Backoffice service will run these stored procedures sequenctially using StoredProcedures.xml configuration.
        /// </summary>
        /// <Created By>Kiran Abhang</Created>
        /// <Creaded On>16th May 2011</Creaded>
        /// <param name="ODSObjectTypeName">Type of ODS object type for which data population stored procedures
        /// will be created into MOM and ODS databases.</param>        
        /// <param name="MOMAndODSSpecificInfoXML">This contains the information which requires to generate the stored procedure.
        /// It contains information like ODS and MOM databases names, name of the stored procedures, ODS object type information
        /// and ODS specific target object type information which are used into MOM.</param>
        /// <param name="MOMObjectTypeAndSchemaInfoXML">It contains the schema information for MOM specific object type
        /// which needs to be created into ODS database too with ODS conventions..</param>
        /// <param name="MOMRelationshipCollectionXML">It contains the relevant relationship details which are related to the MOM object type.
        /// The MOM object type is also expected to be created into ODS database with ODS conventions.</param>
        /// <param param name="ODSRelationshipCollectionXML">It contains the relevant relationship details
        /// which are related to the new ODS object type.</param>
        /// <param name="ODSDataPopulationStoredProcedure">The newly created stored procedure which resides into MOM database.
        /// This stored procedure is used to populate the relevant data from MOM to ODS database.</param>
        /// <param name="ODSManageRelationsStoredProcedure">The newly created stored procedure which resides into ODS database.
        /// This stored procedure is used to manage the relations of the populated objects into ODS database.</param>
        /// <returns>On success it will return NULL else an error object will be returned with relevant information.</returns>
        virtual public OMError PrepareODSPopulationStoredProcedures(String ODSObjectTypeName, 
                String MOMAndODSSpecificInfoXML, String MOMObjectTypeAndSchemaInfoXML,
                String MOMRelationshipCollectionXML, String ODSRelationshipCollectionXML,
                ref String ODSDataPopulationStoredProcedure, ref String ODSManageRelationsStoredProcedure)
        {
            return (null);
        }

        /// <summary>
        /// This method is used to generate the queries to drop the stored procedures from MOM and ODS databases.
        /// These stored procedures are used for ODS data population and for the relationship (assembly and assoentityfacet) management.
        /// </summary>
        /// <Created By>Kiran Abhang</Created>
        /// <Creaded On>16th May 2011</Creaded>
        /// <param name="MOMAndODSSpecificInfoXML">This contains the name of stored procedures which needs to be deleted from MOM
        /// and ODS databases. It also contains the name of the databases which are used while creating the stored procedures.</param>
        /// <param name="DeleteDataPopulationQuery">This contains the query to delete/drop the stored procedure
        /// which is used for ODS data population.</param>
        /// <param name="DeleteManageRelationsQuery">This contains the query to delete/drop the stored procedure 
        /// which is used for managing the relations (assembly and assoentityfacet) for ODS objects.</param>
        /// <returns>On success it will return NULL else an error object will be returned with relevant information.</returns>
        virtual public OMError PrepareQueryToDropODSStoredProcedures(String MOMAndODSSpecificInfoXML,
                ref String DeleteDataPopulationQuery, ref String DeleteManageRelationsQuery)
        {
            return (null);
        }

        #endregion

        #region Methods For Cache
        /// <summary>
        /// 
        /// </summary>
        /// <param name="TypeName"></param>
        /// <param name="TableName"></param>
        /// <param name="OutputXML"></param>
        /// <returns></returns>
        public OMError ReadFromCache(bool isCollection,string cachedItemName, string TypeName,
                                         string FieldListXML, ref string OutputXML)
        {
            DataTable       SystemTable         = null;
            ListDictionary  FieldList           = null;
           
            DataRow[]       rows                = null;
            OMError         ErrorObj            = null;
            List<DynamicSQLParameter> lDynamicSQLParameters = new List<DynamicSQLParameter>();

            try
            {
                //Read from Cache
                SystemTable = CacheManager.GetCacheItem(cachedItemName) as DataTable;
                if (SystemTable == null)    //If its not there in Cache then return Error
                {
                    return new OMError();
                }

                if ((ErrorObj = getFilteredRows(FieldListXML, SystemTable, ref FieldList, ref rows)) != null)
                    return ErrorObj;


                if (rows.Length > 0)
                {
                    prepareXMLForCacheTable(isCollection, TypeName, ref OutputXML, SystemTable, FieldList, rows);
                }

                m_Clause = null;
                m_SortClause = null;
                m_ClauseForDataTables = null;
                //DynamicParameters.Clear();
                if (!string.IsNullOrEmpty(m_Clause))
                {
                    foreach (DynamicSQLParameter oParameter in DynamicParameters)
                    {
                        if (m_Clause.Contains(" " + oParameter.m_ParameterName + " "))
                        {
                            lDynamicSQLParameters.Add(oParameter);
                        }
                    }
                }
                foreach (DynamicSQLParameter oParameter in lDynamicSQLParameters)
                {
                    DynamicParameters.Remove(oParameter);
                }
                lDynamicSQLParameters.Clear();
                return null;
            }
            catch
            {
                return new OMError();
            }
        }

        private void prepareXMLForCacheTable(bool isCollection,string TypeName, ref string OutputXML, 
            DataTable SystemTable, ListDictionary FieldList,DataRow[] rows)
        {
            XmlTextWriter   xmlWriter           = null;
            StringWriter    xmlContent          = null;
            char            QUOTE_CHAR          = '\"';
            int             INDENTATION_LEVEL   = 2;
            String          FieldType           = null;
            String          FieldName           = null;


            xmlContent  = new StringWriter();
            xmlWriter   = new XmlTextWriter(xmlContent);
            xmlWriter.QuoteChar     = QUOTE_CHAR;
            xmlWriter.Formatting    = Formatting.Indented;
            xmlWriter.Indentation   = INDENTATION_LEVEL;

            xmlWriter.WriteStartDocument();

            if(isCollection == true)
                xmlWriter.WriteStartElement(TypeName.ToUpper() + "COLLECTION");

            if (FieldList == null)
            {
                foreach (DataRow row in rows)
                {
                    xmlWriter.WriteStartElement(TypeName.ToUpper());
                    foreach (DataColumn col in SystemTable.Columns)
                    {
                        FieldType = col.DataType.ToString();
                        FieldName = col.ColumnName;
                        object FieldValue = row[FieldName];
                        addFieldToXML(xmlWriter, FieldType, FieldName, FieldValue);
                    }

                    xmlWriter.WriteEndElement();

                    if (isCollection == false)
                        break;
                }

                if (isCollection == true)
                    xmlWriter.WriteEndElement();
                xmlWriter.WriteEndDocument();
                OutputXML = xmlContent.ToString();
                //Remove the UTF-16 tag
                if(OutputXML.IndexOf('>') > 0)
                    OutputXML = OutputXML.Substring(OutputXML.IndexOf('>') + 3, OutputXML.Length - OutputXML.IndexOf('\n') - 1);
                xmlWriter.Close();
            }
            else
            {
                foreach (DataRow row in rows)
                {
                    xmlWriter.WriteStartElement(TypeName.ToUpper());
                    foreach (String colName in FieldList.Values)
                    {
                        DataColumn col = SystemTable.Columns[colName];
                        FieldType = col.DataType.ToString();
                        FieldName = col.ColumnName;
                        object FieldValue = row[FieldName];
                        addFieldToXML(xmlWriter, FieldType, FieldName, FieldValue);
                    }

                    xmlWriter.WriteEndElement();

                    if (isCollection == false)
                        break;
                }
                if (isCollection == true)
                    xmlWriter.WriteEndElement();

                xmlWriter.WriteEndDocument();
                OutputXML = xmlContent.ToString();
                //Remove the UTF-16 tag
                if (OutputXML.IndexOf('>') > 0)
                    OutputXML = OutputXML.Substring(OutputXML.IndexOf('>') + 3, OutputXML.Length - OutputXML.IndexOf('\n') - 1);
                xmlWriter.Close();
            }
        }

        private OMError getFilteredRows(string FieldListXML, DataTable SystemTable, ref ListDictionary FieldList, 
             ref DataRow[] rows)
        {
            XmlDocument     FieldXMLDocument    = null;
            XmlNode         FieldCollectionNode = null;
            StringBuilder   SortClauseBuilder   = null;
            XmlNodeList     FieldNodes          = null;
            XmlNodeList     SortFieldNodes      = null;
            XmlNode         FieldNameNode       = null;
            XmlNode         FieldSortNode       = null;
            XmlNode         SortTypeNode        = null;
            String          SortClause          = null;
            try
            {
                if (FieldListXML != null)
                {
                    SortClauseBuilder = new StringBuilder();
                    FieldList = new ListDictionary();
                    FieldXMLDocument = new XmlDocument();
                    FieldXMLDocument.LoadXml(FieldListXML);

                    FieldCollectionNode = FieldXMLDocument.SelectSingleNode("FIELDCOLLECTION");
                    FieldNodes = FieldCollectionNode.SelectNodes("FIELD");

                    foreach (XmlElement FieldNode in FieldNodes)
                    {
                        FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                        FieldList.Add(FieldNameNode.InnerText.ToUpper(), FieldNameNode.InnerText.ToUpper());
                    }

                    SortFieldNodes = FieldCollectionNode.SelectNodes("FIELD/SORTORDER");
                    if (SortFieldNodes.Count > 0)
                    {
                        for (int NodeCount = 0; NodeCount < SortFieldNodes.Count; NodeCount++)
                        {
                            FieldSortNode = FieldCollectionNode.SelectSingleNode("FIELD[SORTORDER='" + NodeCount + "']");
                            if (FieldSortNode != null)
                            {
                                FieldNameNode = FieldSortNode.SelectSingleNode("FIELDNAME");
                                SortTypeNode = FieldSortNode.SelectSingleNode("SORTTYPE");
                                if ((SortTypeNode.InnerXml).ToUpper() == "ASCENDING")
                                {
                                    SortClauseBuilder.AppendFormat("{1} ASC,", ChangeFieldNameIncaseItIsReserved(FieldNameNode.InnerXml.ToUpper()));
                                }
                                else
                                {
                                    SortClauseBuilder.AppendFormat("{1} DESC,", ChangeFieldNameIncaseItIsReserved(FieldNameNode.InnerXml.ToUpper()));
                                }

                            }
                        }
                        SortClause = SortClauseBuilder.ToString();
                        SortClause = SortClause.Substring(0, SortClause.Length - 1);
                        //Provide the filter to select the rows.
                        rows = SystemTable.Select(m_ClauseForCache, SortClause);
                        m_ClauseForCache = null;
                    }
                    else
                    {
                        if (m_SortClause != null)
                        {
                            SortClause = m_SortClauseForCache;
                            if (SortClause.IndexOf(".") >= 0)
                                SortClause = SortClause.Substring(SortClause.LastIndexOf(".") + 1, SortClause.Length - SortClause.LastIndexOf(".") - 1);
                            rows = SystemTable.Select(m_ClauseForCache, SortClause);
                            m_ClauseForCache        = null;
                            m_SortClauseForCache    = null;
                        }
                        else
                        {
                            //Provide the filter to select the rows.
                            rows = SystemTable.Select(m_ClauseForCache);
                            m_ClauseForCache = null;
                        }
                    }
                }
                else
                {
                    if (!String.IsNullOrEmpty(m_SortClause))
                    {
                        SortClause = m_SortClauseForCache;
                        if (SortClause.IndexOf(".") >= 0)
                            SortClause = SortClause.Substring(SortClause.IndexOf(".") + 1, SortClause.Length - SortClause.IndexOf(".") - 1);

                        rows  = SystemTable.Select(m_ClauseForCache, SortClause);
                        
                        m_ClauseForCache        = null;
                        m_SortClauseForCache    = null;
                    }
                    else
                    {
                        //Provide the filter to select the rows.
                        rows = SystemTable.Select(m_ClauseForCache);
                        m_ClauseForCache = null;
                    }
                }
                return null;
            }
            catch
            {
                return new OMError();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="FieldType"></param>
        /// <param name="FieldName"></param>
        /// <param name="FieldValue"></param>
        /// <param name="xml"></param>
        private void addFieldToXML(XmlTextWriter xmlWriter,string FieldType, string FieldName, object FieldValue)
        {
            DateTime DateValue;
           
            
            xmlWriter.WriteStartElement(FieldName.ToUpper());
            if (FieldValue != null)
            {
                switch (FieldType)
                {
                    case "System.DateTime":
                        DateValue = Convert.ToDateTime(FieldValue);
                       
                        xmlWriter.WriteStartElement("DAY");
                        xmlWriter.WriteString(DateValue.Day.ToString());
                        xmlWriter.WriteEndElement();

                        xmlWriter.WriteStartElement("MONTH");
                        xmlWriter.WriteString(DateValue.Month.ToString());
                        xmlWriter.WriteEndElement();

                        xmlWriter.WriteStartElement("YEAR");
                        xmlWriter.WriteString(DateValue.Year.ToString());
                        xmlWriter.WriteEndElement();

                        if (DateValue.Hour != 0)
                        {
                            xmlWriter.WriteStartElement("HOUR");
                            xmlWriter.WriteString(DateValue.Hour.ToString());
                            xmlWriter.WriteEndElement();

                            xmlWriter.WriteStartElement("MINUTE");
                            xmlWriter.WriteString(DateValue.Minute.ToString());
                            xmlWriter.WriteEndElement();

                            xmlWriter.WriteStartElement("SECOND");
                            xmlWriter.WriteString(DateValue.Second.ToString());
                            xmlWriter.WriteEndElement();
                        }
                        break;
                    default:
                        xmlWriter.WriteString(FieldValue.ToString());
                        break;
                }
            }
            
            xmlWriter.WriteEndElement();
           
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        virtual public bool RemoveFromCache(string name)
        {
            String cacheItemName = null;
            bool   cacheEnabled  = false;

            cacheEnabled    = true;//Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
            cacheItemName   = TenantName.ToUpper() + "_" + name.ToUpper();
            if (cacheEnabled  == true && CacheManager.IsCached(cacheItemName) == true)
            {
                //return CacheManager.RemoveCacheItem(cacheItemName);
                if (cacheItemName.Trim().Contains("ALOM_SCHEMA"))
                    OMDataManager.CacheItemsToRemove.Add(cacheItemName);
                else if(!OMDataManager.CacheItemsToRemove.Contains(cacheItemName))
                    return CacheManager.RemoveCacheItem(cacheItemName);
            }
            return false;
        }
        #endregion

        #region Methods for Preparing Dynamic SQL which is database dependent.
        /// <summary>
        /// This is a virtual Function which is to be derived in respective Database QueryProcessor.
        /// This function is responsible for generating a Stored Procedure which is completely database dependent.
        /// </summary>
        /// <returns></returns>
        public virtual OMError PrepareDynamicSQL(ref NSearchCommon NSearchCommonObj, ref StringBuilder DynamicSQL,
                                            ref List<String> ExecBeforeDynamicQueries, ref List<String> CleanupAfterDynamicQueries)
        {
            return null;
        }

        /// <summary>
        /// This is a virtual Function which is to be derived in respective Database QueryProcessor.
        /// This function is responsible for generating a Stored Procedure which is completely database dependent.
        /// </summary>
        /// <returns></returns>
        public virtual OMError PrepareDynamicSQL_Scalar(ref NSearchCommon NSearchCommonObj, ref StringBuilder DynamicSQL)
        {
            return null;
        }

        public virtual OMError ExecuteDynamicSQL(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            return null;
        }

        public virtual OMError ExecuteDynamicSQLEx(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            return null;
        }

        /// <summary>
        /// Executes the list of dynamic SQL's on the same connection.
        /// </summary>
        /// <param name="DynamicSQLs"></param>
        /// <param name="ConnectionString"></param>
        /// <param name="NSearchCommonObj"></param>
        /// <param name="SPOutputXMLs"></param>
        /// <returns>OMError Object, null incase no error is occured</returns>
        public virtual OMError ExecuteDynamicSQLs(List<String> DynamicSQLs, Dictionary<String,String> ConnectionStrings, NSearchCommon NSearchCommonObj, ref List<String> SPOutputXMLs)
        {
            OMError ErrorObj = null;
            String SPOutputXML = null;
            foreach (String DynamicSQL in DynamicSQLs)
            {
                if(NSearchCommonObj.m_Paging)
                    ErrorObj = ExecutePagingDynamicSQL(DynamicSQL, ConnectionStrings["ConnectionString"], NSearchCommonObj, ref SPOutputXML);
                else
                    ErrorObj = ExecuteDynamicSQL(DynamicSQL, ConnectionStrings["ConnectionStringForRead"], NSearchCommonObj, ref SPOutputXML);
               
                if (ErrorObj != null)
                    return ErrorObj;
                
                SPOutputXMLs.Add(SPOutputXML);
                SPOutputXML = null;
            }
            
            return null;

        }

        public virtual OMError ExecuteDynamicSQLsEx(List<String> DynamicSQLs, Dictionary<String, String> ConnectionStrings, NSearchCommon NSearchCommonObj, ref List<String> SPOutputXMLs)
        {
            OMError ErrorObj = null;
            String SPOutputXML = null;
            foreach (String DynamicSQL in DynamicSQLs)
            {
                if (NSearchCommonObj.m_Paging)
                    ErrorObj = ExecutePagingDynamicSQLEx(DynamicSQL, ConnectionStrings["ConnectionString"], NSearchCommonObj, ref SPOutputXML);
                else
                    ErrorObj = ExecuteDynamicSQLEx(DynamicSQL, ConnectionStrings["ConnectionStringForRead"], NSearchCommonObj, ref SPOutputXML);

                if (ErrorObj != null)
                    return ErrorObj;

                SPOutputXMLs.Add(SPOutputXML);
                SPOutputXML = null;
            }

            return null;

        }

        public virtual OMError ExecuteDynamicSQL_Scalar(String DynamicSQL, String ConnectionStrings, ref DataSet DS_Result)
        {
            return null;
        }

        public virtual OMError ExecutePagingDynamicSQL(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            return null;
        }

        public virtual OMError ExecutePagingDynamicSQLEx(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            return null;
        }

        /// <summary>
        /// Public method which prepares the query for a Search object for simple search and 
        /// returns the Target object list after executing that query.
        /// Query is prepared either from the Dummy query string by simply replacing the dummy field 
        /// name with actual search field name in the Search object or from the Post fix string 
        /// in case the field is a named list field.
        /// </summary>
        /// <param name="ObjectTypeNode">Root node of the XML containing Object schema information</param>
        /// <param name="TargetObjects">List of Target objects to be returned back</param>
        /// <returns>OMError Object, null incase no error is occured</returns>
        public virtual OMError SearchTargetObjectForQueryString(XmlNode ObjectTypeNode, String DummyQuery, String DummyField,ref string TEQuery)
        {
            return null;
        }

        /// <summary>
        /// Public method which prepares the query for a Search object for document search and 
        /// returns the Target object list after executing that query.        
        /// </summary>
        /// <param name="ObjectTypeNode">Root node of the XML containing Object schema information</param>
        /// <param name="SearchString">String to be searched in files</param>
        /// <param name="RelationshipName">All relationships to be searched with separator</param>
        /// <param name="TEQuery">Final query string</param>
        /// <returns>OMError Object, null incase no error is occured</returns>
        public virtual OMError SearchTargetObjectForQueryStringInFile(XmlNode ObjectTypeNode, String SearchString, String RelationshipName, ref string TEQuery)
        {
            return null;
        }

        /// <summary>
        /// Public method which has to be derived in derived class. This method gets all the related facets and prepares a Simple Search Query 
        /// </summary>
        /// <param name="ObjectTypeNode">Facet Type Node</param>
        /// <param name="TargetObjectType">Object Type</param>
        /// <param name="TargetObjectTypeID">Object Type ID</param>
        /// <param name="RFQuery">returns a simple search query </param>
        /// <returns></returns>
        public virtual OMError SearchSubFacetsForQueryString(XmlNode ObjectTypeNode, string TargetObjectType, ulong TargetObjectTypeID, String DummyQuery, String DummyField,ref string RFQuery)
        {
            return null;
        }

        /// <summary>
        /// Overloaded method with additional paprameter, the search string is passed to this method.
        /// </summary>
        /// <param name="ObjectTypeNode">Entity/Facet Type Node</param>
        /// <param name="TargetObjectType">Object Type i.e. Entity/Facet</param>
        /// <param name="TargetObjectTypeID">Object Type ID i.e. EntityTypeID/FacetTypeID</param>
        /// <param name="DummyQuery"></param>
        /// <param name="DummyField"></param>
        /// <param name="simpleSearchQuery">The search string.</param>
        /// <param name="RFQuery"></param>
        /// <returns></returns>
        public virtual OMError SearchSubFacetsForQueryString(XmlNode ObjectTypeNode, string TargetObjectType, 
                                                            ulong TargetObjectTypeID, String DummyQuery, 
                                                String DummyField, String simpleSearchQuery, ref string RFQuery)
        {
            return null;
        }

        /// <summary>
        ///  This method gets all the related facets and prepares a Document Search Query 
        /// </summary>
        /// <param name="ObjectTypeNode">Facet Type Node</param>
        /// <param name="TargetObjectType">Object Type</param>
        /// <param name="simpleSearchQuery">String to be searched in documents</param>
        /// <param name="RFQuery">returns a simple search query </param>
        /// <returns></returns>
        public virtual OMError SearchSubFacetsForQueryStringInFile(XmlNode ObjectTypeNode, ulong TargetObjectTypeID, 
                                                                                    String simpleSearchQuery, ref string RFQuery)
        {
            return null;
        }
        /// <summary>
        /// NSearchCommonObj function prepares a filter clause based on Date Macro.
        /// </summary>
        /// <param name="FieldName">FieldName on which filter is specified</param>
        /// <param name="InputFieldValue">Contains Date macros value. e.g : THIS_WEEK,THIS_YEAR</param>
        /// <param name="InputFilter">XML Node which has all information for Filter Node</param>
        /// <returns></returns>
        public virtual StringBuilder PrepareDateMacroFilter(String FieldName, String InputFieldValue, String RelationshipName)
        {
            return null;
        }

        /// <summary>
        /// This is a generic method which returns Entity/Facet type information. Consumer of this method is only Search component of OM.
        /// </summary>
        /// <param name="ObjectType">ENTITY/FACET</param>
        /// <param name="TypeID">Entity Type ID or FACET Type ID</param>
        /// <param name="ObjectTypeNode">returns an XMLNode which point to root node of TypeXML</param>
        /// <returns>Error object if there is an error else NULL</returns>
        public virtual OMError ReadTypeDetails(String ObjectType, ulong TypeID, ref XmlNode ObjectTypeNode)
        {
            OMError             ReadErrObj              = null;
            String              ObjectTypeXML           = null;
            XPathDocument       ObjectTypeXPathDoc      = null;
            XPathNavigator      ObjectTypeNavigator     = null;
            XmlNode             FieldCollNode           = null;
            XmlDocument         TypeXMLDocument         = null;
            XPathDocument       FieldCollXPDoc          = null;
            XPathNavigator      FieldCollectionNav      = null;
            String              FieldCollectionXML      = null;
            String              TypeName                = null;
            String              TableName               = null;

            try
            {
                switch (ObjectType.ToUpper())
                {
                    case "ENTITY":
                        if ((ReadErrObj = AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, TypeID.ToString(), OMConjunction.NOTHING)) != null)
                            return ReadErrObj;

                        TypeName = "ENTITYTYPE";
                        TableName = "ALOM_ENTITYTYPE";
                        
                        break;

                    case "FACET":
                        if ((ReadErrObj = AddClause("FACETTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, TypeID.ToString(), OMConjunction.NOTHING)) != null)
                            return ReadErrObj;

                        TypeName = "FACETTYPE";
                        TableName = "ALOM_FACETTYPE";
                        
                        break;
                }

                try
                {
                    if ((ReadErrObj = ReadRecord(TypeName, TableName, null, ref ObjectTypeXML)) != null)
                        return ReadErrObj;
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }

                if (String.IsNullOrEmpty(ObjectTypeXML))
                {
                    ReadErrObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND, "EntityType Not Found", OMErrorObjectType.ENTITYTYPE, "EntityType", OMErrorType.ERROR, null, null);
                    return ReadErrObj;
                }

                ObjectTypeXPathDoc = new XPathDocument(new StringReader(ObjectTypeXML));
                ObjectTypeNavigator = ObjectTypeXPathDoc.CreateNavigator();

                if (ObjectType.ToUpper() == "ENTITY")
                    ObjectTypeNavigator = ObjectTypeNavigator.SelectSingleNode("ENTITYTYPE");
                else
                    ObjectTypeNavigator = ObjectTypeNavigator.SelectSingleNode("FACETTYPE");

                TypeXMLDocument = new XmlDocument();
                GenerateTypeXML(ObjectTypeNavigator, ObjectType, ref TypeXMLDocument);

                //Get the fields of EntityType from ALOM_SCHEMA Table...
                if ((ReadErrObj = AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, TypeID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ReadErrObj = AddClause("OBJECTTYPE", OMFieldDataType.STRING, OMOperator.EQUALTO, ObjectType.ToUpper(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ReadErrObj = AddSortClause("DISPLAYORDER", "ALOM_SCHEMA", "ASC", null)) == null)
                        {
                            if ((ReadErrObj = ReadAllRecord("FIELD", "ALOM_SCHEMA", null, ref FieldCollectionXML)) == null)
                            {
                                //Prepare the OutputXML.
                                if (!String.IsNullOrEmpty(FieldCollectionXML))
                                {
                                    FieldCollXPDoc = new XPathDocument(new StringReader(FieldCollectionXML));
                                    FieldCollectionNav = FieldCollXPDoc.CreateNavigator();
                                    FieldCollectionNav = FieldCollectionNav.SelectSingleNode("FIELDCOLLECTION"); 
                                    FieldCollNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "FIELDCOLLECTION", "");
                                    FieldCollNode.InnerXml  = FieldCollectionNav.InnerXml;
                                    ObjectTypeNode = TypeXMLDocument.DocumentElement;
                                    ObjectTypeNode.AppendChild(FieldCollNode);
                                }
                            }
                        }
                    }
                }

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReadErrObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED, "Field Read Failed", OMErrorObjectType.ENTITYTYPE, "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);
                    return ReadErrObj;
                }

                return ReadErrObj; 
            }
            catch(Exception Ex)
            {
                return new OMError(Ex.Message);
            }

        }

        public virtual OMError ReadTypeDetails(String ObjectType, String ObjectTypeName, ref XmlNode ObjectTypeNode)
        {
            OMError ReadErrObj = null;
            String ObjectTypeXML = null;
            XPathDocument ObjectTypeXPathDoc = null;
            XPathNavigator ObjectTypeNavigator = null;
            XmlNode FieldCollNode = null;
            XmlDocument TypeXMLDocument = null;
            XPathDocument FieldCollXPDoc = null;
            XPathNavigator FieldCollectionNav = null;
            String FieldCollectionXML = null;
            String TypeName = null;
            String TableName = null;

            try
            {
                switch (ObjectType.ToUpper())
                {
                    case "ENTITY":
                        if ((ReadErrObj = AddClause("ENTITYTYPENAME", OMFieldDataType.STRING, OMOperator.EQUALTO, ObjectTypeName, OMConjunction.NOTHING)) != null)
                            return ReadErrObj;

                        TypeName = "ENTITYTYPE";
                        TableName = "ALOM_ENTITYTYPE";

                        break;

                    case "FACET":
                        if ((ReadErrObj = AddClause("FACETTYPENAME", OMFieldDataType.STRING, OMOperator.EQUALTO, ObjectTypeName, OMConjunction.NOTHING)) != null)
                            return ReadErrObj;

                        TypeName = "FACETTYPE";
                        TableName = "ALOM_FACETTYPE";

                        break;
                }

                try
                {
                    if ((ReadErrObj = ReadRecord(TypeName, TableName, null, ref ObjectTypeXML)) != null)
                        return ReadErrObj;
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }

                if (String.IsNullOrEmpty(ObjectTypeXML))
                {
                    if (string.Compare(ObjectType, "ENTITY", true) == 0)
                    {
                        ReadErrObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND, "EntityType Not Found", OMErrorObjectType.ENTITYTYPE, "EntityType", OMErrorType.ERROR, null, null);
                        return ReadErrObj;
                    }
                    else
                    {
                        ReadErrObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND, "FacetType Not Found", OMErrorObjectType.FACETTYPE, "FacetType", OMErrorType.ERROR, null, null);
                        return ReadErrObj;
                    }
                }

                string TypeID = "0";
                XmlDocument InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(ObjectTypeXML);

                ObjectTypeNode = InputXMLDocument.DocumentElement;
                XmlNode EntityTypeIDNode = ObjectTypeNode.SelectSingleNode("ENTITYTYPEID");
                if (EntityTypeIDNode != null)
                    TypeID = EntityTypeIDNode.InnerText.ToString();

                ObjectTypeXPathDoc = new XPathDocument(new StringReader(ObjectTypeXML));
                ObjectTypeNavigator = ObjectTypeXPathDoc.CreateNavigator();

                if (ObjectType.ToUpper() == "ENTITY")
                    ObjectTypeNavigator = ObjectTypeNavigator.SelectSingleNode("ENTITYTYPE");
                else
                    ObjectTypeNavigator = ObjectTypeNavigator.SelectSingleNode("FACETTYPE");

                TypeXMLDocument = new XmlDocument();
                GenerateTypeXML(ObjectTypeNavigator, ObjectType, ref TypeXMLDocument);

                //Get the fields of EntityType from ALOM_SCHEMA Table...
                if ((ReadErrObj = AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, TypeID, OMConjunction.AND)) == null)
                {
                    if ((ReadErrObj = AddClause("OBJECTTYPE", OMFieldDataType.STRING, OMOperator.EQUALTO, ObjectType.ToUpper(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ReadErrObj = AddSortClause("DISPLAYORDER", "ALOM_SCHEMA", "ASC", null)) == null)
                        {
                            if ((ReadErrObj = ReadAllRecord("FIELD", "ALOM_SCHEMA", null, ref FieldCollectionXML)) == null)
                            {
                                //Prepare the OutputXML.
                                if (!String.IsNullOrEmpty(FieldCollectionXML))
                                {
                                    FieldCollXPDoc = new XPathDocument(new StringReader(FieldCollectionXML));
                                    FieldCollectionNav = FieldCollXPDoc.CreateNavigator();
                                    FieldCollectionNav = FieldCollectionNav.SelectSingleNode("FIELDCOLLECTION");
                                    FieldCollNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "FIELDCOLLECTION", "");
                                    FieldCollNode.InnerXml = FieldCollectionNav.InnerXml;
                                    ObjectTypeNode = TypeXMLDocument.DocumentElement;
                                    ObjectTypeNode.AppendChild(FieldCollNode);
                                }
                            }
                        }
                    }
                }

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReadErrObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED, "Field Read Failed", OMErrorObjectType.ENTITYTYPE, "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);
                    return ReadErrObj;
                }

                return ReadErrObj;
            }
            catch (Exception Ex)
            {
                return new OMError(Ex.Message);
            }

        }

        /// <summary>
        /// this is a helper function which prepares Type XML  
        /// </summary>
        /// <param name="ObjectTypeNavigator">XPath navigator which contains informatiom from ALOM_ENTITYTYPE/ALOM_FACETTYPE table</param>
        /// <param name="ObjectType">ENTITY/FACET</param>
        /// <param name="TypeXMLDocument">TypeXMLDocument which has ENTITY/FACET information in formatted way</param>
        /// <returns>Error object if there is an error else NULL</returns>
        private OMError GenerateTypeXML(XPathNavigator ObjectTypeNavigator, String ObjectType,ref XmlDocument TypeXMLDocument)
        {
            XPathNavigator      TempXPNav                   = null;
            String              Description                 = null;     //String variable storing the value of Description.
            XmlNode             TypeIDNode                  = null;     //DOM Node representing the TypeID node.
            XmlNode             NameNode                    = null;     //DOM Node representing the Name node.
            XmlNode             DescriptionNode             = null;     //DOM Node representing the Description node.
            XmlNode             TypeNode                    = null;     //DOM Node representing the Type node.
            XmlNode             ObjectTypeNode              = null;     //DOM Node representing the ObjectType node.
            XmlNode             TableNameNode               = null;
            XmlNode             PrimaryKeyNode              = null;
            XmlNode             IsBuiltInNode               = null;
            String              TypeID                      = null;
            String              TypeName                    = null;

            try
            {
                switch (ObjectType)
                {
                    case "ENTITY":
                        TempXPNav = ObjectTypeNavigator.SelectSingleNode("ENTITYTYPEID");
                        TypeID = TempXPNav.Value;

                        TempXPNav = ObjectTypeNavigator.SelectSingleNode("ENTITYTYPENAME");
                        TypeName = TempXPNav.Value;

                        TempXPNav = ObjectTypeNavigator.SelectSingleNode("TYPEDESCRIPTION");
                        if (TempXPNav == null)
                            Description = "";
                        else
                            Description = TempXPNav.Value;
                        break;
                    case "FACET":
                        TempXPNav = ObjectTypeNavigator.SelectSingleNode("FACETTYPEID");
                        TypeID = TempXPNav.Value;

                        TempXPNav = ObjectTypeNavigator.SelectSingleNode("FACETTYPENAME");
                        TypeName = TempXPNav.Value;

                        TempXPNav = ObjectTypeNavigator.SelectSingleNode("TYPEDESCRIPTION");
                        if (TempXPNav == null)
                            Description = "";
                        else
                            Description = TempXPNav.Value;
                        break;
                }

                // Prepare the Type XML.
                TypeXMLDocument = new XmlDocument();

                TypeNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPE", "");
                TypeXMLDocument.AppendChild(TypeNode);

                TypeIDNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPEID", "");
                TypeIDNode.InnerText = TypeID;
                TypeNode.AppendChild(TypeIDNode);

                NameNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPENAME", "");
                NameNode.InnerText = TypeName;
                TypeNode.AppendChild(NameNode);

                DescriptionNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPEDESCRIPTION", "");
                DescriptionNode.InnerText = Description;
                TypeNode.AppendChild(DescriptionNode);

                ObjectTypeNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "OBJECTTYPE", "");
                if (ObjectType.ToUpper() == "ENTITY")
                    ObjectTypeNode.InnerText = "ENTITY";
                else
                    ObjectTypeNode.InnerText = "FACET";
                TypeNode.AppendChild(ObjectTypeNode);

                TempXPNav = ObjectTypeNavigator.SelectSingleNode("TABLENAME");
                TableNameNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                TableNameNode.InnerText = TempXPNav.Value;
                TypeNode.AppendChild(TableNameNode);

                TempXPNav = ObjectTypeNavigator.SelectSingleNode("PRIMARYKEY");
                PrimaryKeyNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "PRIMARYKEY", "");
                PrimaryKeyNode.InnerText = TempXPNav.Value;
                TypeNode.AppendChild(PrimaryKeyNode);

                TempXPNav = ObjectTypeNavigator.SelectSingleNode("ISBUILTIN");
                IsBuiltInNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "ISBUILTIN", "");
                IsBuiltInNode.InnerText = TempXPNav.Value;
                TypeNode.AppendChild(IsBuiltInNode);

                return null;
            }
            catch(Exception Ex)
            {
                return new OMError(Ex.Message);
            }
        }
        
        protected OMError PrepareWhereClauseFromFilterList(String TableName,String FilterListXML,String ObjectTypeXML,ref String WhereClause)
        {
            OMError             OMErrorObj          = null;
            XPathDocument       FilterListXMLDoc    = null;
            XPathNavigator      FilterListNode      = null;
            XPathNodeIterator   FilterNodes         = null; 
            FilterCollection    FilterCollectionObj = null;
            StringBuilder       SBWhereClause       = null;
            String              Conjunction         = null;
            StringBuilder       FilterClause        = null;
            XmlDocument         ObjectTypeXMLDoc    = null;  
            XmlNode             ObjectTypeNode      = null;
            XPathNavigator      IsCaseSensitiveNode = null;
            NameValueCollection Field_FilterColl    = null;
            String[]            Filters             = null;
            XPathNavigator      ExprValueNode       = null;
            XPathNavigator      ValueNode           = null;
            XmlDocument         FilterXMLDoc        = null;

            try
            {
                ObjectTypeXMLDoc                = new XmlDocument();
                FilterCollectionObj             = new FilterCollection();
                FilterXMLDoc                    = new XmlDocument();
                SBWhereClause                   = new StringBuilder();
                FilterClause                    = new StringBuilder();
                Field_FilterColl                = new NameValueCollection();
                
                ObjectTypeXMLDoc.LoadXml(ObjectTypeXML);
                ObjectTypeNode                  = ObjectTypeXMLDoc.DocumentElement;
                FilterListXMLDoc                = new XPathDocument(new StringReader(FilterListXML));
                FilterListNode                  = FilterListXMLDoc.CreateNavigator();
                FilterNodes                     = FilterListNode.Select("//FILTER");
                FilterCollectionObj.m_XMLNode   = FilterXMLDoc.CreateElement("FILTER");

                if(FilterNodes.Count > 0)
                {
                    SBWhereClause.Append("WHERE ");
                    while (FilterNodes.MoveNext())
                    {
                        FilterCollectionObj.m_FieldName         = FilterNodes.Current.SelectSingleNode("EXPRESSION").Value;
                        
                        // Special processing for DateNode.
                        ExprValueNode                           = FilterNodes.Current.SelectSingleNode("EXPRESSIONVALUE");
                        // Special Processing for Date Filter
                        ValueNode                               = ExprValueNode.SelectSingleNode("VALUE");
                        if (ValueNode != null)
                        {
                            ExprValueNode.MoveToParent();
                            FilterCollectionObj.m_XMLNode.InnerXml = ExprValueNode.InnerXml;
                        }
                        else
                        {
                            FilterCollectionObj.m_FieldValue    = ExprValueNode.Value;
                        }

                        FilterCollectionObj.m_Operator          = FilterNodes.Current.SelectSingleNode("OPERATOR").Value;
                        if ((IsCaseSensitiveNode = FilterNodes.Current.SelectSingleNode("ISCASESENSITIVE")) != null)
                        {
                            if (IsCaseSensitiveNode.Value.ToUpper() == "TRUE")
                                FilterCollectionObj.m_IsCaseSensitive = true;
                            else
                                FilterCollectionObj.m_IsCaseSensitive = false;
                        }
                        else
                        {
                            FilterCollectionObj.m_IsCaseSensitive = false;
                        }

                        FilterClause.Remove(0, FilterClause.Length);
                        if ((OMErrorObj = GetFilterString(TableName, FilterCollectionObj, ref FilterClause, ref ObjectTypeNode, false)) == null)
                        {
                            
                            for (int i = 0; i < FilterCollectionObj.m_ParameterizedVariables.Count; i++)
                            {
                                DynamicSQLParameter m_Parameter = new DynamicSQLParameter(FilterCollectionObj.m_ParameterizedVariables[i], FilterCollectionObj.m_FieldValueParameterized[i].Replace("''","'"), FilterCollectionObj.m_FieldDataType);
                                if (m_DeferQueryExecution)
                                {
                                    DynamicParameters_Defer.Add(m_Parameter);
                                }
                                {
                                    DynamicParameters.Add(m_Parameter);
                                }

                            }
                            Field_FilterColl.Add(FilterCollectionObj.m_FieldName, FilterClause.ToString());
                        }
                        else
                        {
                            return OMErrorObj;
                        }
                    }

                    for (int Index = 0; Index < Field_FilterColl.Count; Index++)
                    {
                        Filters         =  Field_FilterColl.GetValues(Index);
                        for (int Count = 0; Count < Filters.Length; Count++)
                        {
                            SBWhereClause.AppendFormat("{0}{1}",Conjunction, Filters[Count]);
                            Conjunction = " OR ";
                        }
                        Conjunction     = " AND ";
                    }
                }
                WhereClause = SBWhereClause.ToString();
            }
            catch (Exception ExceptionObj)
            {
                OMErrorObj = new OMError(OMErrorCodes.EXCEPTION_OCCURED,
                          ExceptionObj.Message, OMErrorObjectType.DATAACCESSLAYER,
                          null, OMErrorType.EXCEPTION, ExceptionObj, null);
                return OMErrorObj;
            }
            return null;
        }

               
        /// <summary>
        /// This method returns m_clause for deferredQueryExecution.
        /// </summary>
        /// <returns></returns>
        public virtual String GetClauseForDataTables()
        {
            return (m_ClauseForDataTables);
        }

        public string GetNextParameterizedVariable()
        {
            string strReturnString = "@Var" + int_ParameterizedVariableId.ToString();
            int_ParameterizedVariableId++;
            return strReturnString;
        }


        #endregion

        public void WriteLog(String TenantName, String SolutionName, String PageName, String SessionID, String ContentToWrite)
        {
            lock (this)
            {
                //if (ReadRegisterykey())
                //{
                PageName = "ExpressionText";
                String InstallationDir = ALCommon.GetInstallationDir();
                int LastIndex = PageName.LastIndexOf('/');
                int ExtIndex = PageName.LastIndexOf('.');

                if (LastIndex > 0 && ExtIndex > 0)
                {
                    PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                    PageName = PageName.Replace(' ', '_');
                }
                String DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                String LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                FileStream LogFileStream = null;
                StreamWriter LogStreamWriter = null;

                try
                {
                    DirectoryInfo Root = new DirectoryInfo(DirName);
                    if (Root.Exists == false)
                        Root.Create();

                    if (File.Exists(LogFileName))
                        LogFileStream = new FileStream(LogFileName, FileMode.Append);
                    else
                        LogFileStream = new FileStream(LogFileName, FileMode.Create);

                    LogStreamWriter = new StreamWriter(LogFileStream);
                    StringBuilder TimeStamp = new StringBuilder();

                    //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                    //LogStreamWriter.WriteLine("=====================================================================");
                    //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                    LogStreamWriter.WriteLine(ContentToWrite);
                    LogStreamWriter.WriteLine();
                }
                catch (Exception ex)
                {
                    PageName = "WriteLogError";
                    InstallationDir = ALCommon.GetInstallationDir();
                    LastIndex = PageName.LastIndexOf('/');
                    ExtIndex = PageName.LastIndexOf('.');

                    if (LastIndex > 0 && ExtIndex > 0)
                    {
                        PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                        PageName = PageName.Replace(' ', '_');
                    }
                    DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                    LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                    FileStream LogFileStreamEx = null;
                    StreamWriter LogStreamWriterEx = null;

                    try
                    {
                        DirectoryInfo Root = new DirectoryInfo(DirName);
                        if (Root.Exists == false)
                            Root.Create();

                        if (File.Exists(LogFileName))
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Append);
                        else
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Create);
                        string strMessageToWrite = SessionID + "," + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + "," + ex.Message;
                        LogStreamWriterEx = new StreamWriter(LogFileStreamEx);
                        StringBuilder TimeStamp = new StringBuilder();

                        //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                        //LogStreamWriter.WriteLine("=====================================================================");
                        //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                        LogStreamWriterEx.WriteLine(strMessageToWrite);
                        LogStreamWriterEx.WriteLine();
                    }
                    catch
                    {

                    }
                    finally
                    {
                        if (LogStreamWriterEx != null)
                            LogStreamWriterEx.Close();

                        if (LogFileStreamEx != null)
                            LogFileStreamEx.Close();

                        LogStreamWriterEx = null;
                        LogFileStreamEx = null;
                    }
                }
                finally
                {
                    if (LogStreamWriter != null)
                        LogStreamWriter.Close();

                    if (LogFileStream != null)
                        LogFileStream.Close();

                    LogStreamWriter = null;
                    LogFileStream = null;
                }
                //}
            }
        }

        private IDataReader ReadFromCache(string query)
        {
            IDataReader reader = null;
            try
            {
                query = query.Replace("from", "FROM");
                query = query.Replace("where", "WHERE");
                query = query.Replace("order by", "ORDER BY");
                String[] delimiters = { "FROM", "WHERE", "ORDER BY" };
                String[] clauses = query.Split(delimiters, 4, StringSplitOptions.None);
                string tableName = clauses[1].Trim().ToUpper();

                string whereClause = clauses[2];
                string sortClause = null;
                if (clauses.Length > 3)
                    sortClause = clauses[3];
                DataTable resultTable = null;
                DataTable cachedTable = null;
                DataRow[] rows;
                string cachedItemName = "";
                //use cache only if from clause contains single table
                if (tableName.IndexOf(',') == -1 && tableName.Contains("JOIN") == false)
                {
                    cachedItemName = TenantName.ToUpper() + "_" + tableName;
                    cachedTable = CacheManager.GetCacheItem(cachedItemName) as DataTable;
                    if (cachedTable != null)
                    {
                        if (sortClause == null)
                        {
                            rows = cachedTable.Select(whereClause);
                        }
                        else
                        {
                            rows = cachedTable.Select(whereClause, sortClause);
                        }

                        //if number of rows returned is zero, return  null so that query can run to the table 
                        if (rows.Length != 0)
                        {
                            resultTable = rows.CopyToDataTable();//Create a copy with selected rows and return a dataReader
                            reader = resultTable.CreateDataReader();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                reader = null;  //set reader to null so that errors in this ,layer does not affect the caller.
            }
            return reader;
        } 

    }
}